REM Create database objects used by the Trace Global Catalog

create table EPC_COLLECTION
(
    COLLECTION_ID		NUMBER CONSTRAINT pk_coll_id PRIMARY KEY,
    COLLECTION_NAME		VARCHAR2(255),
    JOB_ID			NUMBER,
    NEXT_EXEC_TIME		VARCHAR2(45),
    DATABASE_TYPE		NUMBER,
    FORMATTER_DATABASE		VARCHAR2(255),
    FORMATTER_SCHEMA		VARCHAR2(30),
    DURATION			VARCHAR2(8),
    RESUBMIT_FLAG		NUMBER(1),
    HIDE_FLAG			NUMBER(1)
);
create public synonym EPC_COLLECTION for EPC_COLLECTION;

create table EPC_COLLECTION_SUMMARY
(
    COLLECTION_ID		NUMBER CONSTRAINT fk_sum_id REFERENCES 
				EPC_COLLECTION (COLLECTION_ID) ON DELETE CASCADE,
    NODE			VARCHAR2(80),
    STATUS			NUMBER,
    LAST_STATUS_UPDATE_TIME	VARCHAR(45),
    COLLECTION_FILE_SPEC	VARCHAR2(255)
);
create public synonym EPC_COLLECTION_SUMMARY for EPC_COLLECTION_SUMMARY;
create index i_coll_sum_id on epc_collection_summary (collection_id);

create table EPC_COLLECTION_HISTORY
(
    COLLECTION_ID		NUMBER CONSTRAINT fk_his_id REFERENCES 
				EPC_COLLECTION (COLLECTION_ID) ON DELETE CASCADE,
    NODE			VARCHAR2(80),
    STATUS			NUMBER,
    STATUS_UPDATE_TIME		DATE
);
create public synonym EPC_COLLECTION_HISTORY for EPC_COLLECTION_HISTORY;
create index i_coll_his_id on epc_collection_history (collection_id);

create table EPC_FACILITY
(
    FACILITY_ID			NUMBER CONSTRAINT pk_fac_id PRIMARY KEY,
    VENDOR			NUMBER,
    FACILITY_NUMBER             NUMBER(4),
    FACILITY_NAME               VARCHAR2(20),
CONSTRAINT unq_fac_id UNIQUE (VENDOR, FACILITY_NUMBER)
);

create public synonym EPC_FACILITY for EPC_FACILITY;

create table EPC_COLLECTION_FACILITIES
(
    COLLECTION_ID		NUMBER CONSTRAINT fk_fac_coll_id REFERENCES 
				EPC_COLLECTION (COLLECTION_ID) ON DELETE CASCADE,
    FACILITY_ID	                NUMBER CONSTRAINT fk_fac_id
    				REFERENCES EPC_FACILITY (FACILITY_ID)
);
				
create public synonym EPC_COLLECTION_FACILITIES for EPC_COLLECTION_FACILITIES;
create index i_coll_fac_id on epc_collection_facilities (collection_id);



create sequence EPC_COLLECTION_ID_ MINVALUE 1 MAXVALUE 99999999 CYCLE;
create public synonym EPC_COLLECTION_ID for EPC_COLLECTION_ID_;
create sequence EPC_FACILITY_ID MINVALUE 1 MAXVALUE 99999999 CYCLE;
create public synonym EPC_FACILITY_ID for EPC_FACILITY_ID_;

