/* =========================================
 *	Input/Output 
 * ========================================= */
#define Out32(addr, w) ( *(volatile uint_t *)(addr) = (w) )
#define Inp32(addr)     ( *(volatile uint_t *)(addr) )


/*************************************************************
 **	Locking Mechanism for Multiprocessor system         **
 *************************************************************
 **
 **	By default we use mutex locks. But spinlocks are    **
 **     supported also. Move  #if 0/#endif pairs to cover   **
 **     mutex defines if you want to use spinlocks.         **
 *************************************************************/
 
#if 0

/* =============================
 *     Sleep/Wakeup/Lock 
 *
 *     using spinlocks (lock_t)
 * ============================== */
#define   coco_lock_t                lock_t
#define   COCO_LOCK_INIT(lockp)      spinlock_init(lockp, "cocoml") 
#define   COCO_LOCK(lockp)           LOCK(lockp, plhi)
#define   COCO_UNLOCK(lockp, s)      UNLOCK(lockp, s)
#define   COCO_TRYLOCK(lockp)        TRYLOCK(lockp, plhi)  
#define   SLEEP_EVENT(svp, lockp, s) SV_WAIT(svp, lockp, s)

#endif


/* ================================
 *     Sleep/Wakeup/Lock
 *
 *     using mutex locks (mutex_t)
 * ================================= */
#define   coco_lock_t                mutex_t
#define   COCO_LOCK_INIT(lockp)      MUTEX_INIT(lockp, MUTEX_DEFAULT, "cocoml") 
#define   COCO_LOCK(lockp)           MUTEX_LOCK(lockp, PRIBIO|PCATCH)
#define   COCO_UNLOCK(lockp, s)      MUTEX_UNLOCK(lockp)
#define   COCO_TRYLOCK(lockp)        MUTEX_TRYLOCK(lockp) 
#define   SLEEP_EVENT(svp, lockp, s) SV_WAIT(svp, lockp, 0)


/*  used for both kinds of locks  */
#define   WAKE_EVENT(svp)            SV_SIGNAL(svp) 

/* ==========================
 *	Misc. defaults
 * ========================== */
#define  DEVICE_ID      0x0001	
#define  VENDOR_ID      0x11a4 
#define  DRIVER_PREFIX  "coco_"
#define  AMCC_RAM_SIZE        64 
#define  CONFIG_RAM_SIZE      16 
#define  NORMAL_DMA_RAM_SIZE  16 
#define  COCO_CONFIG_HDR      68
#define  END_OF_CHAIN         0x80000000
#define  RW_TIMER             500   /*  wait for read/write in clock ticks   */
#define  SIMRW_TIMER          500   /*  wait for sim R/W in  clock ticks     */
#define  CHAIN_FACTOR          10   /*  extra room for adjust chains         */
#define  MAP_PAGES              5   /*  number of pages ideal for mapping    */
#define  LESS_MAP             500   /*  bytes less report for bytes to map   */

/* =================================
 *   Configuration Register bits 
 * ================================= */

#define CONFIG_CCRES   0x00000001    /*  reset when zero                   */
#define CONFIG_FRES    0x00000002    /*  Input/Output Fifo reset when 0    */
#define CONFIG_FSDATI  0x00000004    /*  Inp. Fifo serial config data      */
#define CONFIG_FSDATO  0x00000008    /*  Out. Fifo serial config data      */
#define CONFIG_FSCLK   0x00000010    /*  In/Out Fifo serial config clock   */
#define CONFIG_LUTSEL  0x00000020    /*  External LUT banck selection (=0) */

/*  bits 6-9 is RAM address  */
#define CONFIG_DMA_READ_ADDR   0x00000000  /* DMA Read Address             */
#define CONFIG_DMA_WRITE_ADDR  0x00000040  /* DMA Write Address            */
#define CONFIG_SC_READ_ADDR    0x00000080  /* Scatter-Gather Read Address  */
#define CONFIG_SC_WRITE_ADDRR  0x00000000  /* Scatter-Gather Write Address */

/* ============================
 *     Mode Register
 * ============================ */
#define COCO_MODE  0x01
#define COCO_SWAP  0x02
#define COCO_SLICE 0x04
#define COCO_DELAY 0x08
#define COCO_FLAG  0x20
#define COCO_WRENA 0x40

/* ==========================================
 * 	Configuration Register	
 * ========================================== */

#define DMAREG_CCRES   0x00000001L  /* Chameleon reset when zero           */
#define DMAREG_HICRES  0x00000001L  /* Chameleon reset when zero           */
#define DMAREG_FRES    0x00000002L  /* I/O Fifo reset when zero            */
#define DMAREG_FSDATI  0x00000004L  /* Input Fifo serial config. data      */
#define DMAREG_FSDATO  0x00000008L  /* Output Fifo serial config. data     */
#define DMAREG_FSCLK   0x00000010L  /* I/O serial config clock             */
#define DMAREG_LED     0x00000020L  /* external LUT bank selection (=0)    */

/*  bit 6-9: Ram address   */
#define DMAREG_RAMRADR 0x00000000L  /* DMA read address                    */
#define DMAREG_RAMWADR 0x00000040L  /* DMA write address                   */
#define DMAREG_RAMPRRD 0x00000080L  /* scatter-gather read address         */
#define DMAREG_RAMPRWR 0x000000C0L  /* scatter-gather write address        */
#define DMAREG_RAMRCNT 0x00000100L  /* read count copy                     */
#define DMAREG_RAMWCNT 0x00000140L  /* write count copy                    */

#define DMAREG_FILL    0x00000400L  /* enable automatic ram fill for DMA   */
#define DMAREG_PTEN    0x00008000L  /* Fifo Interface Enable               */
#define DMAREG_INTREN  0x00010000L  /* DMA Read Interrupt Enable           */
#define DMAREG_INTWEN  0x00020000L  /* DMA Write Interrupt Enable          */
#define DMAREG_INTPREN 0x00040000L  /* data-chained DMA read int. enable   */
#define DMAREG_INTPWEN 0x00080000L  /* data-chained DMA write int. enable  */

/*  bit 20-21: Device Selection    */ 
#define DMAREG_RAM     0x00000000L   /* RAM   				   */
#define DMAREG_RCNT    0x00100000L   /* DMA Read Count   		   */
#define DMAREG_WCNT    0x00200000L   /* DMA Write Count  		   */
#define DMAREG_STAT    0x00300000L   /* Status/Fifo control Register       */

#define DMAREG_REN     0x10000000L   /* DMA Read Enable			   */
#define DMAREG_WEN     0x20000000L   /* DMA Write Enable		   */
#define DMAREG_PREN    0x40000000L   /* data-chained DMA read enable       */
#define DMAREG_PWEN    0x80000000L   /* data-chained DMA write enable      */

#define DMAREG_NVIFEN  0x00800000L   /* Mailbox Interface Enable	   */
#define DMAREG_DMACVT  0x00400000L   /* Unused                             */ 

/*  Unknown !!     */
#define DMAREG_HIDOIT  0x00000400L
#define DMAREG_HISCLK  0x00000800L
#define DMAREG_HISDI   0x00001000L
#define DMAREG_HISDO   0x00000080L
#define EOFPROG        0xF0000000L


/* ==============================
 *     AMCC Register Offsets
 * ============================== */
#define AMCC_OP_REG_OMB1         0x00
#define AMCC_OP_REG_OMB2         0x04
#define AMCC_OP_REG_OMB3         0x08
#define AMCC_OP_REG_OMB4         0x0c
#define AMCC_OP_REG_IMB1         0x11
#define AMCC_OP_REG_IMB2         0x14
#define AMCC_OP_REG_IMB3         0x18
#define AMCC_OP_REG_IMB4         0x1c
#define AMCC_OP_REG_FIFO         0x20
#define AMCC_OP_REG_MWAR         0x24
#define AMCC_OP_REG_MWTC         0x28
#define AMCC_OP_REG_MRAR         0x2c
#define AMCC_OP_REG_MRTC         0x30
#define AMCC_OP_REG_MBEF         0x34
#define AMCC_OP_REG_INTCSR       0x38
#define AMCC_OP_REG_MCSR         0x3c
#define AMCC_OP_REG_MCSR_NVDATA  (AMCC_OP_REG_MCSR + 2) /* Data in byte 2 */
#define AMCC_OP_REG_MCSR_NVCMD   (AMCC_OP_REG_MCSR + 3) /* Command in byte 3 */


/*   Amcc INTCSR interrupt bits  */
#define AMCC_INTCSR_WEN    0x00004000
#define AMCC_INTCSR_REN    0x00008000
#define AMCC_INTCSR_INTMB4 0x00001f00

/*  enable Output Mbox4, byte 3 only  */
#define AMCC_INTCSR_MASK   AMCC_INTCSR_INTMB4

#if 0   /*  Write/Read Completion Interrupt enable only  */
#define AMCC_INTCSR_MASK   AMCC_INTCSR_WEN|AMCC_INTCSR_REN

/**   "Write/Read Completion Interrupt" with Output Maibox4 */ 
#define AMCC_INTCSR_MASK   AMCC_INTCSR_WEN|AMCC_INTCSR_REN|AMCC_INTCSR_INTMB4
#endif

#define AMCC_INTCSR_RCLR   0x00080000  /*  Read Transfer Complete Clear  */
#define AMCC_INTCSR_WCLR   0x00040000  /*  Write Transfer Complete Clear */
#define AMCC_INTCSR_RST    0x00330000  /*  Target/Master Abort and Out Mbox */

/*   Amcc MCSR bits   */
#define AMCC_REN           0x00004000   /* Read Enable  */
#define AMCC_WEN           0x00000400   /* Write Enable */
#define AMCC_RFMAN 	   0x00002000L
#define AMCC_WFMAN 	   0x00000200L
#define AMCC_RFPRI 	   0x00001000L  /* Read Priority over Write */
#define AMCC_WFPRI 	   0x00000100L  /* Write Priority over Read */
#define AMCC_RST_FIFOS     0x06000000L  /* Reset Fifos  */
#define AMCC_RST_ADDON     0x01000000L  /* Reset Add-on */
#define AMCC_MCSR_MASK     AMCC_RST_FIFOS | AMCC_RST_ADDON


/* Outgoing Mailbox Register 4, byte 3   */
#define AMCC_MB_EOFDMAR    0x01000000	/* 1 = end of read DMA        */
#define AMCC_MB_EOFDMAW    0x02000000   /* 1 = end of write DMA       */
#define AMCC_MB_EOFPRDMAR  0x04000000   /* 1 = end of prog. read DMA  */
#define AMCC_MB_EOFPRDMAW  0x08000000	/* 1 = end of prog. write DMA */
#define AMCC_MB_DIAGN      0x10000000   /* Diagnostic flag	      */
#define AMCC_MB_COCOMODE   0x20000000   /* Chameleon flag	      */
#define AMCC_MB_FIFORSTR   0x40000000	/* AMCC input Fifo Reset      */
#define AMCC_MB_FIFORSTW   0x80000000   /* AMCC output Fifo Reset     */


/* ==========================================================
 *	32 bit version of structures defined in coco_user.h
 * ========================================================== */
typedef struct {
        int              buf_size;
        uint_t           buf;
} coco_buf32_t;

typedef struct {
        int     buf_size;      /*  buffer size in int32   */
        int     adjust_chain;  /*  1 = adjust read/write chain entries */
        uint_t  r_buf;   /*  read buffer            */
        uint_t  w_buf;   /*  write buffer           */
} coco_rw32_t;

/*  chained DMA block    */
typedef struct {
        uint_t   nextaddr;
        uint_t   addr;
        int      size;
} coco_dmapage_t;

/* ==============================
 *      Device Information
 * ==============================
 *
 * 	status:		Shows whether the driver is attached/opened, etc.
 *
 *      DMA Stuff
 *      ----------
 *	dmacfg: 	The boards Configuration default setting.
 *	dmabits:	DMA operation bit setting (in addition to dmacfg)
 *	dmatype:	Single or Chain DMA type
 *	dmastat:	curretnt status of DMA (Idle, Wait, etc.) 
 *	dmacmd:		Board's DMA command (Transp or Convert)
 *	dmawait:	Semaphore for wait/wakeup 
 *	wp_addr:	Residual phys. address for Write DMA.
 *	rp_addr: 	Residual phys. address for Read  DMA.	
 *	rp_size:	Residual size in bytes for Read  DMA.
 *	wp_size:	Residual size in bytes for Write DMA.
 *	bp:		Current buf_t pointer for Read/Write DMA
 *	chain_list:	Address of current chain list buffer
 *	addrList:	Current DMA scatter/gather structure (single read/write)
 *	r_addrList:	Sim. Read/Write Read scatter/gather structure.
 *	w_addrList:	Sim. Read/Write Write scatter/gather structure.
 *	page_no:	Pages left to DMA (single read/write)
 *	r_page_no:	Pages left for Read to DMA (Sim. read/write).	
 *	w_page_no:	Pages left for Write to DMA (Sim. read/write).	
 *
 *      Mapping Memory
 *      ---------------
 *      vhandl:         vhandl_t pointer used for mapping memory to user
 *      mappedkv:       kernel address for mapping to user
 *      mappedkvlen:    length of kernel buffer available for mapping
 *
 *      Board Addresses
 *      ----------------
 *	cfg_adr:	Board's Configuration Area Address 
 *	amcc_adr:	Amcc PCI address
 *	conf_adr:	Board's Config Register address
 *	norm_adr:	Board's Normal DMA Registers address
 *      cfg_map:        piomap_t for Config address space (if allocated)
 *      conf_map:       piomap_t for Config Register (if allocated).
 *      amcc_map:       piomap_t for Amcc Registers (if allocated)
 *      norm_map:       piomap_t for Normal DMA Registers (if allocated)
 *
 *      DMA Timing Measurement
 *	----------------------
 *	start_time:     Time DMA started
 *	intr_time:	Time board Interrupt completion of DMA
 *	call_time:	Time Call came into the driver
 *	ret_time:	Time driver returns to the user
 *
 *	Hardware Graph
 *      ---------------
 *      conn:           Our board's connection point (used by all pciio calls)
 *	vhdl:		Vertex handle representing this board.
 *
 *      Interrupt handler
 *	-----------------
 *	dev_intr:	Our Interrupt Handler structure
 *	dev_desc:	Our Device Descriptor structure
 *
 *      Locking/Sleep-wakeup
 *	--------------------
 *      coco_mlock:     spinlock or mutex lock used for locking this struct.
 *      coco_sv:        sv_t structure for sv_wait/sv_signal call pairs.
 *
 *      Timer and ABI info  
 *	------------------
 *	tid:		Timer ID ..timer waiting for board to interrupt. 
 *	kernel_abi:     Our ABI (64 or 32 bit)
 *      user_abi:       User's ABI (64 or 32 bit)
 */
typedef struct {
        int           status;

	/*  DMA Stuff  */ 
        uint_t        dmacfg;
        uint_t        dmabits;
	
	int	      dmatype;
	int	      dmastat;
	uint_t	      dmacmd;
	sema_t        dmawait;
	int	      iostat;
	int	      dmasize;
	alenaddr_t    wp_addr;
	alenaddr_t    rp_addr;
	size_t        rp_size;
	size_t        wp_size;
	buf_t	     *bp;
	caddr_t       chain_list;
	alenlist_t    addrList;
	alenlist_t    r_addrList;
	alenlist_t    w_addrList;
	int	      page_no;
	int	      r_page_no;
	int	      w_page_no;

	/*   Mapping Memory */ 
	vhandl_t      *vhandl;
	caddr_t       mappedkv;
	int	      mappedkvlen;

	/*   Board Addresses  */ 
        caddr_t       cfg_adr;
        caddr_t       amcc_adr;
        caddr_t       conf_adr;
        caddr_t       norm_adr;
        pciio_piomap_t  cfg_map;
        pciio_piomap_t  conf_map;
        pciio_piomap_t  amcc_map;
        pciio_piomap_t  norm_map;


	/*   DMA Timing Measurement  */ 
	struct timespec start_time;
	struct timespec intr_time;
	struct timespec call_time; 
	struct timespec ret_time; 

	/*   Hardware Graph  */ 
	vertex_hdl_t  conn;
	vertex_hdl_t  vhdl;

	/*  Interrupt handler  */
        pciio_intr_t  dev_intr;
        device_desc_t dev_desc;

	/*  Locking/Sleep-wakeup  */ 
	coco_lock_t   coco_mlock;
	sv_t	      coco_sv;

        /*   Timer and ABI info  */
	toid_t        tid;
	int	      kernel_abi;
 	int	      user_abi;

} card_t;

/*  bits for status  */
#define  CARD_ATTACHED  0x01
#define  CARD_OPEN      0x02

/*  dmatype values    */
#define  DMA_PROG     0  /*  chained - default value  */
#define  DMA_SINGLE   1

/*  dmastat values    */
#define  DMA_IDLE       0
#define  DMA_LUT_WAIT   1
#define  DMA_READ_WAIT  2
#define  DMA_WRITE_WAIT 3
#define  DMA_RW_WAIT    4

/*  iostat values  */
#define IO_OK        0
#define IO_ERROR     1
#define IO_TIME      2 
