#ifndef _VK_OVERLAYGL_H
#define _VK_OVERLAYGL_H

#include <Vk/VkComponent.h>
#include <GL/gl.h>
#include <GL/glx.h>

class VkOverlayGL : public VkComponent {
public:
  VkOverlayGL (const char *name, Widget parent);
  ~VkOverlayGL();
  virtual const char *className();

  void   makeGLAreaCurrent();
  void   makeOverlayAreaCurrent();

  void enableGLArea(Boolean enabled);
  void enableOverlayArea(Boolean enabled);
  void raiseOverlayArea(Boolean);

  GLXContext getGLContext() {return glContext;}
  GLXContext getOverlayContext() {return overlayContext;}  

  Widget getGLWidget() {return glArea;}
  Widget getOverlayWidget() {return overlayArea;}

  void addGLCallback(String name, XtCallbackProc func, XtPointer data);
  void addOverlayCallback(String name, XtCallbackProc func, XtPointer data);
  void addInput(int fd, XtInputCallbackProc func, XtPointer data);

  void handleGLInput(XtPointer callData);
  void handleOverlayInput(XtPointer callData);

  void handleGLExpose(XtPointer callData);
  void handleOverlayExpose(XtPointer callData);

  void handleGLInit(XtPointer callData);
  void handleOverlayInit(XtPointer callData);

  void handleGLResize(XtPointer callData);
  void handleOverlayResize(XtPointer callData);

  
  void   setRaisedOverlay(Boolean val) {overlayRaised = val;}
  void   setOverlayAreaEnabled(Boolean val) {overlayAreaEnabled = val;}
  void   setGLAreaEnabled(Boolean val) {glAreaEnabled = val;}

  Boolean isOverlayAreaEnabled() {return overlayAreaEnabled;}
  Boolean isGLAreaEnabled() {return glAreaEnabled;}

  // Members and methods that deal with UI elements used by RapidApp
  static const char *const overlayInputCallback;
  static const char *const glInputCallback;

  static const char *const overlayExposeCallback;
  static const char *const glExposeCallback;

  static const char *const overlayInitCallback;
  static const char *const glInitCallback;

  static const char *const overlayResizeCallback;
  static const char *const glResizeCallback;

protected:
  GLXContext	glContext;
  GLXContext    overlayContext;
  XVisualInfo*	  visInfo;
  XVisualInfo*	  overlayVisInfo;

  Widget          frame;
  Widget          form;
  Widget          glArea;
  Widget          overlayArea;

  Bool            overlayRaised;
  Bool            overlayAreaEnabled;
  Bool            glAreaEnabled;

  unsigned long   pixel;
  XColor	  col;
  Colormap	  cmap;

  void updateOverlayState();
  void updateMappedState();

private:
  static String _defaultResources[];
  static VkComponent *CreateVkOverlayGL(const char *name, Widget parent);
  static void *RegisterVkOverlayGLInterface();
  void afterRealizeHook();
  Widget parent;
  
};

#endif
