////////////////////////////////////////////////////////////////////////
//
// ScratchWindow.h
//	
//	A window for rendering OpenGL.  This class has no relation to the
//	dmu library.  It is just some sample code to show how to open
//	and set up a double buffering window for OpenGL rendering.
//	The following is an example of how this class is used.
//
//      ScratchWindow *window;
//      window = ScratchWindow::create(theDisplay, sizeX, sizeY);
//      if (!window){
//         fprintf (stderr, "Failed to create scratch window!\n");
//         exit (-1);
//      }
//      window->setContext();
//
//      glViewport( 0, 0, sizeX, sizeY);
//      glMatrixMode(GL_PROJECTION);
//      glLoadIdentity();
//      glOrtho(0,sizeX,0,sizeY,0,1000);
//      . . . continue Open GL here . . 
//
// Copyright 1995, Silicon Graphics, Inc.
// ALL RIGHTS RESERVED
//
// UNPUBLISHED -- Rights reserved under the copyright laws of the United
// States.   Use of a copyright notice is precautionary only and does not
// imply publication or disclosure.
//
// U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND:
// Use, duplication or disclosure by the Government is subject to restrictions
// as set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights
// in Technical Data and Computer Software clause at DFARS 252.227-7013 and/or
// in similar or successor clauses in the FAR, or the DOD or NASA FAR
// Supplement.  Contractor/manufacturer is Silicon Graphics, Inc.,
// 2011 N. Shoreline Blvd. Mountain View, CA 94039-7311.
//
// THE CONTENT OF THIS WORK CONTAINS CONFIDENTIAL AND PROPRIETARY
// INFORMATION OF SILICON GRAPHICS, INC. ANY DUPLICATION, MODIFICATION,
// DISTRIBUTION, OR DISCLOSURE IN ANY FORM, IN WHOLE, OR IN PART, IS STRICTLY
// PROHIBITED WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SILICON
// GRAPHICS, INC.
//
////////////////////////////////////////////////////////////////////////

#ifndef _ScratchWindow_H
#define _ScratchWindow_H

#include <sys/types.h>		// size_t

#include <GL/glx.h>

#include <Xm/Xm.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

class ScratchWindow
{
public:
    // The create function returns NULL if the window could not be
    // created.  The window is guaranteed to have 8 bits each of red,
    // greed, blue, and alpha.
    static ScratchWindow* create( Display*, int width, int height,int *argc, String *argv );
    virtual ~ScratchWindow();
    
    // setContext() makes this window the current GL context.  The
    // coordinate system is set up so that the rectangle from 
    // (0,0,0) to (width-1,height-1,0) will fill the window.
    virtual void   setContext()		= 0;

    virtual int    getWidth() 		= 0;
    virtual int    getHeight() 		= 0;
    virtual size_t getBufferSize()	= 0;
    virtual GLXContext getContext()     = 0;
    virtual void   addGLCallback(String name,
			       XtCallbackProc func,
			       XtPointer data) = 0;
    virtual void   eventLoop() = 0;
    virtual void addInput(int fd,XtInputCallbackProc func,XtPointer data) = 0;

};

#endif // _ScratchWindow_H
