//
//  dmuICjencoder.h
//
//  Author: Tim Davison
//  Date:   Oct, 1996
//
// <dmu>
//  This object provides the ability to jpeg encode images passed to it.
//  This object is a subclass of the <I>dmuIC</I> object.  This object
//  implements the setup method and creates a jpeg encoder.  The only
//  thing that this object has to do is create a valid image conversion
//  context within the setup method.  In addition, there is a method
//  for setting the quality of the compression.  
//  </dmu>
//
#ifndef _DMUICJENCODER_H
#define _DMUICJENCODER_H

#include "dmuIC.h"

// Symbols for the min and max quality allowed for setQuality
#define DMU_DMIC_JENCODE_MIN_QUALITY 0
#define DMU_DMIC_JENCODE_MAX_QUALITY 100

#define DMU_DMIC_JENCODE_DEFAULT_QUALITY  75

class dmuICjencoder : public dmuIC {
public:

  // Constructor:  This is the constructor that setups up some default
  // values.  If you use this constructor, you must set the image
  // width and height before you call the setup method.  This constructor
  // will also set the default quality to DMU_DMIC_JENCODE_DEFAULT_QUALITY.
  dmuICjencoder();

  // Constructor: You probably want to use this constructor.  It set some
  // default values and sets the image width and height.  
  dmuICjencoder(int imageSizeX, int imageSizeY);

  // Destructor:  Right now this does nothing other than implicitly calling
  // the superclass' destructor
  virtual ~dmuICjencoder();

  // This is the method that creates the image conversion context.
  // This method overrides the base class method.  It sets changes some
  // of the base class defaults.  The changes are shown here:
  // <TABLE BODER=1>
  // <TR><TD> Conversion Direction</TD><TD>
  //            DM_IC_CODE_DIRECTION_ENCODE</TD></TR>
  // </TABLE>
  int setup();

  // Allows you to set the quality of the jpeg compression.  The range of
  // values allowed are between DMU_DMIC_JENCODE_MIN_QUALITY = 0 and
  // DMU_DMIC_JENCODE_MAX_QUALITY = 100.  This value is the jpg Q factor
  // multiplied by 100 
  int setQuality(int quality);
  
protected:
  
private:
  // variable to store the current compression quality factor
  float quality;
};

#endif // _DMUICJENCODER_H
