#ifndef _DMUMOVIEINR_H
#define _DMUMOVIEINR_H

#include "dmuMovieIn.h"

// <dmu> The dmuMovieInRender class implements the functionality
// needed to read, decompress and render a movie into an off-screen
// DMbuffer.  To do this, the object uses the movie library to perform
// an <i>mvRenderMovieToOpenGL (3dm)</i> operation into an off screen
// rendering area (an OpenGL PBuffer).  </P>

// <P>If the movie is JPEG compressed, then the O2 will use hardware
// acceleration where it can to perform the decompression.  Note that
// this is not the fastest way that movies can be read in the dmu
// library.  A faster path for this case is to use a
// dmuCompressedMovieIn object to read the raw, compressed frames, and
// then pass them down the chain to a dmuICjdecoder object to be
// decompressed.</P>

// <P> The public interface for dmuMovieInRender is no different than
// that of a dmuMovieIn object.  The implementation of certain virtual
// methods (both public and protected) are what provides the
// functionality described above.  See <i>dmuMovieIn</i> for more
// information on the methods available to this class.</P>
// </dmu>

class dmuMovieInRender: public dmuMovieIn{
public:

  dmuMovieInRender();
  ~dmuMovieInRender();

  // This method allows you to describe where the rendering of the
  // movie will occur in the buffer, if the bufferSize > imageSize for
  // the movie.
  void setImageOffset(float x, float y);


  // This method allows for zooming of the movie by the factors
  // passed in for X and Y zoom.  Note that zooming may result in 
  // rendering larger than the buffer size if you aren't careful.
  void setImageZoom(float x, float y);

  int init(dmuParams *params = NULL);
  
protected:
  // Implements the appropriate read behavior to read a compressed or
  // uncompressed movie frame, and if needed, decompress it.  The 
  // result is a rendered video frame in a dmbuffer. (Which is 
  // associated with pbuffer)
  virtual int getImageFrameTime(long long movieTime);
  virtual int getImageFrameIndex(int frameIndex);

  // Variables required for creating the Pbuffer.  We need a PBuffer
  // so that we can use the mvRenderToOpenGL operation of the movie 
  // library to render the movie (using whatever decompression we 
  // need) into a dmBuffer.
  Display         *dpy;
  DMparams        *pbufParams;
  GLXFBConfigSGIX fbConfig;
  GLXPbufferSGIX  pbuffer;
  GLXContext      context;

  float offsetX;
  float offsetY;
  float zoomX;
  float zoomY;
private:
};

#endif
