//
//  dmuNetworkRecv.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// <dmu>
// This object provide the ability to get dm buffers from across
// the network.  This object is a subclass of the <I>dmuNetwork</I>
// object.  This object must be the head of a chain.  
//
// </dmu>
//
#ifndef _DMUNETWORKRECV_H
#define _DMUNETWORKRECV_H

#include "dmuNetwork.h"
#include "pentTCPSocketServer.h"
#include "pentData.h"

// Thread and semaphore stuff
#include <ulocks.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

class dmuNetworkRecv : public dmuNetwork {
public:

  // Constructor: This constructor sets up some default values.
  // If you use this constructor, you must set the image
  // width and height before you call the setup method.
  dmuNetworkRecv();

  // Constructor: You probably want to use this constructor.  It set some
  // default values and sets the image width and height.  
  dmuNetworkRecv(int imageSizeX, int imageSizeY);

  // Destructor:  Right now this does nothing.
  ~dmuNetworkRecv();

  // This method creates the dmbuffer pool 
  int init(dmuParams *params = NULL);

  // starts the object reading from the network.  This method actually
  // sprocs a read thread that will try to get data from the network.
  virtual int start(int propagate = DMU_PROPAGATE);
  
  // This method will stop the read thread from reading anymore data
  // from the network.  
  virtual int stop();
  
  // This method should never be called on this object.
  // This object can never be anything but the head of a chain
  // so nothing should pass buffers to this object.
  int receive(DMbuffer inBuffer);

  // This method will pass the buffer that was just read from the network
  // on down to the children of this object
  int handleEvents(fd_set fdSet);

  // tell the object what the packing format of that data to expect.
  // This is very important.  The size of each dmbuffer depends on the
  // packing, so this must be set correctly before calling init.  
  void setPacking(DMimagepacking _packing)    { packing = _packing;   }
  // return the current packing
  DMimagepacking getPacking()                 { return packing;       }

  // Tells the object that size of the image so it can allocate
  // buffers of the correct size
  void setImageDimensions(int _imageSizeX, int _imageSizeY) {
    imageSizeX = _imageSizeX;
    imageSizeY = _imageSizeY;
  }

  // this is the loop that reads from the network.  This should never
  // be call directly, it is called indirectly from the start method.
  void readLoop();

  // This method returns the fd to use in the select call in the
  // main event loop.  
  virtual int getFD();

  // return the data send along with the buffer sent across the
  // network.  
  int getLastHeaderVal()                      { return lastHeaderVal;}

protected:
  virtual int easyToAdd() {return DMU_TRUE;};

  int firstFrameHappened;
  int poolFD;
  int handledCurrent;

  usema_t *canRead;

  int canReadFD;

  usema_t *currentLock;
  int readLoopPID;
  int lastHeaderVal;

  void lockCurrentBuffer()    {  uspsema (currentLock);  }
  void unlockCurrentBuffer()  {  usvsema (currentLock);  }
  
  DMbuffer currentBuffer;
  DMbuffer readBuffer;
  
private:
  pentTCPSocketServer *srv;
  DMimagepacking    packing;
};

#endif //_DMUNETWORKRECV_H
