#ifndef DV_UTIL_H
#define DV_UTIL_H

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <unistd.h>
#include <bstring.h>
#include <sys/time.h>


/* --------------------- dvc1394.h ----------------------------- */

#define DVC1394_SET_CHANNEL             1
#define DVC1394_RELEASE_FRAME           2
#define DVC1394_GET_FRAME               3
#define DVC1394_GET_FILLED              4

#define ROUNDTOPAGE(x) (((x)+4095)& ~4095)

typedef struct dvc1394_buf_s {
    long long frame[18000];     /* A dvc frame */
    char pad0[3456];            /* Pad to end of page */
    long long cycletimer;       /* The 1394 cycletimer of the first block */
    int valid;                  /* 1 if all the consistency checks passed */
    int ntsc;
    char pad1[4096-16];         /* Pad to end of page */
} dvc1394_buf_t;

typedef struct dvc1394_state_s {
    int 	   fd;
    dvc1394_buf_t *bufs;
    int		   nbufs;
    int		   mmapsize;
}dvc1394_state_t;


dvc1394_state_t *
dvc1394openport(int nbufs);

void
dvc1394closeport(dvc1394_state_t *state);

void
dvc1394setchannel(dvc1394_state_t *state, int channel);

void
dvc1394getframe(dvc1394_state_t *state, long long **framep, int *validflag);

void
dvc1394releaseframe(dvc1394_state_t *state);

int
dvc1394getfilled(dvc1394_state_t *state);

int
dvc1394getfd(dvc1394_state_t *state);

int
dvc1394isntsc(dvc1394_state_t *state);

#endif


