#ifndef BRUSH_H
#define BRUSH_H

// GL
#include <GL/glu.h>

// System V
#include <stdio.h>
#include <assert.h>

#define  PEN(x)             ((penDisplayListOffset) + (x))
#define PREVIEW_MODE 1
#define DRAW_MODE    2

#define PROFILE_LINEAR    1
#define PROFILE_GAUSSIAN  2
#define PROFILE_SOLID     3
#define PROFILE_COSINE    4
#define MAX_PROFILES      4

typedef struct _profile {
  unsigned char *pixels;
  int width;
  int height;
} profile;

int pbufferMode12bit = 0;

class Brush {
public:
  Brush();
  ~Brush();
  
  void buildBrush(int _viewWidth, int _viewHeight);
  void draw(int x, int y);
  void setProfile(int profileNum, int width, int height,
		  unsigned char *pixels);
  void setScale(float scaleX, float scaleY);
  void setDynamicScale(float scaleX, float scaleY);
  void setAngle(float angle);
  void setFlatness(float flatness);
  float getNormalizedWidth(void) { return (width * scaleX * (1.0 - flatness));}
  float getNormalizedHeight(void) { return (height * scaleY);}
  float getMaxNormWidth(void) { return (width * scaleX);}
  float getMaxNormHeight(void) { return (height * scaleY);}
  int getWidth(void) {return width;}
  int getHeight(void) {return height;}
  float getScaleX(void) { return scaleX;}
  float getScaleY(void) { return scaleY;}
  float getAngle(void) { return angle;}
  float getFlatness(void) { return flatness;}
  void previewDraw();
  void loadTex(int _width, int _height,  unsigned char *pixels);
  void setIntensity(float i);
  void setDynamicIntensity(float i);
  void setColor(float r, float g, float b);
  void setBgImg(int _x, int _y, unsigned char *_bgImgPixels);
  void transform(int _x, int _y);
  void previewTransform();
  void setPaintMode(int _mode);
  int  getPaintMode( void ) { return whichPen; };
  void setConv(int conv);
  void useProfile(int profile);
  
  // pens...
  enum pens {
    PAINT,
    PROCESS,
    COMPOSITE,
    TEXTURE_QUAD,
    VIEWPORT,
    DO_PAINT,
    DO_PROCESS,
    DO_COMPOSITE,
    PREVIEW_PAINT,
    PREVIEW_PROCESS,
    PREVIEW_COMPOSITE,
    TRANSFORM_PAINT,
    TRANSFORM_PROCESS,
    TRANSFORM_COMPOSITE,
    PEN_COUNT
  };

  enum convs {
    BLUR,
    EDGE,
    SHARPEN
  };
  
protected:
  unsigned int width;
  unsigned int height;

  unsigned int viewWidth;
  unsigned int viewHeight;
  
  float scaleX, staticScaleX;
  float scaleY, staticScaleY;
  float angle;
  float flatness;
  float intensity;

  int whichPen;
  int whichConv;
  
  unsigned char *pixels;
  unsigned char *bgImgPixels;

  int bgImgWidth;
  int bgImgHeight;
  
  int profileMode;
  
  float color[4];
  
private:

  profile  profileList[MAX_PROFILES];
  
  //   enum pens {
//     PEN_AND_VIEWPORT,
//     TEXTURED_PEN,
//     TRANSFORM,
//     PEN_COMPOSITE,
//     PEN_COUNT
//   };

  unsigned int penDisplayListOffset;

};
#endif


