#ifndef CANVAS_H
#define CANVAS_H

#include <GL/glx.h>

#include "libpbrush/Brush.h"
#include "libpbrush/CatmullRomStroke.h"
#include "libdmu/dmuGLPaintBuffer.h"
#include "libdmu/dmuVideoOut.h"

#define STEP_SIZE 1
#define PRESSURE_SCALE 4.0f
#define PRESSURE_RAMP (255.0f/(PRESSURE_SCALE))

class Canvas {
public:
  Canvas();
  ~Canvas();

  enum {
    NONE_BRUSH_PRESSURE,
    SCALE_BRUSH_PRESSURE,
    FADE_BRUSH_PRESSURE,
    BOTH_BRUSH_PRESSURE
  };

  
  void setup(Widget widget, int width, int height, XtEventHandler proc,
	     XtPointer callData);
  void setupBrush(int width, int height, unsigned char *pixels);
  void redraw();
  void xferPaintBuffer(Widget glWidget, GLXContext glScreenContext = NULL);
  Brush *getBrush();
  void handleEvent(Widget w,  XEvent *e);
  void expose(void);
  void refresh(void);
  virtual void setInterpolation(Bool onOff);
  virtual void setInterpolationStepSize(int size);
  virtual void setStampStepSize(int size);
  void setFrontImg(unsigned char *img);
  void brushChange();
  void setPressureMode(int pressureMode);
  
protected:
  static void drawCB(int x, int y, Bool first, void *callData);
  void draw(int x, int y, Bool firstPoint);

  void allocPBuffer(Display *dpy);

  CatmullRomStroke *stroke;
  Brush *brush;

  
#ifdef IMPACT_GRAPHICS
  // we want to use libdmu, but it doesn't run on impact, so 
  // we'll use a different pbuffer allocation scheme.
  GLXDrawable pbuffer;  
  GLXContext glContext;
#else 
  // use libdmu to allocate the pbuffer, so we can go to videoOut
  dmuGLPaintBuffer *paintBuffer;
  dmuVideoOut      *videoOut;

  int videoSizeX, videoSizeY;
#endif //IMPACT_GRAPHICS

  unsigned char *pixels;
  unsigned char *brushPixels;
  
  unsigned int width;
  unsigned int height;

  unsigned int stepSize;
  unsigned int pressureScale;
  unsigned int pressureSteps;
  
  Bool EXPOSE;
  Bool RESET_BOUNDING_BOX;
  
  int minX, maxX;
  int minY, maxY;
  int brushLastMaxNormWidth;
  int brushLastMaxNormHeight;
  
  // pressure
  int lastPressure;
  Bool pressurePaint;
  Bool newStroke;
  int pressureMode;


  // cached mouse location for updating pointer on videoOut
  int lastMouseX;
  int lastMouseY;

  int lastCursorX;
  int lastCursorY;

  Display *dpy;
  Widget displayWidget;
private:
  static void prepGLRenderArea(int w, int h);
  void renderCursor(int x, int y);  
};

#endif
