#ifndef STROKE_H
#define STROKE_H

// Local
#include "ParametricCurve.h"
#include "Primitive.h"

// Wacom Tablet
#include "libpxinput/WacomTablet.h"


class Stroke {
  public:
    Stroke( Display *display, ParametricCurve * = NULL );

    void registerPrimitiveCallback(Primitive::RenderPrimitiveCallback);
    void registerDirectRenderCallback(ParametricCurve::DirectRenderCallback,
				      void *userData);

    virtual void	draw( void );
    virtual void	stroke( void );
    virtual Bool 	handleEvent( Window, XEvent * );
    virtual Bool 	handleMouseEvent( Window, XEvent * ) = 0;
    virtual Bool 	handleWacomEvent( Window, XEvent * ) = 0;
    virtual Bool	isDirty( void );
    virtual Bool	isSelected( void );
    virtual void	Deselect( void );
    virtual int		getPressure( void );
    virtual Bool	setPressureCurve( char *values  );
    virtual void	setPressureScaleValue( int divisor  );
    virtual int		getPressureScaleValue( void );
    virtual void	routeEvents( Widget w, unsigned int mask,
				     XtEventHandler proc = NULL,
				     XtPointer callData = NULL);
  void enableInterp(Bool onOff) { curve->enableInterp(onOff); }
  
    // Painting Characteristics
    virtual void	setStepSize( unsigned int size );
    virtual void	setInterpolationStepSize( unsigned int size );

  protected:
    typedef enum { INACTIVE,
		   MOVE_KNOT,
		   MOVE_WEIGHT,
		   MOVE_CURVE,
		   TACK,
		   STATE_COUNT
    } State;

    WacomTablet *wacom;

    Primitive		primitives;
    int			pressureScaleValue;
    State		state;
    ParametricCurve	*curve;
    Bool		selected;
    Bool		knotSelected;
    Bool		dirty;
  
    typedef struct {
	int startx;
	int starty;
	int lastx;
	int lasty;
	Time startTime;
	int clickCount;

	ParametricCurve::KNOT	knot;
	ParametricCurve::WEIGHT weight;
    } EventStaticData;

    EventStaticData eventStatic;

    static void handleEventsCB( Widget w, XtPointer callData, XEvent *e,
				Boolean *cont  );
};

#endif

