#ifndef PENT_DATA_H
#define PENT_DATA_H

#include "pentCommon.h"

class pentData {
public:
  pentData();
  ~pentData();

  // give the data object an fd to talk over.
  // The user should never call this, instead, pass the data object
  // to the sockets class in the handshake call.
  int setFD(int _fd);
  int getFD(int *_fd);

  // this will tell this object if the connection it is using is a reliable
  // connection like TCP or an unreliable one like UDP
  int setConnType(int _connType);
  int getConnType(int *_connType);
  
  // with this call, the user program specifies the size of the header to
  // send across the network.  This must be the same on both sides of the
  // network.  It is up to the user program to ensure that the header size
  // stays the same.  
  int setHeaderSize(int _headerSize);
  // this will merely return the current header size
  int getHeaderSize(int *_headerSize);


  // the user program sends the actual data with this call.  Before this
  // call will work, you must have given the data object an fd - probably
  // by passing the data object to the socket class handshake method.  You
  // must also have setup the header size for this class to expect.
  // Arguments:
  //    _headerBuf:    pointer to the data to send as the header
  //    _dataBuf:      pointer to the data to send as the data
  //    _dataBufSize:  Size of the data you want to send
  int sendData(void *_headerBuf, void *_dataBuf, int _dataBufSize);

  // get the header for the next packet.
  int getHeader(void *_headerBuf);
  // get the size of the data for the next packet
  int getSize(int *_dataBufSize);
  // Get the data portion of the packet
  int getData(void *_dataBuf);
  // get both the data and the size of the next packet
  int getData(void *_dataBuf, int *_dataBufSize);

  // give the object a name for debugging printing
  int setName(char *_name);
  char *getName() {return name;}

  // print out when you try to return an error with the PENT_ERR macro
  int setPrintReturnErr(int _printReturnErr);

  int setDstAddr(struct sockaddr *_dstAddr, int _dstAddrLen);

  
  //
  //   OLD
  //
  //
#ifdef OLD  
  // These next two calls must be alternated or else, you will
  // lose synchronization with the network.  The getCurHeader call
  // has to happen first, then you can call getCurData.  
  // This call will read the next header from the network.  
  int getCurHeader(void *_headerBuf, int _headerBufSize, int *_dataSize);
  // gives you the next data message from the network.  
  int getCurData(void *_dataBuf, int *_sizeRead);
#endif

protected:
  int     fd;
  char   *name;
  int     headerSize;
  int     curDataSize;
  int     connType;
  int     printReturnErr;
  struct sockaddr *dstAddr;
  int     dstAddrLen;

  int readHeader;
  int readSize;
  int readIOV(struct iovec *_iov, int _iovLen, int *_totalRead, int flags = 0);
};

#endif
