#ifndef PSOUND_H
#define PSOUND_H

#include "PhlatServer.h"

// pent networking stuff
#include "libpent/pentCommon.h"
#include "libpent/pentTCPSocketClient.h"
#include "libpent/pentData.h"


class PSound{

public:
  PSound();
  ~PSound();

  // Playback controls
  int startPlaying();
  int stopPlaying();

  // Network controls
  int startTransmitting();
  int stopTransmitting();
  int  setPentSendData(pentData *dataObj);

  // This should be protected, but useable by phlat, but I don't know the 
  // syntax for friend classes, so I'll make it public for now XXXX
  PSound *getNextSound() {return (serverQueueNext);}
  void setNextSound(PSound *nextSound) {serverQueueNext = nextSound;}


  // getSampleRate - get the sample rate of my sound file 
  int getSampleRate() {return (sampleRate);}

  // getSamples - fill the buffer with n seconds worth of samples.  
  //              Do the up-sampling or downsampling needed to get to 
  //              sampleRate.  (buf must be allocated already).  return
  //              either PHLAT_OK, PHLAT_ERROR or PHLAT_SOURCE_FINISHED.
  //              (the last one meaning that we don't have anything
  //              else to contribute
  virtual int getSamples(float nSeconds,int serverRate, 
		 int serverChannels,Sample *serverBuf);


  // setIntensity - Set the instensity scale factor of the sample
  float setIntensity (float val);
  float getIntensity()   { return intensityScale; }


  // Pitch shift - If you don't implement this feature, then override
  //               setPitchShift() to return either error or unimplemented
  //               return value
  virtual int setPitchShift(float val);
  float getPitchShift() { return (pitchShift); }


  bool isPlaying() {return playing;}
  bool isTransmitting() {return transmitting;}


  // This is called by the server to tell us to send our
  // potentially-resampled and pitch shifted samples over the network.
  int sendSamples(float nSeconds,int serverRate, Sample *serverBuffer);

protected:

  static PhlatServer *server;
  static PhlatServer *initializeServer();

  // Pointer to the next sound in the server queue
  PSound *serverQueueNext;
  int curSample;

  // Abstract source info (specific info in the subclasses)
  int sampleRate;
  int nChannels;

  // Scale factor - before mix, samples are scaled by this -1.0 -> 1.0 value
  float intensityScale;

  // See notes above in setPitchShift();
  float pitchShift;

  // Flags to determine if we're playing, transmitting, or both.
  bool playing;
  bool transmitting;

  pentTCPSocketClient *pentClientSocket;
  pentData            *pentDataObj;

private:

};

#endif
