#ifndef _PHLAT_SERVER
#define _PHLAT_SERVER


#define DEBUG1(x) x
#define DEBUG2(x) x
#define DEBUG3(x) 

#define CHUNK_TIME ((float)0.2)
#define OUTPUT_CHANNELS 2

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/prctl.h>
#include <sys/unistd.h>
#include <signal.h>

#include <audio.h>

// Types
typedef int16_t Sample;

#ifndef bool
typedef char bool;
#endif 

#define TRUE 1
#define FALSE 0

// Define the type and size of the samples we'll be using
// Any files that use other formats will have to be converted
// to this type via the virtual format function
#define PHLAT_SAMPLE_FORMAT AF_SAMPFMT_TWOSCOMP
#define PHLAT_SAMPLE_BITS   16
#define PHLAT_SAMPLE_BYTES  2

// Constants for return values
#define PHLAT_NETWORK_ERROR       -2
#define PHLAT_ERROR           -1
#define PHLAT_OK              0
#define PHLAT_SOURCE_FINISHED 1

// CD return values
#define PHLAT_CD_ERROR      -10
#define PHLAT_CD_NO_DRIVE   -11
#define PHLAT_CD_NOT_AUDIO  -12
#define PHLAT_CD_NOT_READY  -13


// Network defines
#define PHLAT_PORT 12000


class PSound;


class PhlatServer{
public:
  // Constructor
  PhlatServer();
  ~PhlatServer();
  
  // Initialization - start up the server proc and init the AL
  int initializeServer();


  // Main server loop where we do stuff
  void serverLoop();

  // Sound queue methods (linked lists) - Anything on the queue
  // is either playing or transmitting sound
  void addSound(PSound *newSound);
  void removeSound(PSound *targetSound);

  // Methods to query the audio sample rate
  static long getOutputRate();
  static long getInputRate();

protected:

  // Head of a list of (PSound) sound sources.
  PSound *sourceHead;
  int    numSound;

  // Process ID of the server process (-1 if we don't have a server PID)
  static int serverPID;

  // Current output sample rate (event network-transmitted sounds
  // are re-sampled to this rate)
  int      sampleRate;

  // Buffers
  int    bufferSize;
  Sample *outBuf;
  Sample *scratchBuf;

  // AL variables
  ALconfig config;
  ALport   outPort;
  int      portFD;
  fd_set   writeFDSet;

  // Mix the scratch buffer into the output buffer for playback
  void mixScratch(float nSeconds,float intensityScale);

private:
  // Allocate the appropriate sized buffers, according to the 
  // current sample rate and CHUNK_TIME variables
  int allocateBuffers();
  int initializePort();
  
};

#endif
