#ifndef X_INPUT_DEVICE_H
#define X_INPUT_DEVICE_H
 
/* X11 */
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

/* SGI X11 Extensions for input */
#include <X11/extensions/XInput.h>
#include <X11/extensions/SGIMisc.h>

#define DeviceKeyPressMask		(1L<<0)
#define DeviceKeyReleaseMask		(1L<<1)
#define DeviceButtonPressMask		(1L<<2)
#define DeviceButtonReleaseMask		(1L<<3)
#define DeviceMotionNotifyMask		(1L<<4)
#define DeviceFocusInMask		(1L<<5)
#define DeviceFocusOutMask		(1L<<6)
#define DeviceStateNotifyMask		(1L<<7)
#define DeviceMappingNotifyMask		(1L<<8)
#define DevicePointerMotionHintMask	(1L<<9)
#define DeviceButton1MotionMask		(1L<<10)
#define DeviceButton2MotionMask		(1L<<11)
#define DeviceButton3MotionMask		(1L<<12)
#define DeviceButton4MotionMask		(1L<<13)
#define DeviceButton5MotionMask		(1L<<14)
#define DeviceButtonMotionMask		(1L<<15)
#define DeviceOwnerGrabButtonMask	(1L<<16)
#define DeviceButtonPressGrabMask	(1L<<17)
#define ProximityInMask			(1L<<18)
#define ProximityOutMask		(1L<<19)
#define ChangeDeviceNotifyMask		(1L<<20)

class XInputDevice {

  public:
    XInputDevice( Display *display, char *deviceName );
    ~XInputDevice( void );

    XKeyInfoPtr			key;
    XButtonInfoPtr		button;
    XValuatorInfoPtr		valuator;
    XAxisInfoPtr		axis;

    Bool			isKeyClass;
    Bool			isButtonClass;
    Bool			isValuatorClass;

    XEventClass enabledEventClass[20];
    int enabledEventID[20];
    int event_class_count;

    unsigned int		deviceKeyPressEventID;
    unsigned int		deviceKeyReleaseEventID;
    unsigned int		deviceButtonPressEventID;
    unsigned int		deviceButtonReleaseEventID;
    unsigned int		deviceMotionNotifyEventID;
    unsigned int		deviceFocusInEventID;
    unsigned int		deviceFocusOutEventID;
    unsigned int		deviceStateNotifyEventID;
    unsigned int		deviceMappingNotifyEventID;
    unsigned int		devicePointerMotionHintEventID;
    unsigned int		deviceButton1MotionEventID;
    unsigned int		deviceButton2MotionEventID;
    unsigned int		deviceButton3MotionEventID;
    unsigned int		deviceButton4MotionEventID;
    unsigned int		deviceButton5MotionEventID;
    unsigned int		deviceButtonMotionEventID;
    unsigned int		deviceOwnerGrabButtonEventID;
    unsigned int		deviceButtonPressGrabEventID;
    unsigned int		proximityInEventID;
    unsigned int		proximityOutEventID;
    unsigned int		changeDeviceNotifyEventID;

    virtual Bool		printDeviceList( Display *display );
    virtual Bool	 	control( char *name, char *value );
    virtual Bool	 	query( char *name, char *value );
    virtual Bool		isValid( void );
    virtual void		selectInput( Window win, unsigned int mask );
    static  Boolean		dispatchXInputEvents( XEvent *event );

  protected:
    Display *display;
    XID device_id;
    char *deviceName;
    Bool validDevice;
    XDevice *device;

  private:
    virtual  XID		getDeviceID( Display *display, 
				     char *deviceName = NULL,
				     Bool PRINT_DEVICE_LIST = False);

};

#endif
