#ifndef _SO_DM_BUFFER_BACKGROUND_H
#define _SO_DM_BUFFER_BACKGROUND_H

#include <Inventor/nodes/SoShape.h>
#include <Inventor/fields/SoSFTrigger.h>
#include <Inventor/fields/SoSFMatrix.h>

/**
 * Shape Node
 * <p>
 * This shape node automatically tracks the viewport and resizes
 * accordingly. It is useful for texturemapping video, movies, etc.
 * to the background of an Inventor scene -- thus tacking advantage
 * of hardware texturemapping, anti-aliasing, etc. for arbitrary
 * window sizes.
 * <p>
 * <b>Techinical Note</b>
 * This shape disables GL_DEPTH since it is typically rendered in the
 * background. For this reason, the background node should be the
 * first item rendered in a path, or it will obscure any remaining
 * objects.
 */
class SoDMBufferBackground : public SoShape {

    SO_NODE_HEADER(SoShape);

  public:

    // Constructor
    SoDMBufferBackground();

    SoSFTrigger		syncIn;
    
  SoINTERNAL public:
    static void		initClass();

  protected:
    virtual void	GLRender(SoGLRenderAction *action);
    virtual void	generatePrimitives(SoAction *action);
    virtual void	computeBBox(SoAction *action,
				    SbBox3f &box, SbVec3f &center);
  private:
    virtual ~SoDMBufferBackground();
    
};

#endif 
