#include "SoDMBufferEngine.h"
#include <Inventor/fields/SoSFMatrix.h>

// System V
#include <stropts.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/prctl.h>

SO_ENGINE_ABSTRACT_SOURCE( SoDMBufferEngine );

unsigned int SoDMBufferEngine::burnCount = 0;
usptr_t *SoDMBufferEngine::arena = NULL;
usema_t *SoDMBufferEngine::burnSema = NULL;
pid_t SoDMBufferEngine::child = NULL;
list<SoDMBufferEngine::BurnItem *> SoDMBufferEngine::burnStack;

void
SoDMBufferEngine::initClass( void ){
    SO_ENGINE_INIT_ABSTRACT_CLASS( SoDMBufferEngine, SoEngine, "Engine");

    static SbBool doOnce = True;
    if ( doOnce ){
	//--- Semaphores -----------------------------------
	usconfig(CONF_ARENATYPE, US_SHAREDONLY);
	arena = usinit(mktemp("/tmp/alXXXXXX"));
	if (arena == NULL)
	    perror("Trouble creating arena.\n");

	burnSema = usnewsema(arena, 1);
	if (burnSema == NULL)
	    perror("SbDMBuffer::Trouble creating mutex semaphore.\n");

	//--- Semaphores  -----------------------------------

	//--- Engine Thread Creation ------------------------
	prctl(PR_SETEXITSIG, SIGTERM);

	if ((child = sproc(burn, PR_SALL, NULL)) == -1)
	    fprintf(stderr, "Form::open unable to thread dmu engines.\n");

	doOnce  = False;
	//--- Thread Creation -------------------------------
    }
}

SoDMBufferEngine::SoDMBufferEngine(){
    SO_ENGINE_CONSTRUCTOR( SoDMBufferEngine );
    SO_ENGINE_ADD_INPUT( frameTrigger, () );

    SO_ENGINE_ADD_OUTPUT( textureBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( linearBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( scale, SoSFMatrix );

    frameTrigger.connectFrom( SoDB::getGlobalField("realTime") );
    isBuiltIn = True;
};

void
SoDMBufferEngine::evaluate(){
    SO_ENGINE_OUTPUT( scale, SoSFMatrix, setValue( scaleMatrix ));
    SO_ENGINE_OUTPUT( textureBits, SoSFDMBufferImage, setValue( engineDMBuffer ));
    SO_ENGINE_OUTPUT( linearBits, SoSFDMBufferImage, setValue( engineDMBuffer ));
}


void
SoDMBufferEngine::registerBurnItem( dmuObj *dmuobj ){

    BurnItem *bi = new BurnItem;
    bi->dmuobj = dmuobj;
    bi->started = False;

    uspsema(burnSema);

    burnStack.push_back( bi );
    burnCount++;

    usvsema(burnSema);
}

void
SoDMBufferEngine::burn( void *){

    int    dmuErr;
    fd_set readFDSet;
    int	   maxFD = -1;

    for (;;){
	uspsema(burnSema);
	if ( burnCount == 0 ){
	    usvsema(burnSema);
	    continue;
	}

	FD_ZERO(&readFDSet);
	list<SoDMBufferEngine::BurnItem *>::iterator i;
	for(i=burnStack.begin();i!=burnStack.end();i++){
	    if ( !(*i)->started ){
		(*i)->dmuobj->start();
		(*i)->started = True;
	    }
	    (*i)->dmuobj->setFD(&readFDSet,&maxFD);
	}

	select(maxFD + 1, &readFDSet, NULL, NULL, NULL);
	for(i=burnStack.begin();i!=burnStack.end();i++)
	    dmuErr = (*i)->dmuobj->handleEvents(readFDSet);
	usvsema(burnSema);
    }
}
