#include "SoDMBufferMovieEngine.h"

// Inventor
#include <Inventor/fields/SoSFMatrix.h>

SO_ENGINE_SOURCE( SoDMBufferMovieEngine );

void
SoDMBufferMovieEngine::initClass( void ){
    // Ensure Parent is Init'd!
    if (SoDMBufferEngine::getClassTypeId().isBad())
	SoDMBufferEngine::initClass();
    SO_ENGINE_INIT_CLASS( SoDMBufferMovieEngine, SoEngine, "Engine");
}
    
SoDMBufferMovieEngine::SoDMBufferMovieEngine( void ):
     SoDMBufferEngine()
{
    SO_ENGINE_CONSTRUCTOR( SoDMBufferMovieEngine );

    SO_ENGINE_ADD_INPUT( filename, ("") );

    SO_ENGINE_ADD_OUTPUT( textureBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( linearBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( scale, SoSFMatrix );

    movieIn = NULL;
    isBuiltIn = True;

}

SoDMBufferMovieEngine::~SoDMBufferMovieEngine(){
    delete engineDMBuffer;
}


void
SoDMBufferMovieEngine::inputChanged(SoField *whichField){
    if ( whichField == &filename &&
	 movieIn == NULL ){

	int dmuErr, movieFD;
	int imageSizeX,imageSizeY;
	int bufferSizeX,bufferSizeY;
    
	Display *display = XOpenDisplay(":0.0");
	// Create dmu objects
	movieIn = new dmuMovieInRender;
	movieIn->setName("movie1");

	texDrain = new dmuGLTex2d;
	texDrain->setName("tex1");

	movieContext = new dmuGLCtx;
	movieContext->setName("movieContext");

	// DMU Chain
	movieIn->addObject(movieContext);
	movieContext->addObject(texDrain);

	dmuErr = movieIn->setup((char *) filename.getValue().getString());
	if (dmuErr != DMU_OK){
	    fprintf (stderr, "movie setup error\n");
	    exit (-1);
	}

	movieIn->getImageDimensions(&imageSizeX,&imageSizeY);
	movieIn->getBufferDimensions(&bufferSizeX,&bufferSizeY);
	engineDMBuffer = new SbDMBuffer( movieContext, texDrain );

	movieContext->setup(display,bufferSizeX,bufferSizeY);
	texDrain->setup(display,bufferSizeX,bufferSizeY);

	dmuErr = movieIn->init();
	if (dmuErr != DMU_OK){
	    fprintf (stderr,"movie setup error\n");
	    exit (-1);
	}

	float rx = (float) imageSizeX/(float) bufferSizeX;
	float ry = (float) imageSizeY/(float) bufferSizeY;

	float scale[4][4] = { rx,  0,  0,  0,
			      0, ry,  0,  0,
			      0,  0,  1,  0,
			      0,  0,  0,  1 };

	scaleMatrix.setValue( scale );

	registerBurnItem( movieIn );
    }
    evaluate();
}

void
SoDMBufferMovieEngine::stop(){
    // XXX - Add to libdmu please
    /*
      if ( MOVIE_LAYOUT == LAYOUT_LINEAR ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieInLinear->stop();

      if ( MOVIE_LAYOUT == LAYOUT_GRAPHICS ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieIn->stop();
	*/
}

void
SoDMBufferMovieEngine::start(){
    // XXX - Add to libdmu please
    /*
      if ( MOVIE_LAYOUT == LAYOUT_LINEAR ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieInLinear->start();

      if ( MOVIE_LAYOUT == LAYOUT_GRAPHICS ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieIn->start();
	*/
}
