#ifndef  _SO_SF_DMBUFFER_MOVIE_ENGINE_H
#define  _SO_SF_DMBUFFER_MOVIE_ENGINE_H

// Inventor
#include <Inventor/fields/SoSFString.h>

// DMUIV
#include "SoDMBufferEngine.h"

// DMU
#include "libdmu/dmuMovieInRender.h"
#include "libdmu/dmuGLTex2d.h"
#include "libdmu/dmuGLCtx.h"

/**
 * SoDMBufferMovieEngine renders a movie
 * to it's outputs: <b>textureBits</b> and <b>linearBits</b>. These
 * outputs are typically tied to an SoSFDMBufferImage (such as the
 * image field of an SoDMBufferTexture2 node.
 * <p>
 * The scale varable is adjusted, based on the size of the movie,
 * to properly scale downstream textures when connected to an
 * SoDMBufferTexture2 object.
 * <p>
 * <b>Fields from class SoDMBufferEngine:</b>
 * <pre>
 *    SoEngineOutput textureBits;
 *    SoEngineOutput linearBits;
 *    SoEngineOutput scale;
 * </pre>
 * <p>
 * @see SoDMBufferDMICMovieEngine.html
 */
class SoDMBufferMovieEngine : public SoDMBufferEngine {

    SO_ENGINE_HEADER( SoDMBufferMovieEngine );

  public:

    SoSFString	filename;

    static void 	initClass();

    /**
     * Constructor
     */
    SoDMBufferMovieEngine( void );

    /**
     * Stop dynamically updating the current video image.
     */
    virtual void stop( void );

    /**
     * Resume dynamically updating the current video image.
     */
    virtual void start( void );

  protected:
    dmuMovieInRender *movieIn;
    dmuGLCtx   *movieContext;
    dmuGLTex2d *texDrain;

    virtual void        inputChanged(SoField *whichField);

  private:

    ~SoDMBufferMovieEngine();

};

#endif
