#include "SoDMBufferRawDVEngine.h"

// Inventor
#include <Inventor/fields/SoSFMatrix.h>

SO_ENGINE_SOURCE( SoDMBufferRawDVEngine );

void
SoDMBufferRawDVEngine::initClass( void ){
    // Ensure Parent is Init'd!
    if (SoDMBufferEngine::getClassTypeId().isBad())
	SoDMBufferEngine::initClass();
    SO_ENGINE_INIT_CLASS( SoDMBufferRawDVEngine, SoEngine, "Engine");
}
    
SoDMBufferRawDVEngine::SoDMBufferRawDVEngine( void ):
     SoDMBufferEngine()
{
    SO_ENGINE_CONSTRUCTOR( SoDMBufferRawDVEngine );

    SO_ENGINE_ADD_INPUT( filename, ("") );

    SO_ENGINE_ADD_OUTPUT( textureBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( linearBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( scale, SoSFMatrix );

    movieIn = NULL;
    isBuiltIn = True;

}

SoDMBufferRawDVEngine::~SoDMBufferRawDVEngine(){
    delete engineDMBuffer;
}


void
SoDMBufferRawDVEngine::inputChanged(SoField *whichField){
    if ( whichField == &filename &&
	 movieIn == NULL ){

	int dmuErr, movieFD;
	int imageSizeX,imageSizeY;
	int bufferSizeX,bufferSizeY;
    
	Display *display = XOpenDisplay(":0.0");
	// Create dmu objects
	movieIn = new dmuMovieInRawDV;
	movieIn->setName("movie1");

	texDrain = new dmuGLTex2d;
	texDrain->setName("tex1");

	movieContext = new dmuGLPixelTrans;
	movieContext->setName("movieContext(pixelTrans)");

	decoder = new dmuICdvdecoder;
	decoder->setName("movieDecoder");

	// DMU Chain
	movieIn->addObject(decoder);
	decoder->addObject(movieContext);
	movieContext->addObject(texDrain);

	dmuErr = movieIn->setup((char *) filename.getValue().getString());
	if (dmuErr != DMU_OK){
	    fprintf (stderr, "movie setup error\n");
	    exit (-1);
	}

	movieIn->getImageDimensions(&imageSizeX,&imageSizeY);
	movieIn->getBufferDimensions(&bufferSizeX,&bufferSizeY);
	movieIn->setBufferDimensions(imageSizeX,imageSizeY);

	decoder->setImageDimensions(imageSizeX,imageSizeY);
	decoder->setDstPacking(DM_IMAGE_PACKING_CbYCrY);
	decoder->setup();

	engineDMBuffer = new SbDMBuffer( movieContext, texDrain );

	movieContext->setup(display,bufferSizeX,bufferSizeY);
	movieContext->setImageDimensions(imageSizeX, imageSizeY);
	movieContext->setInputFormat(GL_YCRCB_422_SGIX);

	texDrain->setup(display,bufferSizeX,bufferSizeY);

	dmuErr = movieIn->init();
	if (dmuErr != DMU_OK){
	    fprintf (stderr,"movie setup error\n");
	    exit (-1);
	}

	float rx = (float) imageSizeX/(float) bufferSizeX;
	float ry = (float) imageSizeY/(float) bufferSizeY;

	float scale[4][4] = { rx,  0,  0,  0,
			      0, ry,  0,  0,
			      0,  0,  1,  0,
			      0,  0,  0,  1 };

	scaleMatrix.setValue( scale );

	registerBurnItem( movieIn );
	registerBurnItem( decoder );
	//registerBurnItem( movieContext );
    }
    evaluate();
}

void
SoDMBufferRawDVEngine::stop(){
    // XXX - Add to libdmu please
    /*
      if ( MOVIE_LAYOUT == LAYOUT_LINEAR ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieInLinear->stop();

      if ( MOVIE_LAYOUT == LAYOUT_GRAPHICS ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieIn->stop();
	*/
}

void
SoDMBufferRawDVEngine::start(){
    // XXX - Add to libdmu please
    /*
      if ( MOVIE_LAYOUT == LAYOUT_LINEAR ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieInLinear->start();

      if ( MOVIE_LAYOUT == LAYOUT_GRAPHICS ||
      MOVIE_LAYOUT == LAYOUT_BOTH )
      movieIn->start();
	*/
}
