#include "SoDMBufferSceneEngine.h"

// Inventor
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoResetTransform.h>
#include <Inventor/actions/SoSearchAction.h>

// System V
#include <stropts.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/prctl.h>

SO_ENGINE_SOURCE( SoDMBufferSceneEngine );

void
SoDMBufferSceneEngine::initClass( void ){
    // Ensure Parent is Init'd!
    if (SoDMBufferCompositeEngine::getClassTypeId().isBad())
	SoDMBufferCompositeEngine::initClass();
    SO_ENGINE_INIT_CLASS( SoDMBufferSceneEngine, SoDMBufferCompositeEngine, "SoDMBufferCompositeEngine");
}

SoDMBufferSceneEngine::SoDMBufferSceneEngine()
{
    SO_ENGINE_CONSTRUCTOR( SoDMBufferSceneEngine );

    SO_ENGINE_ADD_INPUT( filename, ( "" ) );

    sceneGraph = NULL;
    sceneManager = new SoSceneManager();
    isBuiltIn = True;
};

void
SoDMBufferSceneEngine::composite(SbVec2s &size){
    if ( sceneGraph != NULL && inputImage.isConnected() ){
//  	glPushAttrib(GL_ENABLE_BIT);
	glEnable(GL_DEPTH_TEST);
	sceneManager->render(False);
//  	glPopAttrib();
    }
}

void 
SoDMBufferSceneEngine::stop( void ){
}

void 
SoDMBufferSceneEngine::start( void ){
}

void 
SoDMBufferSceneEngine::inputChanged(SoField *whichField){
    if (  sceneGraph == NULL &&
	  filename.getValue().getString() != "" &&
	  inputImage.isConnected() ){
	sceneGraph = readFile( filename.getValue().getString() );
	if ( sceneGraph != NULL ){
	    sceneGraph->ref();

	    SoCamera *camera;
	    // Look for camera...
	    {
		SoSearchAction sa;
		sa.setType(SoCamera::getClassTypeId());
		sa.apply(sceneGraph);

		SoPath *path = sa.getPath();
		if ( path == NULL ){
		    camera = new SoPerspectiveCamera;
		    sceneGraph->insertChild(camera,0);
		} else
		    camera = (SoCamera *) path->getTail();
	    }

	    SoGroup *lightGroup;
	    // Look for Headlight...
	    {
		SoSearchAction sa;
		sa.setType(SoLight::getClassTypeId());
		sa.apply(sceneGraph);

		SoPath *path = sa.getPath();
		if ( path == NULL ){
		    SoRotation *lightRotation = new SoRotation;
		    lightRotation->rotation.connectFrom(&camera->orientation);
		    SoDirectionalLight *light = new SoDirectionalLight();

		    lightGroup = new SoGroup;
		    lightGroup->addChild(lightRotation);
		    lightGroup->addChild(light);
		    lightGroup->addChild(new SoResetTransform());
		    sceneGraph->insertChild(lightGroup,1);
		}
	    }

	    // Setup ViewportRegion
	    SbVec2s size;
	    inputImage.startEditing(size);
	    inputImage.finishEditing();

	    sceneManager->setWindowSize( size );

	    camera->viewAll(sceneGraph, SbViewportRegion(size));
	    sceneManager->setSceneGraph(sceneGraph);

	    // disable events
	    sceneManager->setHandleEventAction(NULL);
	}
    }
    evaluate();
}

SoSeparator *
SoDMBufferSceneEngine::readFile(const char *filename){

    SoInput input;
    SbBool success;

    success = !input.openFile(filename);
    if ( success != 0 ){
	perror("Unable to open file\n");
	return NULL;
    }

    SoSeparator *root = SoDB::readAll(&input);

    if ( root == NULL ){
	perror("Unable to read file\n");
	return NULL;
    }
    input.closeFile();
    return root;
}
