// Inventor
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/elements/SoGLTextureEnabledElement.h>
#include <Inventor/elements/SoTextureQualityElement.h>
#include <Inventor/elements/SoTextureCoordinateElement.h>
#include <Inventor/elements/SoTextureMatrixElement.h>
#include "SoDMBufferTexture2.h"

SO_NODE_SOURCE(SoDMBufferTexture2);

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Constructor
//
// Use: public

void
SoDMBufferTexture2::initClass() {
    if (getClassTypeId().isBad())
	SoTexture2::initClass();

    SO_NODE_INIT_CLASS(SoDMBufferTexture2, SoTexture2, "SoTexture2");
}

SoDMBufferTexture2::SoDMBufferTexture2(): SoTexture2()
//
////////////////////////////////////////////////////////////////////////
{
    SO_NODE_CONSTRUCTOR(SoDMBufferTexture2);

    SO_NODE_ADD_FIELD(dmBufferImage, ( NULL ));
    SO_NODE_ADD_FIELD(transform, ( 1, 0, 0, 0,
				   0, 1, 0, 0,
				   0, 0, 1, 0,
				   0, 0, 0, 1, ));
    isBuiltIn = True;
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Destructor
//
// Use: private

SoDMBufferTexture2::~SoDMBufferTexture2()
//
////////////////////////////////////////////////////////////////////////
{
    SoTexture2::~SoTexture2();
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Performs typical action
//
// Use: extender

void
SoDMBufferTexture2::doAction(SoAction *action)
//
////////////////////////////////////////////////////////////////////////
{
    SoState *	state = action->getState();

    if (dmBufferImage.isIgnored() ||
	SoTextureOverrideElement::getImageOverride(state))
	return; // Texture being overriden or this node ignored
    if (isOverride()) {
	SoTextureOverrideElement::setImageOverride(state, TRUE);
    }
}

////////////////////////////////////////////////////////////////////////
//
// Description:
//    Performs GL rendering on a texture node.
//
// Use: extender

void
SoDMBufferTexture2::GLRender(SoGLRenderAction *action)
//
////////////////////////////////////////////////////////////////////////
{
    SoState *	state = action->getState();

    if (dmBufferImage.isIgnored() ||
	SoTextureOverrideElement::getImageOverride(state))
	return; // Texture being overriden or this node ignored
    if (isOverride()) {
	SoTextureOverrideElement::setImageOverride(state, TRUE);
    }

    SbDMBuffer *dmBuffer = dmBufferImage.getValue();

    if ( dmBuffer != NULL ){

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrapS.getValue());
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrapT.getValue());
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, model.getValue());
 	glBlendColorEXT( blendColor.getValue()[0],
 			 blendColor.getValue()[1],
 			 blendColor.getValue()[2],
 			 blendColor.getValue()[3] );

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	dmBuffer->defineTex();

//
//        SoTextureMatrixElement::translateBy(state, this, 
//                *transform.translation);
//        SoTextureMatrixElement::rotateBy(state, this,
//                *transform.rotation);
//        SoTextureMatrixElement::scaleBy(state, this,
//                *transform.translation);

	SoTextureMatrixElement::mult(state, this, 
				     transform.getValue());

//  	glMatrixMode(GL_TEXTURE);
//  	glLoadMatrixf(*transform.getValue().getValue());
//  	glMatrixMode(GL_MODELVIEW);
      
    	float texQuality = SoTextureQualityElement::get(state);
        if (texQuality == 0 || image.isIgnored()) {
	    SoGLTextureEnabledElement::set(state, FALSE);
	    return;
        }
        else {
	    SoGLTextureEnabledElement::set(state, TRUE);
        }	

//	SoGLTextureEnabledElement::set(state, TRUE);
//	SoTextureCoordinateElement::setDefault(state, this);
    }

}

