#include "SoDMBufferVideoEngine.h"

// Inventor
#include <Inventor/fields/SoSFMatrix.h>

SO_ENGINE_SOURCE( SoDMBufferVideoEngine );

void
SoDMBufferVideoEngine::initClass( void ){
    // Ensure Parent is Init'd!
    if (SoDMBufferEngine::getClassTypeId().isBad())
	SoDMBufferEngine::initClass();
    SO_ENGINE_INIT_CLASS( SoDMBufferVideoEngine, SoEngine, "Engine");
}
    
SoDMBufferVideoEngine::SoDMBufferVideoEngine( int _layout,
					      const char* _videoSource, 
					      int _originX,
					      int _originY,
					      int _zoomNumerator,
					      int _zoomDenominator,
					      int _aspectNumerator,
					      int _aspectDenominator,
					      int _width, 
					      int _height,
					      int _bufferCount): 
    SoDMBufferEngine()
{
    SO_ENGINE_CONSTRUCTOR( SoDMBufferVideoEngine );

    SO_ENGINE_ADD_INPUT( videoSource, ("") );

    SO_ENGINE_ADD_OUTPUT( textureBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( linearBits, SoSFDMBufferImage );
    SO_ENGINE_ADD_OUTPUT( scale, SoSFMatrix );

    
    layout = _layout;
    originX = _originX;
    originY = _originY;
    zoomNumerator = _zoomNumerator;
    zoomDenominator = _zoomDenominator;
    aspectNumerator = _aspectNumerator;
    aspectDenominator = _aspectDenominator;
    width = _width;
    height = _height;
    bufferCount = _bufferCount;

    isBuiltIn = True;

    VIDEO_LAYOUT = layout;

    videoIn = NULL;

    if (_videoSource != NULL){
      videoSource.setValue(_videoSource);
    }
}

void SoDMBufferVideoEngine::inputChanged(SoField *whichField){
  if (whichField != &videoSource){
    return;
  }
  
  int dmuErr, videoFD;
  Display *display = XOpenDisplay(":0.0");

  // Create dmu objects
  videoIn = new dmuVideoIn;
  videoIn->setName("video");
  
  videoContext = new dmuGLCtx;
  videoContext->setName("videoContext");
  
  // Create the dmu chain
  videoIn->addObject(videoContext);
  
  if ( VIDEO_LAYOUT == LAYOUT_LINEAR )
    texDrain = NULL;
  else{
    texDrain = new dmuGLTex2d;
    texDrain->setName("texture");
    videoContext->addObject(texDrain);
  }
  engineDMBuffer = new SbDMBuffer( videoContext, texDrain );
  
  
  int videoType= dmuVideo::getTypeByName((videoSource.getValue()).getString());
  if (videoType < 0){
    fprintf (stderr, "Requested video type unavailable.. Using default\n");
    videoType = VL_ANY;
  }

  dmuErr = videoIn->setup(videoType);
  if (dmuErr != DMU_OK){
    fprintf (stderr, "video setup error\n");
    exit (-1);
  }
  
  videoIn->setPacking(VL_PACKING_ABGR_8);
  videoIn->setCaptureType(VL_CAPTURE_INTERLEAVED);
  videoIn->setFormat(VL_FORMAT_RGB);
  videoIn->setZoomSize(width,height);
  videoIn->setOrigin(originX,originY);
  if ( VIDEO_LAYOUT == LAYOUT_LINEAR )
    videoIn->setLayout(VL_LAYOUT_LINEAR);
  else
    videoIn->setLayout(VL_LAYOUT_GRAPHICS);
  
  videoContext->setup(display,width,height);
  if ( VIDEO_LAYOUT == LAYOUT_GRAPHICS )
    texDrain->setup(display,width,height);
  
  dmuErr = videoIn->init();
  if (dmuErr != DMU_OK){
    fprintf (stderr,"video setup error\n");
    exit (-1);
  }
  
  float rx = (float) (width-originX)/(float) width;
  float ry = (float) (height-originY)/(float) height;
  
  float scale[4][4] = { rx,  0,  0,  0,
			0, ry,  0,  0,
			0,  0,  1,  0,
			0,  0,  0,  1 };
  
  scaleMatrix.setValue( scale );
  
  registerBurnItem( videoIn );
}

SoDMBufferVideoEngine::~SoDMBufferVideoEngine(){
    delete videoIn;
}

void
SoDMBufferVideoEngine::stop(){
    // XXX - Add to libdmu please
    /*
      if ( VIDEO_LAYOUT == LAYOUT_LINEAR ||
      VIDEO_LAYOUT == LAYOUT_BOTH )
      videoIn->stop();

      if ( VIDEO_LAYOUT == LAYOUT_GRAPHICS ||
      VIDEO_LAYOUT == LAYOUT_BOTH )
      videoIn->stop();
	*/
}

void
SoDMBufferVideoEngine::start(){
    // XXX - Add to libdmu please
    /*
      if ( VIDEO_LAYOUT == LAYOUT_LINEAR ||
      VIDEO_LAYOUT == LAYOUT_BOTH )
      videoIn->start();

      if ( VIDEO_LAYOUT == LAYOUT_GRAPHICS ||
      VIDEO_LAYOUT == LAYOUT_BOTH )
      videoIn->start();
	*/
}
