// Inventor
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCallback.h>

// DMUIV
#include "libdmuiv/SoDMBufferTexture2.h"
#include "libdmuiv/SoDMBufferVideoEngine.h"

// Prototypes
SoSeparator *readFile( const char * );
void renderCallback(void *, SoAction *action);

// Global Variables

  // Inventor
  static SoSeparator *childSceneGraph;
  SoXtExaminerViewer *viewer;
  SoPerspectiveCamera *camera;
  SoDMBufferTexture2 *texture;
  Widget slvWindow;

void 
main(int argc, char *argv[]){
    
    if( argc != 2 ){
	perror("Expected an Inventor Filename.\n");
	exit(2);
    }

    slvWindow = SoXt::init( argv[0] );
    if ( slvWindow == NULL ){
	perror("Unable to create main Inventor window.\n");
	exit(2);
    }

    SoDMBufferTexture2::initClass();
    SoSFDMBufferImage::initClass();
    SoDMBufferEngine::initClass();
    SoDMBufferVideoEngine::initClass();

    viewer = new SoXtExaminerViewer( slvWindow );

    SoSeparator *root = new SoSeparator;
    root->ref();

    camera = new SoPerspectiveCamera;
    root->addChild( camera );

    SoCallback *renderCB = new SoCallback;
    renderCB->setCallback(renderCallback);
    root->addChild(renderCB);

    texture = new SoDMBufferTexture2();
    root->addChild(texture);

    SoDMBufferVideoEngine *videoIn = new SoDMBufferVideoEngine;
    texture->dmBufferImage.connectFrom( &videoIn->textureBits );
    texture->transform.connectFrom( &videoIn->scale );

    childSceneGraph = readFile( argv[1] );
    root->addChild( childSceneGraph );

    camera->viewAll( root, viewer->getViewportRegion() );
    viewer->setSceneGraph( root );
    viewer->setTitle( argv[0] );
    viewer->show();

    SoXt::show(slvWindow);
    SoXt::mainLoop();

}

void renderCallback(void *, SoAction *action){
    if (!action->isOfType(SoGLRenderAction::getClassTypeId()))
	return;

    static GLXDrawable ivDrawable = glXGetCurrentDrawable();
    static GLXContext  ivContext  = glXGetCurrentContext();

    static Bool grabbed = False;

    SbVec2s size;
    if ( grabbed )
	texture->dmBufferImage.finishEditing();

    texture->dmBufferImage.startEditing(size);
    SbVec2s vprSize = size;
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(0,1,0,1);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glViewport(0, 0, vprSize[0], vprSize[1]);

    glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
    glColor3f(1.0f,0.0f,0.0f);
    glDisable(GL_DEPTH_TEST);

    glRectf(0.25,0.25,0.75,0.75);

    glPopAttrib();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    
    grabbed = True;

    glXMakeCurrent(XtDisplay(slvWindow),ivDrawable,ivContext);
}

///////////
//
// readFile
//
//   Constructs a SceneGraph given a filename.
//
///////////
SoSeparator *
readFile(const char *filename){

    SoInput input;
    SbBool success;

    success = !input.openFile(filename);
    if ( success != 0 ){
	perror("Unable to open file\n");
	return NULL;
    }

    SoSeparator *root = SoDB::readAll(&input);

    if ( root == NULL ){
	perror("Unable to read file\n");
	return NULL;
    }
    input.closeFile();
    return root;
}
