// Inventor
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/actions/SoWriteAction.h>

// DMUIV
#include "libdmuiv/SoDMBufferTexture2.h"
#include "libdmuiv/SoDMBufferVideoEngine.h"
#include "libdmuiv/SoDMBufferMovieEngine.h"
#include "libdmuiv/SoDMBufferRawDVEngine.h"
#include "libdmuiv/SoDMBuffer1394Engine.h"
#include "libdmuiv/SoDMBufferBackground.h"
#include "libdmuiv/SoDMBufferSceneEngine.h"

// Prototypes
SoSeparator *readFile( const char * );

void 
main(int argc, char *argv[]){
    
    Widget slvWindow;

    if( argc != 2 ){
	perror("Expected an Inventor Filename.\n");
	exit(2);
    }

    slvWindow = SoXt::init( argv[0] );
    if ( slvWindow == NULL ){
	perror("Unable to create main Inventor window.\n");
	exit(2);
    }

    SoDMBufferTexture2::initClass();
    SoSFDMBufferImage::initClass();
    SoDMBufferMovieEngine::initClass();
    SoDMBufferRawDVEngine::initClass();
    SoDMBuffer1394Engine::initClass();
    SoDMBufferVideoEngine::initClass();
    SoDMBufferSceneEngine::initClass();
    SoDMBufferBackground::initClass();

    SoXtExaminerViewer *viewer;
    SoPerspectiveCamera *camera = new SoPerspectiveCamera;

    viewer = new SoXtExaminerViewer( slvWindow );
    viewer->setTransparencyType(SoGLRenderAction::BLEND);

    SoSeparator *root = readFile( argv[1] );
    if ( root != NULL ){
	root->ref();

	camera->viewAll( root, viewer->getViewportRegion() );
	viewer->setSceneGraph( root );
	viewer->setTitle( argv[0] );
	viewer->show();

	SoXt::show(slvWindow);
	SoXt::mainLoop();
    }

}

///////////
//
// readFile
//
//   Constructs a SceneGraph given a filename.
//
///////////
SoSeparator *
readFile(const char *filename){

    SoInput input;
    SbBool success;

    success = !input.openFile(filename);
    if ( success != 0 ){
	perror("Unable to open file\n");
	return NULL;
    }

    SoSeparator *root = SoDB::readAll(&input);

    if ( root == NULL ){
	perror("Unable to read file\n");
	return NULL;
    }
    input.closeFile();
    return root;
}
