// Inventor
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/sensors/SoTimerSensor.h>

// DMUIV
#include "libdmuiv/SoDMBufferTexture2.h"
#include "libdmuiv/SoDMBufferVideoEngine.h"

// Prototypes
void renderCallback(void *, SoAction *action);

// Global Variables

  // Inventor
  SoXtExaminerViewer *viewer;
  SoPerspectiveCamera *camera;
  Widget slvWindow;
  SoSFDMBufferImage *linearImage;
  SoDMBufferTexture2 *texture;

void 
main(int argc, char *argv[]){
    
    slvWindow = SoXt::init( argv[0] );

    SoSFDMBufferImage::initClass();
    SoDMBufferEngine::initClass();
    SoDMBufferVideoEngine::initClass();

    viewer = new SoXtExaminerViewer( slvWindow );

    SoSeparator *root = new SoSeparator;
    root->renderCaching = SoSeparator::OFF;
    root->ref();

    camera = new SoPerspectiveCamera;
    root->addChild( camera );

    SoCallback *renderCB = new SoCallback;
    renderCB->setCallback(renderCallback);
    root->addChild(renderCB);

    SoDMBufferVideoEngine *videoIn = new SoDMBufferVideoEngine(SoDMBufferVideoEngine::LAYOUT_LINEAR);
    texture = new SoDMBufferTexture2;

    linearImage = new SoSFDMBufferImage;
    linearImage->connectFrom( &videoIn->linearBits );
    texture->dmBufferImage.connectFrom( &videoIn->textureBits );

    root->addChild( texture );
    root->addChild( new SoCube());

    camera->viewAll( root, viewer->getViewportRegion() );
    viewer->setSceneGraph( root );
    viewer->setTitle( argv[0] );
    viewer->show();

    SoXt::show(slvWindow);
    SoXt::mainLoop();

}

void renderCallback(void *, SoAction *action){
    if (!action->isOfType(SoGLRenderAction::getClassTypeId()))
	return;

    static GLXDrawable ivDrawable = glXGetCurrentDrawable();
    static GLXContext  ivContext  = glXGetCurrentContext();

    static Bool grabbed = False;

    static const unsigned char *pixels;

    SbVec2s size;
    if ( grabbed )
	linearImage->release();

    glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT);

    pixels = linearImage->getImage(size);
    glPixelTransferf(GL_RED_SCALE, 1.0);    // Default
    glPixelTransferf(GL_GREEN_SCALE, 1.0);  // Default
    glPixelTransferf(GL_BLUE_SCALE, 1.0);   // Default
    glPixelTransferf(GL_ALPHA_SCALE, 1.0);  // Default
    glPixelTransferf(GL_RED_BIAS, 0.0);     // Default
    glPixelTransferf(GL_GREEN_BIAS, 0.0);   // Default
    glPixelTransferf(GL_BLUE_BIAS, 0.0);    // Default
    glPixelTransferf(GL_ALPHA_BIAS, 0.0);   // Default

    glDisable(GL_TEXTURE_1D);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glDisable(GL_ALPHA_TEST);
    glDisable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(0,1,0,1);
      
    SbVec2s vprSize = viewer->getViewportRegion().getViewportSizePixels();
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glViewport(0, 0, vprSize[0], vprSize[1]);

    glRasterPos2d(0,1);
    glPixelZoom(1,-1);
    glDrawPixels(size[0],size[1],GL_RGBA,GL_UNSIGNED_BYTE,pixels);

    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);

    glPopAttrib();

    grabbed = True;
}
