// Inventor
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/actions/SoWriteAction.h>

// DMUIV
#include "libdmuiv/SoDMBufferTexture2.h"
#include "libdmuiv/SoDMBufferMovieEngine.h"
#include "libdmuiv/SoDMBufferBackground.h"
#include "libdmuiv/SoDMBufferSceneEngine.h"

// Prototypes
SoSeparator *readFile( const char * );

// Global Variables

  // Inventor
  static SoSeparator *childSceneGraph;
  SoXtExaminerViewer *viewer;
  SoPerspectiveCamera *camera;
  Widget slvWindow;

void 
main(int argc, char *argv[]){
    
    if( argc != 3 ){
	perror("Expected an Inventor and Movie Filename.\n");
	exit(2);
    }

    slvWindow = SoXt::init( argv[0] );
    if ( slvWindow == NULL ){
	perror("Unable to create main Inventor window.\n");
	exit(2);
    }

    SoDMBufferTexture2::initClass();
    SoSFDMBufferImage::initClass();
    SoDMBufferMovieEngine::initClass();
    SoDMBufferSceneEngine::initClass();
    SoDMBufferBackground::initClass();

    viewer = new SoXtExaminerViewer( slvWindow );
    viewer->setBackgroundColor(SbColor(255,255,255));

    SoSeparator *root = new SoSeparator;
    root->ref();

    camera = new SoPerspectiveCamera;
    root->addChild( camera );

    SoDMBufferTexture2 *texture = new SoDMBufferTexture2();
    root->addChild(texture);

    SoDMBufferMovieEngine *movieIn = new SoDMBufferMovieEngine();
    movieIn->filename.setValue(argv[2]);
    texture->dmBufferImage.connectFrom( &movieIn->textureBits );
    texture->transform.connectFrom( &movieIn->scale );

    root->addChild( new SoDMBufferBackground() );

    childSceneGraph = readFile( argv[1] );
    root->addChild( childSceneGraph );

    camera->viewAll( root, viewer->getViewportRegion() );
    viewer->setSceneGraph( root );
    viewer->setTitle( argv[0] );
    viewer->show();

    SoWriteAction wa;
    wa.apply(root);

    SoXt::show(slvWindow);
    SoXt::mainLoop();

}

///////////
//
// readFile
//
//   Constructs a SceneGraph given a filename.
//
///////////
SoSeparator *
readFile(const char *filename){

    SoInput input;
    SbBool success;

    success = !input.openFile(filename);
    if ( success != 0 ){
	perror("Unable to open file\n");
	return NULL;
    }

    SoSeparator *root = SoDB::readAll(&input);

    if ( root == NULL ){
	perror("Unable to read file\n");
	return NULL;
    }
    input.closeFile();
    return root;
}
