/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/uio.h>
#include <sys/schedctl.h>
#include "handy.h"


/*
 * set non-degrading priority
 * use high priority if running as root
 */
void setscheduling()
{
    static int tell = 1;
    /*
     * swap permissions so that we become root for a moment.
     * setreuid() should let us set real uid to effective uid = root
     * if the program is setuid root
     *
     */
    setreuid(geteuid(), getuid());
    setregid(getegid(), getgid());

    if ((schedctl(NDPRI, 0, NDPHIMIN)< 0) && tell) {
         fprintf(stderr, "Run as root to enable real time scheduling\n");
	 tell = 0;
    }

    /* swap permissions back, now we're just "joe user" */
    setreuid(geteuid(), getuid());
    setregid(getegid(), getgid());

}

void setcpu(int cpu_num)

{
    static int tell = 1;
    /*
     * swap permissions so that we become root for a moment.
     * setreuid() should let us set real uid to effective uid = root
     * if the program is setuid root
     *
     */
    setreuid(geteuid(), getuid());
    setregid(getegid(), getgid());

    if ((sysmp(MP_MUSTRUN, cpu_num) < 0) && tell) {
         fprintf(stderr, "setcpu() - Run as root to enable cpu selection\n");
         tell = 0;
    }

    /* swap permissions back, now we're just "joe user" */
    setreuid(geteuid(), getuid());
    setregid(getegid(), getgid());

}
