/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <dmedia/cl.h>
#include <dmedia/cl_impactcomp.h>
#include "common/handy.h"

CLhandle clInitDecompressor(int u, int v, int winx, int winy, int qf, int rgb)
{
    int paramBuf[64];
    int i, j,  n;
    CLhandle decompressor;

    n = 0;
    paramBuf[n++] = CL_INTERNAL_IMAGE_WIDTH;
    paramBuf[n++] = u;
    paramBuf[n++] = CL_INTERNAL_IMAGE_HEIGHT;
    paramBuf[n++] = v;
    paramBuf[n++] = CL_IMAGE_WIDTH;
    paramBuf[n++] = winx;
    paramBuf[n++] = CL_IMAGE_HEIGHT;
    paramBuf[n++] = winy;
    paramBuf[n++] = CL_FORMAT;
    paramBuf[n++] = (rgb) ? CL_RGBX : CL_YUV422;
    paramBuf[n++] = CL_INTERNAL_FORMAT;
    paramBuf[n++] = CL_YUV422;
    paramBuf[n++] = CL_IMPACT_BUFFER_ADVISE;
    paramBuf[n++] = CL_BUFFER_ADVISE_NOACCESS;

    CC (clOpenDecompressor(CL_JPEG_IMPACT, &decompressor));
    CC (clSetParams(decompressor, paramBuf, n));

    if (qf >= 101) {
	CC (clSetParam(decompressor, CL_IMPACT_LOSSLESS_JPEG, TRUE));
    }

    return decompressor;
}

CLhandle clInitCompression(CLhandle compressor, int u, int v, int qf)
{
    int n;
    int paramBuf[64];

    n = 0;
    paramBuf[n++] = CL_IMAGE_WIDTH;
    paramBuf[n++] = u;
    paramBuf[n++] = CL_IMAGE_HEIGHT;
    paramBuf[n++] = v; 
    paramBuf[n++] = CL_FORMAT;
    paramBuf[n++] = CL_YUV422;
    paramBuf[n++] = CL_ENABLE_IMAGEINFO;
    paramBuf[n++] = TRUE;

    if (compressor == NULL) {
        CC (clOpenCompressor(CL_JPEG_IMPACT, &compressor));
    }
    CC (clSetParams(compressor, paramBuf, n));

    if (qf < 101) {
	CC (clSetParam(compressor, CL_JPEG_QUALITY_FACTOR, qf));
        fprintf(stderr, "Setting qf = %d\n", qf);
    }
    else if (qf >= 101) {
	CC (clSetParam(compressor, CL_IMPACT_LOSSLESS_JPEG, TRUE));
        fprintf(stderr, "Doing lossless\n");
    }

    return compressor;
}
