/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

void expandyuv(int pixels, unsigned char *syuvBuffer, unsigned char *srgbBuffer)
{
    unsigned char *bufferEnd = srgbBuffer + pixels * 4;
    register unsigned char *yuvBuffer = syuvBuffer;
    register unsigned char *rgbBuffer = srgbBuffer;
    register unsigned char tmp0, tmp1, tmp2, tmp3;

    /* Matrix wants Cr (V), then Y, then Cb (U). */
    /* Stored as Cb(U), Y, Cr(V)		 */
    /* First byte being Cr implies a GL_RGBA type for pixel drawing*/

   /*
    * This corrsponds to:
    *
    * R = Cr*colorMat[0] + Y*colorMat[4] + Cb*colorMat[8]  + alpha*colorMat[12];
    * G = Cr*colorMat[1] + Y*colorMat[5] + Cb*colorMat[9]  + alpha*colorMat[13];
    * B = Cr*colorMat[2] + Y*colorMat[6] + Cb*colorMat[10] + alpha*colorMat[14];
    * A = Cr*colorMat[3] + Y*colorMat[7] + Cb*colorMat[11] + alpha*colorMat[15];
    */

    while (rgbBuffer < bufferEnd) {

        /* First pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */
        *rgbBuffer++ = 255;    /* alpha  */ 

        /* Second pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 
        *rgbBuffer++ = 255;    /* alpha  */ 

        /* Third pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */
        *rgbBuffer++ = 255;    /* alpha  */ 

        /* Fourth pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 
        *rgbBuffer++ = 255;    /* alpha  */ 


        /* Fifth pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */
        *rgbBuffer++ = 255;    /* alpha  */ 

        /* Sixth pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 
        *rgbBuffer++ = 255;    /* alpha  */ 

        /* Seventh pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */
        *rgbBuffer++ = 255;    /* alpha  */ 

        /* Eigth pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 
        *rgbBuffer++ = 255;    /* alpha  */ 
    }
}

void expandyuv_p(int pixels, unsigned char *syuvBuffer, unsigned char *srgbBuffer)
{
    unsigned char *bufferEnd = srgbBuffer + pixels * 3;
    register unsigned char *yuvBuffer = syuvBuffer;
    register unsigned char *rgbBuffer = srgbBuffer;
    register unsigned char tmp0, tmp1, tmp2, tmp3;

    /* Matrix wants Cr (V), then Y, then Cb (U). */
    /* Stored as Cb(U), Y, Cr(V)		 */
    /* First byte being Cr implies a GL_RGBA type for pixel drawing*/

   /*
    * This corrsponds to:
    *
    * R = Cr*colorMat[0] + Y*colorMat[4] + Cb*colorMat[8]  + alpha*colorMat[12];
    * G = Cr*colorMat[1] + Y*colorMat[5] + Cb*colorMat[9]  + alpha*colorMat[13];
    * B = Cr*colorMat[2] + Y*colorMat[6] + Cb*colorMat[10] + alpha*colorMat[14];
    * A = Cr*colorMat[3] + Y*colorMat[7] + Cb*colorMat[11] + alpha*colorMat[15];
    */

    while (rgbBuffer < bufferEnd) {

        /* First pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */

        /* Second pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 

        /* Third pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */

        /* Fourth pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 


        /* Fifth pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */

        /* Sixth pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 

        /* Seventh pixel */
        tmp0 = *yuvBuffer++; 
        tmp1 = *yuvBuffer++; 
        tmp2 = *yuvBuffer++; 
        tmp3 = *yuvBuffer++; 

        *rgbBuffer++ = tmp2;   /* Grab Cr = V */ 
	*rgbBuffer++ = tmp1;   /* Grab Y */
        *rgbBuffer++ = tmp0;   /* Grab Cb = U */

        /* Eigth pixel */
        *rgbBuffer++ = tmp2;   /* old Cr = V */
	*rgbBuffer++ = tmp3;   /* New Y  */
        *rgbBuffer++ = tmp0;   /* old Cb = U */ 
    }
}
