/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "handy.h"
#include "jinfo.h"
#include "fileutils.h"

fileHeader_t *fileCreate(char *fname, int direct) 
{
    fileHeader_t *f;

    f = (fileHeader_t *) memalign(sizeof(fileHeader_t), sizeof(fileHeader_t)); 
    memset(f, 0x0, sizeof(fileHeader_t));

    OC (f->fd = open(fname, O_CREAT | O_RDWR, 0666 ));

    lseek64(f->fd, (off64_t)0, SEEK_SET);
    if (direct) {
	OC(fcntl(f->fd, F_SETFL, FDIRECT));
    }

    f->headerSize = sizeof(fileHeader_t);
    OC (write(f->fd, f, sizeof(fileHeader_t)));
    
    fsync(f->fd);
    return f;
}

void fileInit(char *fname, fileHeader_t *f, int direct) 
{
    int fd;

    OC (fd = open(fname, O_RDONLY ));
    lseek64 (fd, (off64_t)0, SEEK_SET);

    OC (read(fd, f, sizeof(fileHeader_t)));
    f->fd = fd;

    if (direct) {
	OC(fcntl(fd, F_SETFL, FDIRECT));
    }
}


off64_t fileDataBytes(fileHeader_t *f)
{
    return f->dataBytes; 
}

void writeHeader(fileHeader_t *f)
{
    /* Write out the correct header info.       */
    lseek64 (f->fd, (off64_t)0, SEEK_SET);

    if (f->compressed) {
	f->indexOffset = f->headerSize + f->dataBytes; 
    }
    else {
	f->dataBytes = (off64_t)(f->readSize) * (off64_t)(f->Nfields);
    }

    OC (write(f->fd, f, sizeof(fileHeader_t)));
}

void writeIndexBuffer(fileHeader_t *f, jFieldInfo_t *indexbuffer)
{
    int indexBytes;

    indexBytes = (f->Nfields + 1) * sizeof(jFieldInfo_t);    
    lseek64(f->fd, f->indexOffset, SEEK_SET);
    OC (write(f->fd, indexbuffer, indexBytes));
}

void readIndexBuffer(fileHeader_t *f, jFieldInfo_t *indexbuffer)
{
    int indexBytes;

    indexBytes = (f->Nfields + 1) * sizeof(jFieldInfo_t);
    lseek64(f->fd, f->indexOffset, SEEK_SET);
    OC (read(f->fd, indexbuffer, indexBytes));
}

jFieldInfo_t *createIndexBuffer(int nfields)
{
    int indexBytes;

    indexBytes = (nfields + 1) * sizeof(jFieldInfo_t);
    return (jFieldInfo_t *) malloc(indexBytes);
}
