/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <GL/GLwMDrawA.h>
#include <GL/gl.h>
#include "handy.h"

void do_single(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int count = 0;
    char *sourceBuffer1, *sourceBuffer2;

    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);

    while (*playing != ec) {
	GetInputFields (&sourceBuffer1, &sourceBuffer2);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);
        count++;
        SendToVideo (count);
    }

}

void do_blend(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    long x, y;
    int count = 0;
    float bf = 0;
    float bf_inc;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    bf_inc = .01;
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_CONSTANT_ALPHA_EXT,GL_ONE_MINUS_CONSTANT_ALPHA_EXT);
    glBlendEquationEXT(GL_FUNC_ADD_EXT);
    glLoadIdentity();

    while (*playing != ec) {
	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glDisable(GL_BLEND);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);

        glEnable(GL_BLEND);
        glBlendColorEXT(0.0, 0.0, 0.0, bf);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);

        count++;
        SendToVideo (count);
    
        bf += bf_inc;
        if ((bf > .995) || (bf < 0.005) ) {
	    bf_inc = -bf_inc;
        } 
    }

}

void do_wipe(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    long x, y;
    int count = 0;
    float bf = 0;
    float bf_inc;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glLoadIdentity();
    glDisable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);
    bf_inc = .01;

    while (*playing != ec) {
        GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glDisable(GL_TEXTURE_2D);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);
        glEnable(GL_TEXTURE_2D);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);
        glTranslatef(bf_inc, 0.0, 0.0);
        glCallList(1);

        count++;
        SendToVideo (count);

        bf += bf_inc;
        if ((bf > 1.995) || (bf < -1.995) ) {
	    bf_inc = -bf_inc;
        }
    }

}

void do_pip(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    long x, y;
    int count = 0;
    float bf = 0;
    float bf_inc;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glDisable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);
    bf_inc = .01;
    glLoadIdentity();
    glScalef(0.5, 0.5, 0.5);

    while (*playing != ec) {

	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glDisable(GL_TEXTURE_2D);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);
        glEnable(GL_TEXTURE_2D);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);
        glTranslatef(bf_inc, 0.0, 0.0);
        glCallList(1);

        count++;
        SendToVideo (count);

        bf += bf_inc;
        if ((bf > 1.995) || (bf < -1.995) ) {
	    bf_inc = -bf_inc;
        }
    }

}

void do_cross(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    int count = 0;
    long x, y;
    float bf = 0;
    float bf_inc;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glDisable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);
    bf_inc = .01;
    glLoadIdentity();

    while (*playing != ec) {

	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glDisable(GL_TEXTURE_2D);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);

        glEnable(GL_TEXTURE_2D);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);

        glLoadIdentity();
	glTranslatef(-bf, -bf, 0.0);
        glCallList(2);

        glLoadIdentity();
	glTranslatef(bf, -bf, 0.0);
        glCallList(3);

        glLoadIdentity();
	glTranslatef(bf, bf, 0.0);
        glCallList(4);

        glLoadIdentity();
	glTranslatef(-bf, bf, 0.0);
        glCallList(5);

        count++;
        SendToVideo (count);

        bf += bf_inc;
        if ((bf > 1.0) || (bf < -1.0) ) {
	    bf_inc = -bf_inc;
        }
    }

}

void do_boxblend(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    int count = 0;
    long x, y;
    float bf = 0;
    float bf_inc;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);
    glEnable(GL_BLEND);
    glBlendFunc(GL_CONSTANT_ALPHA_EXT,GL_ONE_MINUS_CONSTANT_ALPHA_EXT);
    glBlendEquationEXT(GL_FUNC_ADD_EXT);

    bf_inc = .01;
    glLoadIdentity();

    while (*playing != ec) {

	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glDisable(GL_BLEND);
        glDisable(GL_TEXTURE_2D);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);

        glEnable(GL_TEXTURE_2D);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);

	glEnable(GL_BLEND);

	glBlendColorEXT(0.0, 0.0, 0.0, bf);
        glCallList(2);

	glBlendColorEXT(0.0, 0.0, 0.0, 1.0 - bf);
        glCallList(3);

	glBlendColorEXT(0.0, 0.0, 0.0, bf);
        glCallList(4);

	glBlendColorEXT(0.0, 0.0, 0.0, 1.0 - bf);
        glCallList(5);

        count++;
        SendToVideo (count);

        bf += bf_inc;
        if ((bf > .995) || (bf < 0.005) ) {
	    bf_inc = -bf_inc;
        } 
    }

}

void do_squeeze(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    int count = 0;
    long x, y;
    float bf = 0;
    float bf_inc;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);
    glDisable(GL_BLEND);
    glLoadIdentity();
    bf_inc = .75;

    while (*playing != ec) {

	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glDisable(GL_TEXTURE_2D);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);

        glEnable(GL_TEXTURE_2D);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);
        glRotatef(bf_inc, 1.0, 0.0, 0.0);
        glCallList(1);

        count++;
        SendToVideo (count);
    }

}


void do_perspective(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    int count = 0;
    long x, y;
    float bf = 0;
    float bf_inc;
    float twist = 0;
    float twist_inc;
    float near = .5;
    float trans =  -.95;
    float far  = 2.0; 
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);
    glDisable(GL_BLEND);
    bf_inc = .20;
    twist_inc = .21;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-0.50, 0.50, -0.50, 0.50, near, far);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    while (*playing != ec) {

	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glClear(GL_COLOR_BUFFER_BIT);
        glDisable(GL_TEXTURE_2D);
        glDrawPixels(uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);

        glEnable(GL_TEXTURE_2D);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);
        glLoadIdentity();

        glRotatef(bf, 0.0, 1.0, 0.0);
        bf += bf_inc;

        if (count > 100) {
	    glRotatef(twist, 1.0, 0.0, 0.0);
	    twist += twist_inc;
	}

        glTranslatef(0.0, 0.0, trans);
        glCallList(1);

        count++;
        SendToVideo (count);
    }

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

}

void do_depth(int uSize, int vSize, int tex_width, int tex_height, int tex_format, int tex_name, int *playing, int ec)
{
    int i, val;
    int count = 0;
    long x, y;
    float bf = 0;
    float bf_inc = .125;
    float t1 = 0;
    float t2 = 0;
    float t1_inc = .0035;
    float t2_inc = .0060;
    float near = .5;
    float trans =  -0.75;
    float far  = 2.0; 
    float flip = 1.0;
    char *sourceBuffer1, *sourceBuffer2;

    glClear(GL_COLOR_BUFFER_BIT);
    x = tex_width  - uSize;
    y = tex_height - vSize;

    glEnable(GL_TEXTURE_2D);
    glBindTextureEXT(GL_TEXTURE_2D,tex_name);

    glDisable(GL_BLEND);

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    glClearDepth(1.0);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-0.70, 0.70, -0.70, 0.70, near, far);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    while (*playing != ec) {

	GetInputFields (&sourceBuffer1, &sourceBuffer2);

        glClear(GL_COLOR_BUFFER_BIT);
        glClear(GL_DEPTH_BUFFER_BIT);

        glLoadIdentity();
        glTranslatef(0.0, 0.0, -1.75);
        glTranslatef(0.0, 0.0, t1);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer2);
        glCallList(1);

        glLoadIdentity();
        glTranslatef(0.0, 0.0, trans);
        glTranslatef(0.0, 0.0, t2);

        glRotatef(bf, 1.0, 0.0, 0.0);
        glTexSubImage2DEXT(GL_TEXTURE_2D, 0, x, y, uSize, vSize, tex_format, GL_UNSIGNED_BYTE, sourceBuffer1);
        glCallList(1);

        count++;
        SendToVideo (count);

        i = count % 600;
        if (i == 0) flip = -flip;

        if (i < 275) {
	     bf += bf_inc*flip;
        }
        else {
	     t2 -= t2_inc*flip;
	     t1 += t1_inc*flip;
        }

    }

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glDisable(GL_DEPTH_TEST);

}
