/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <dmedia/vl.h>
#include "handy.h"

void SetVideoOutTiming(VLServer svr, VLPath path, VLNode node, int width, int height)
{
    /* Little routine to set the correct timing for video out.
     *  720 x 288        = VL_TIMING_625_CCIR601
     *  720 x 240 or 243 = VL_TIMING_525_CCIR601
     *  640 x 240 or 243 = VL_TIMING_525_SQ_PIX
     *  768 x 576        = VL_TIMING_625_SQ_PIX
     */

    VLControlValue val;

    if (width == 720) {
         val.intVal = (height == 288) ? VL_TIMING_625_CCIR601 : VL_TIMING_525_CCIR601;
    }
    else {
         val.intVal = (width == 640) ? VL_TIMING_525_SQ_PIX : VL_TIMING_625_SQ_PIX;
    }
    VC (vlSetControl(svr, path, node, VL_TIMING, &val));
}

int GetVLFormat(int packing)
{
    int format;

    switch (packing) {
	case VL_PACKING_ABGR_8:
	case VL_PACKING_BGR_8_P:
	    format = VL_FORMAT_RGB;
            break;
	case VL_PACKING_YVYU_422_8:
	case VL_PACKING_UYV_8_P:
            format = VL_FORMAT_DIGITAL_COMPONENT_SERIAL;
	    break;
	default:
	    ERROR(1, "Unsupported packing"); 
	    break;
    }
    return format;
}
