/*
 * Copyright (c) 1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef handy_h
#define handy_h

/**********************************************************************
*
* Handy error macros
*	Taken from /usr/share/src/dmedia/dmrecord/dmrecord.dmic
*
**********************************************************************/

#ifdef __ANSI_CPP__
#define PRINT_CALL(call)   fprintf( stderr, "%s failed in file %s, line %d\n", # call, __FILE__, __LINE__ );
#else
#define PRINT_CALL(call)   fprintf( stderr, "%s failed in file %s, line %d\n", "call", __FILE__, __LINE__ );
#endif

#define ERROR(x, message)							\
    {										\
	if ( (x) ) {								\
            fprintf( stderr, "%s\n", message );					\
            fprintf( stderr, "in file %s, line %d\n", __FILE__, __LINE__);	\
	    exit(EXIT_FAILURE);							\
	}									\
    }										\

/*
** CC - "Compression Check".  Wraps a CL call that returns <0 on error.
*/

#define CC(call)							      \
    {									      \
	if ( (call) < 0 ) {						      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** CNC -- "Compression NULL Check".  Wraps a CL call that returns NULL
** on error.
*/

#define CNC(call)							      \
    {									      \
	if ( (call) == NULL ) {						      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** VC - "Video Check".  Wraps a VL call that returns <0 on error.
*/

#define VC(call)							      \
    {									      \
	if ( (call) < 0 ) {						      \
	    fprintf( stderr, "%s\n", vlStrError(vlGetErrno()) );\
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** VNC -- "Video NULL Check".  Wraps a VL call that returns NULL
** on error.
*/

#define VNC(call)							      \
    {									      \
	if ( (call) == NULL ) {						      \
	    fprintf( stderr, "%s\n", vlStrError(vlGetErrno()) );\
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** GC - "GLX  Check".  Wraps a glX call that returns False on error.
*/

#define GC(call)							      \
    {									      \
	if ( (call) == False ) {					      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** GNC - "GLX NULL Check".  Wraps a glX call that returns NULL
*/

#define GNC(call)							      \
    {									      \
	if ( (call) == NULL ) {						      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** AC - "Audio Check".  Wraps an AL call that returns <0 on error.
*/

#define AC(call)							      \
    {									      \
	if ( (call) < 0 ) {						      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** ANC -- "Audio NULL Check".  Wraps an AL call that returns NULL
** on error.
*/

#define ANC(call)							      \
    {									      \
	if ( (call) == NULL ) {						      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** DC -- "DM Params Check".  Wraps a call to a dmParams function that
** returns DM_FAILURE on error.
*/

#define DC(call)							      \
    {									      \
	if ( (call) != DM_SUCCESS ) {					      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** MC -- "Movie Check".  Wraps a movie library call that returns
** DM_FAILURE on error.
*/

#define MC(call)							      \
    {									      \
	if ( (call) != DM_SUCCESS ) {					      \
	    fprintf( stderr, "%s\n", mvGetErrorStr(mvGetErrno()) );\
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** OC -- "OS Check".  Wraps a Unix call that returns negative on error.
*/

#define OC(call)							      \
    {									      \
	if ( (call) < 0 ) {						      \
	    perror( "" );					      \
	    PRINT_CALL(call);						      \
            exit( EXIT_FAILURE );					      \
	}								      \
    }									      \

/*
** OCW -- "OS Check Warn".  	Like OC, but prints warning and continues 
**				instead of exiting.
*/

#define OCW(call)							      \
    {									      \
	if ( (call) < 0 ) {						      \
	    fprintf( stderr, " \nWarning: " );				      \
	    perror( "" );					      \
	    PRINT_CALL(call);						      \
	}								      \
    }									      \

#define EVEN(foo) 					\
	( ( (foo/2)*2 == foo )? 1 : 0 )


/* microseconds must be < 1000000 */
#define NAP(seconds,microseconds) \
      {\
          struct timeval tmo;\
          tmo.tv_sec = seconds;\
          tmo.tv_usec = microseconds;\
          select(0, NULL, NULL, NULL, &tmo);\
      }

#endif
