//
//  dmuNetwork.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// <dmu>
// This object will allow you go use the network with libdmu.  This is
// almost entirely a virtual object.  You will never be able to use
// this object directly.  You must use a subclass of this object.  
//
// </dmu>
//
#ifndef _DMUNETWORK_H
#define _DMUNETWORK_H

// includes needed for this object.  
#include "dmuObj.h"
#include "pentCommon.h"
#include "pentData.h"
#include <ulocks.h>

class dmuNetwork : public dmuObj {
public:
  // Constructor: This constructor sets up some default values. and
  // creates a new semaphore
  dmuNetwork();
  // Destructor:  This destructor does nothing beyond freeing up the
  // semaphore we have been using.
  ~dmuNetwork();

  // Tell the object the name of the remote machine
  void setHostname(char *_hostname)     { hostname = _hostname; }
  // Tell the object the port being used on the remote machine
  void setPort(int _port)               { port     = _port; }

  // This method doesn't do anything.  It's a place holder in case it
  // is necessary later.
  virtual int setup();

  // Subclasses of this object will override this method.  
  virtual int init(dmuParams *params = NULL);

  // This will start the object running, most subclasses will override
  // this method.  
  virtual int start( int propagate = DMU_PROPAGATE);
  // Use this method to stop the object from sending things down the chain
  virtual int stop();

  // returns an fd that you can use in the main event loop
  virtual int getFD();

  // Virtual method that subclasses need to override
  virtual int receive(DMbuffer inBuffer);
  
  // Virtual method that subclasses need to override
  virtual int handleEvents(fd_set fdSet);
  
  // This method is used to give the object a network data object
  // to use to access the network.
  int setPentData(pentData *dataObj);

  // add the constraints this object has on it's source
  int addSrcConstraints(DMparams *params);
  
  // add the constraints this object has on it's destination
  int addDstConstraints(DMparams *params);

  // debugging
  // print out when you try to return an error with the DMUIC_ERR macro
  int setPrintReturnErr(int _printReturnErr);
protected:
  char *hostname;
  int port;
  pentData *dataObj;

  // lock around the data object, since we can swap it out.
  usema_t *dataLock;

  virtual void lockData() { /*uspsema(dataLock);*/; }
  virtual void unlockData() { /*usvsema(dataLock);*/; }
  
  virtual int easyToAdd() { return DMU_TRUE; };

  // debugging
  int               printReturnErr;
private:
};

//
//  DEBUGGING macro
//
// this macro will check the return on a statement and if it is not
// DMU_TRUE it will flag an error and print some stuff out.
// 
#define DMUNET_ERR(cond, ret) { \
    if (cond) {\
       if (printReturnErr == DMU_TRUE) { \
          fprintf(stderr, "dmuIC error:%d (line:%d file:%s) errno:%d\n", \
	          ret, __LINE__, __FILE__, errno); \
	  char errorBuf[DM_MAX_ERROR_DETAIL]; \
	  int errorNum; \
          fprintf(stderr, "Error description:%s\n", \
		  dmGetError(&errorNum, errorBuf));\
          fprintf(stderr, "   dmu errorNum:%d\n", errorNum);\
          fprintf(stderr, "   detail:%s\n", errorBuf);\
	  fprintf(stderr, "   object:%s\n", getName());\
	  fprintf(stderr, "   errno:%d\n", errno);\
       } \
       return(ret);\
    }}

#endif //_DMUNETWORK_H
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
