//
//  dmuGLCtx.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// 
#include "dmuGLCtx.h"

//****************************************************************
//
//  Public dmuGLCtx methods
//
//****************************************************************

//----------------------------------------------------------
// dmuGLCtx
//----------------------------------------------------------
//
// Constructor. Intializes some data members
//
dmuGLCtx::dmuGLCtx(){
  context = NULL;
}

//----------------------------------------------------------
// ~dmuGLCtx
//----------------------------------------------------------
//
// Destructor. Frees the GL Context if one has been created
//
dmuGLCtx::~dmuGLCtx(){
  if (context) {
    glXDestroyContext(dpy,context);
  }
}

//----------------------------------------------------------
// init
//----------------------------------------------------------
//
// This method create the GL Context that we will associate
// with the pbuffer
//
int dmuGLCtx::init(dmuParams *_params){
  int err;
  err = createPBuffer();
  if (err!=DMU_OK){
    return (err);
  }

  err = createContext();
  if (err!=DMU_OK){
    return (err);
  }
  
  // Initialize our children
  return (dmuObj::init(_params));
}

//----------------------------------------------------------
// makeCurrentRead
//----------------------------------------------------------
//
// make the object's pbuffer the current readable
//
int dmuGLCtx::makeCurrentRead(){
  GLXContext  currentContext = glXGetCurrentContext();
  GLXDrawable currentWindow = glXGetCurrentDrawable();

  return (glXMakeCurrentReadSGI(dpy,currentWindow,pbuffer,currentContext));
}

//----------------------------------------------------------
// makeCurrentRead
//----------------------------------------------------------
//
// make the object's pbuffer the current readable
//
int dmuGLCtx::makeCurrentRead(GLXDrawable _draw, GLXContext _context){
  return (glXMakeCurrentReadSGI(dpy,_draw,pbuffer,_context));
}

//----------------------------------------------------------
// makeCurrentRead
//----------------------------------------------------------
//
// make the object's pbuffer the current readable
//
int dmuGLCtx::makeCurrentRead(GLXDrawable _draw){
  GLXContext  currentContext = glXGetCurrentContext();

  return (glXMakeCurrentReadSGI(dpy,_draw,pbuffer,currentContext));
}

//----------------------------------------------------------
// makeCurrentWrite
//----------------------------------------------------------
//
// make the object's pbuffer the current writeable
//
int dmuGLCtx::makeCurrentWrite(){
  return (glXMakeCurrentReadSGI(dpy,pbuffer,glXGetCurrentDrawable(),context));
}

//----------------------------------------------------------
// makeCurrent
//----------------------------------------------------------
//
// make the object's pbuffer the current readable and writeable
//
int dmuGLCtx::makeCurrent(){
  return (glXMakeCurrent(dpy,pbuffer,context));
}

//****************************************************************
//
//  Protected methods
//
//****************************************************************

//----------------------------------------------------------
// createContext
//----------------------------------------------------------
//
int dmuGLCtx::createContext() {
  // create the GL context with the config
  context = glXCreateContextWithConfigSGIX(dpy, fbConfig, 
					   GLX_RGBA_TYPE_SGIX, 
					   NULL, GL_TRUE);
  if (!context){
    return (DMU_GL_PBUFFER_SETUP_FAILED);
  }
  return(DMU_OK);
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
