//
//  dmuGLPBuffer.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
#include "dmu.h"
#include "dmuGLPaintBuffer.h"

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// dmuGLPaintbuffer
//----------------------------------------------------------
//
// Constructor. Intializes some data members
//
dmuGLPaintBuffer::dmuGLPaintBuffer() {
  paintBuffer = NULL;
  poolParams  = NULL;
  doDoubleBuffer = 0;
}

//----------------------------------------------------------
// ~dmuGLPaintbuffer
//----------------------------------------------------------
//
// Destructor - does nothing
//
dmuGLPaintBuffer::~dmuGLPaintBuffer() {
}
  
//----------------------------------------------------------
// init
//----------------------------------------------------------
//
// This method creates a
//
int dmuGLPaintBuffer::init (dmuParams *params) {
  int err;

  // create the pbuffer
  err = createPBuffer();
  if (err!=DMU_OK){
    return (err);
  }

  // create a gl context for the pbuffer
  err = createContext();
  if (err!=DMU_OK){
    return (err);
  }

  // create a pool of dmbuffer to use with the pbuffers
  err = createPool();
  if (err!=DMU_OK){
    return (err);
  }

  // --Initialize our children
  return (dmuObj::init(params));
}

//----------------------------------------------------------
// grab
//----------------------------------------------------------
//
// This method overrides the parent class's grab method and
// allocates a dmbuffer from the pool.  Then it passes the buffer
// to itself with the receive method.  At this point it calls the
// parent class's grab method to hold and store the pointer to the
// buffer
//
int dmuGLPaintBuffer::grab() {
  DMstatus stat;

  if (!paintBuffer){
    stat = dmBufferAllocate(bufferPool, &paintBuffer);
    this->receive(paintBuffer);
    dmBufferFree(paintBuffer);
  }
  return (dmuGLCtx::grab());
}

//****************************************************************
//
//  Protected methods
//
//****************************************************************

//----------------------------------------------------------
// createPool
//----------------------------------------------------------
//
int dmuGLPaintBuffer::createPool() {
  int stat;

  // -- Create a dmBuffer pool --
  stat = dmParamsCreate(&poolParams);
  if (stat != DM_SUCCESS){
    return (DMU_PARAMS_CREATE_FAILED);
  }

  stat = dmBufferSetPoolDefaults(poolParams, 0, 0,
				 DM_FALSE, DM_FALSE);
  if (stat != DM_SUCCESS){
    return (DMU_POOL_DEFAULTS_FAILED);
  }

  stat = dmBufferGetGLPoolParams(gfxParams,poolParams);
  if (stat != DM_SUCCESS){
    return (DMU_POOL_CREATE_FAILED);
  }

  stat = dmParamsSetInt( poolParams, DM_BUFFER_COUNT, 10);
  if (stat != DMU_OK){
    return (DMU_BAD_CONSTRAINTS);
  }


  // Create the buffer pool
  DEBUG1(fprintf (stderr, "Create Pool\n"));
  DEBUG1(fprintf (stderr, "Parameters:\n"));
  DEBUG1(printDMParams(poolParams));
  stat = dmBufferCreatePool (poolParams, &bufferPool);
  if (stat != DM_SUCCESS){
    fprintf (stderr, "error:%d",stat);
    return (DMU_POOL_CREATE_FAILED);
  }

  return(DMU_OK);
}

/* PRE-NAB
  // --Initialize our children
  return (dmuObj::init(params));
}



int dmuGLPaintBuffer::grab(){
  DMstatus stat;

  // if we don't have a paint buffer, or if we're double buffering, 
  // we'll allocate a new buffer and attach it 
  if ((!paintBuffer) || (doDoubleBuffer) ){
    stat = dmBufferAllocate(bufferPool, &paintBuffer);
    this->receive(paintBuffer);
    dmuGLCtx::grab();
    dmBufferFree(paintBuffer);
  }
  //return (dmuGLCtx::grab());
  return (DMU_OK);
}
 
int dmuGLPaintBuffer::release(){
  return (dmuGLCtx::release());
*/

/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
