 /****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/

#include "dmu.h"
#include "dmuGLPixelTrans.h"

dmuGLPixelTrans::dmuGLPixelTrans(){
  dstBuffer = NULL;
  poolParams  = NULL;
  inputFormat = GL_YCRCB_422_SGIX;
}

dmuGLPixelTrans::~dmuGLPixelTrans(){
  
}
  

int dmuGLPixelTrans::init (dmuParams *params){
  int error = createPBuffer();
  if (error != DMU_OK) {
    return(error);
  }
    
  // -- Create a dmBuffer pool --
  DMstatus stat;
  
  stat = dmParamsCreate(&poolParams);
  if (stat != DM_SUCCESS){
    return (DMU_PARAMS_CREATE_FAILED);
  }

  stat = dmBufferSetPoolDefaults(poolParams, 0, 0, DM_FALSE, DM_FALSE);
  if (stat != DM_SUCCESS){
    return (DMU_POOL_DEFAULTS_FAILED);
  }

  stat = dmBufferGetGLPoolParams(gfxParams,poolParams);
  if (stat != DM_SUCCESS){
    return (DMU_POOL_CREATE_FAILED);
  }

  stat = dmParamsSetInt( poolParams, DM_BUFFER_COUNT, 4);
  if (stat != DMU_OK){
    return (DMU_BAD_CONSTRAINTS);
  }

  // Create the buffer pool
  DEBUG(fprintf (stderr, "dmuGLPixelTrans: Create Pool\n"));
  DEBUG(fprintf (stderr, "Parameters:\n"));
  DEBUG(printDMParams(poolParams));
  stat = dmBufferCreatePool (poolParams, &bufferPool);
  if (stat != DM_SUCCESS){
    fprintf (stderr, "error:%d",stat);
    return (DMU_POOL_CREATE_FAILED);
  }

  
  context = glXCreateContextWithConfigSGIX(dpy, fbConfig, 
					   GLX_RGBA_TYPE_SGIX, 
					   NULL, GL_TRUE);
  if (!context){
    return (DMU_GL_PBUFFER_SETUP_FAILED);
  }
  
  return (dmuObj::init(params));
}



int dmuGLPixelTrans::grab(){
  DMstatus stat;

  dmuLatch::grab();
  DEBUG2(fprintf(stderr, "mapping the buffer\n"));
  unsigned char *data = mapBuffer();
  
  stat = dmBufferAllocate(bufferPool, &dstBuffer);
  Bool ok;


  ok = glXAssociateDMPbufferSGIX(dpy,pbuffer,gfxParams,dstBuffer);
  if (!ok){
    fprintf (stderr, "Error in associate");
  } else {
    //fprintf (stderr, "grabed\n");
  }

  makeCurrent();
  glViewport( 0, 0, bufferSizeX, bufferSizeY);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,bufferSizeX,0,bufferSizeY,0,1000);

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  
  glDrawPixels(imageSizeX, imageSizeY, inputFormat,
	       GL_UNSIGNED_BYTE, data);

  return (DMU_OK);
}

int dmuGLPixelTrans::setImageDimensions(int newSizeX,int newSizeY){
  imageSizeX = newSizeX;
  imageSizeY = newSizeY;
  return (DMU_OK);

}

int dmuGLPixelTrans::release(){
  if (dstBuffer) {
    dmuObj::propagate(dstBuffer);
  }
  dmBufferFree(dstBuffer);

  unlockLatchBuffer();
  DEBUG2(fprintf (stderr, "released\n"));
  return (DMU_OK);
}

unsigned char *dmuGLPixelTrans::mapDstBuffer() {
   return ((unsigned char *)(dmBufferMapData(dstBuffer)));
}

int dmuGLPixelTrans::setInputFormat(int format){
  inputFormat = format;
  return (DMU_OK);
}


// addSrcConstraints - Add my graphics constraints to anyones params
// who wants to be my source based on my gfxParams (which are set in
// seutp()).
int dmuGLPixelTrans::addSrcConstraints(DMparams *params){
  return (DMU_OK);
}

int dmuGLPixelTrans::handleEvents(fd_set fdSet){
  if (!haveNewBuffer){
    return (DMU_NOT_MY_EVENT);
  }
  else {
    GLXContext  currentContext = glXGetCurrentContext();
    GLXDrawable currentWindow = glXGetCurrentDrawable();
    grab();
    release();
    glXMakeCurrent(dpy,currentWindow,currentContext);
  }
  return (DMU_OK);
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
