//
//  dmuGLPBuffer.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// dmuGLTex2d - wraps the grab() and release() methods of the
// dmuGLPBuffer in a single method, defineTex2D().
//
#include "dmuGLTex2d.h"

int dmuGLTex2d::nextFreeID = 1;

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// dmuGLTex2d
//----------------------------------------------------------
//
// Constructor. Intializes some data members
//
dmuGLTex2d::dmuGLTex2d(){
  gotPBuffer = 0;
  //glGenTexturesEXT(1,&texID);
  texID = nextFreeID++;
  //fprintf (stderr, "TEXIDS (%d,%d)\n",texID,nextFreeID);
  texInitialized = 0;
}

//----------------------------------------------------------
// ~dmuGLTex2d
//----------------------------------------------------------
//
// Destructor - does nothing currently
//
dmuGLTex2d::~dmuGLTex2d(){
}


//----------------------------------------------------------
// defineTex2D
//----------------------------------------------------------
//
// Sets up Open GL to use the PBuffer and it's
// associated DMBuffer data as the source for all texture operations.
// This state holds until a new context is made current, or any other
// call to glXMakeCurrentReadSGI changes the current GL readable.  (or
// if a glTexImage2D or other texture definition operation is called).
// This call will not copy the data into a separate texture memory.
// Instead, it will use the PBuffer's reference to it's associated
// DMBuffer as the texture source directly (i.e. it reads the data 
// pointed to by the pbuffer - it doesn't copy it).
//
int dmuGLTex2d::defineTex2D(){
  //fprintf (stderr, "(%s) ID = %d\n",getName(),texID);
  DEBUG2(fprintf (stderr, "MAKE CURRENT AND COPY TEX IMAGE..\n"));

  GLXContext  currentContext = glXGetCurrentContext();
  GLXDrawable currentWindow = glXGetCurrentDrawable();
  int err;
  
  //fprintf (stderr, "currentContext:%x,currentDrawable:%x",currentContext,
  //currentWindow);
  glXMakeCurrent(dpy,currentWindow,currentContext);

  // This initializes the texture so GL know's what kind of texture
  // to expect.   This happens only once.
  if ( !texInitialized && firstBufferIn){

    glBindTextureEXT(GL_TEXTURE_2D, texID);

    glTexImage2D(GL_TEXTURE_2D, 0,	    
		 GL_RGBA8_EXT,
		 imageSizeX, imageSizeY,
		 0,
		 GL_RGBA,
		 GL_UNSIGNED_BYTE,
		 NULL);

    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE_EXT);

    fprintf (stderr, "Initialize ID %d\n",texID);
  }

  // Release the PBuffer if we've got it grabbed - It's time to 
  // grab more recent data.
  if (gotPBuffer){
    dmuGLPBuffer::release();
  }

  // Grab and associate w/ the PBuffer, the latest data in the pipe.
  // Tell GL to use this data as the source for all subsequent
  // textured operations.
  if ( texInitialized  && firstBufferIn){
    glBindTextureEXT(GL_TEXTURE_2D, texID);

    err = dmuGLPBuffer::grab();
    if (err != DMU_OK){
      fprintf (stderr, "define tex failed\n");
      return (DMU_GL_PBUFFER_ASSOC_FAILED);
    }
    
    gotPBuffer = 1;
    DEBUG2(fprintf (stderr, "TEXTURE MAKE CURRENT..\n"));
    glXMakeCurrentReadSGI(dpy,
			  currentWindow,
			  pbuffer,
			  currentContext);
    
    DEBUG2(fprintf (stderr, "TEXTURE COPYTEX..\n"));

    glCopyTexSubImage2DEXT(GL_TEXTURE_2D, 0, 
			   0, 0,				// offset
			   0, 0,				// x, y
			   imageSizeX, 
			   imageSizeY);
    return (DMU_OK);
  }
  else {
    if (firstBufferIn)
      texInitialized = 1;
  }

    return(DMU_OK);
}

/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
