//
//  dmuNetwork.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
//
#include "dmuNetwork.h"

//****************************************************************
//
//  Public dmuNetwork methods
//
//****************************************************************

//----------------------------------------------------------
// dmuNetwork
//----------------------------------------------------------
//
// Constructor. Initializes some data members and create a semaphore
// to use to lock the data
//
dmuNetwork::dmuNetwork() {
  dataObj = NULL;

  // create a new semaphore to do locking on the buffer
  dataLock = usnewsema(dmuArena, 1);
}


//----------------------------------------------------------
// ~dmuNetwork
//----------------------------------------------------------
//
// Destructor.  Frees the semaphore.
//
dmuNetwork::~dmuNetwork() {
  usfreesema(dataLock, dmuArena);
}

//----------------------------------------------------------
// setup
//----------------------------------------------------------
//
// place holder
//
int dmuNetwork::setup() {
  return(DMU_OK);
}

//----------------------------------------------------------
// init
//----------------------------------------------------------
//
// virtual method that should never be called
//
int dmuNetwork::init(dmuParams *params) {
  return (DMU_VIRTUAL_CALLED);
}

//----------------------------------------------------------
// start
//----------------------------------------------------------
//
// pass the start method down the chain
//
int dmuNetwork::start(int propagate) {
  return (dmuObj::start(propagate));
}

//----------------------------------------------------------
// stop
//----------------------------------------------------------
//
// lock the data and then stop.
//
int dmuNetwork::stop(){
  int err;

  // make sure no other thread is messing with the data
  lockData();

  // stop the object
  err = dmuObj::stop();

  // let the other threads go again
  unlockData();

  return (err);
}

//----------------------------------------------------------
// getFD
//----------------------------------------------------------
//
// No fd to return yet
//
int dmuNetwork::getFD() {
  return(-1);
}


//----------------------------------------------------------
// receive
//----------------------------------------------------------
//
// virtual method that should never be called
//
int dmuNetwork::receive(DMbuffer inBuffer) {
  return (DMU_VIRTUAL_CALLED);
}

//----------------------------------------------------------
// handleEvents
//----------------------------------------------------------
//
// virtual method that should never be called
//
int dmuNetwork::handleEvents(fd_set fdSet) {
  return (DMU_VIRTUAL_CALLED);
}

//----------------------------------------------------------
// setPentData
//----------------------------------------------------------
//
// give the object a network data object to work with
//
int dmuNetwork::setPentData(pentData *_dataObj){
  // make sure no other thread is messing with the data
  lockData();

  // set the new network data object
  dataObj = _dataObj;

  // let the other threads go again
  unlockData();

  return (DMU_OK);
}

//----------------------------------------------------------
// addSrcConstraints
//----------------------------------------------------------
//
// This object doesn't add any constraints to it's source,
// so return DMU_OK
//
int dmuNetwork::addSrcConstraints(DMparams *params) {
  return(DMU_OK);
}

//----------------------------------------------------------
// addDstConstraints
//----------------------------------------------------------
//
// This object doesn't add any constraints to it's destination,
// so return DMU_OK
//
int dmuNetwork::addDstConstraints(DMparams *params) {
  return(DMU_OK);
}

//----------------------------------------------------------
// setPrintReturnErr
//----------------------------------------------------------
//
// Debugging function to tell the object to print an error to stderr
// if it is returning an error.
int dmuNetwork::setPrintReturnErr(int _printReturnErr) {
  printReturnErr = _printReturnErr;

  return(DMU_OK);
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
