//
//  dmuNetworkSend.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// This object sends buffer across the network.  Currently,
// this object can only be the tail of a chain.  
//
#include "dmuNetworkSend.h"

//****************************************************************
//
//  Public dmuNetworkSend methods
//
//****************************************************************

//----------------------------------------------------------
// dmuNetworkSend
//----------------------------------------------------------
//
// Constructor. Does nothing beyond initaliziation of
// member variables
//
dmuNetworkSend::dmuNetworkSend() {
  counter = 0;
  headerVal = 0;
}

//----------------------------------------------------------
// ~dmuNetworkSend
//----------------------------------------------------------
//
// Destructor
//
dmuNetworkSend::~dmuNetworkSend() {
}

//----------------------------------------------------------
// init
//----------------------------------------------------------
//
// This method calls the init method down the chain
//
int dmuNetworkSend::init(dmuParams *_params) {
  return(dmuObj::init());
}

//----------------------------------------------------------
// start
//----------------------------------------------------------
//
// This method calls the start method of the root object(dmuObj).
// You must have the network data object set before calling this
// method
//
int dmuNetworkSend::start(int _propagate) {
  if (!dataObj){
    fprintf (stderr,"dmuNetworkSend::Start occured before setting data obj\n");
    return (DMU_NETWORK_ERROR);
  }
  else{
    return(dmuObj::start(_propagate));
  }
}


//----------------------------------------------------------
// receive
//----------------------------------------------------------
//
// This method actually receives buffers from the network and then
// passes them down the chain
//
int dmuNetworkSend::receive(DMbuffer _inBuffer) {
  char *buf = NULL;
  int bufferSize;

  DEBUG2(fprintf(stderr, "%s: receive\n", getName()));
  
  // make sure nothing else messes with that data when we're messing
  // with it
  lockData();

  // if the object is started...
  if (isStarted()) {
    counter++;

    // add a ref to the object
    dmBufferAttach(_inBuffer);
    
    // get the size of the buffer
    bufferSize = dmBufferGetSize(_inBuffer);
    DEBUG(fprintf(stderr, "%s: BufferSize:%d\n", getName(), bufferSize));
  
    // map the buffer so we can write from it
    buf = (char *) dmBufferMapData(_inBuffer);
    if (!buf) {
      fprintf(stderr, "%s BAD BUFFER!!!\n", getName());
      return DMU_NETWORK_ERROR;
    }

    // send the buffer across the network
    dataObj->sendData(&headerVal, buf, bufferSize);
    DEBUG2(fprintf(stderr, "%s: num of bytes sent:\n", getName(),bufferSize));

    // free up our ref to the buffer
    dmBufferFree(_inBuffer);
  }

  // let other threads mess with the data now.
  unlockData();
  
  return(DMU_OK);
}

//----------------------------------------------------------
// handleEvents
//----------------------------------------------------------
//
// This method returns OK because currently we have no FD associated with the
// send method.  
//
int dmuNetworkSend::handleEvents(fd_set _fdSet) {
  return(DMU_OK);
}
      
//----------------------------------------------------------
// setHeaderValue
//----------------------------------------------------------
//
// This method allows you to tag a buffer with a value.  If you call
// thie method before sending the buffer, the number you set will go
// across the network with the buffer
//
void dmuNetworkSend::setHeaderValue(int _val){
  headerVal = _val;
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
