//
//  dmuObj.c++
//
//  Author: Ed Allard & Tim Davison
//  Date:   Oct, 1996
//
#include "dmuObj.h"
#include "dmuUtil.h"
// not is used yet.  
//#include "dmuBuffer.h"

//**********************************************************
//
// Public Methods
//
//**********************************************************

usptr_t *dmuObj::dmuArena = NULL;

//-----------------------------------------------------------
// dmuObj
//----------------------------------------------------------
//
//  Constructor that sets up the object
//  
dmuObj::dmuObj(){
  objList = NULL;
  buffer = NULL;
  bufferPool = NULL;
  numBuffers = 6;
  imageSizeX  = 0;
  imageSizeY  = 0;
  bufferSizeX  = 0;
  bufferSizeY  = 0;
  name = strdup("No name");
  
  started = DMU_FALSE;
  initialized = DMU_FALSE;

  if (dmuArena == NULL){
    // Create a semaphore arena.
    usconfig(CONF_ARENATYPE, US_SHAREDONLY);
    char *dmuArenaFileName = strdup("/tmp/dmuXXXXXX");
    mktemp(dmuArenaFileName);
    DEBUG1(fprintf(stderr, "\nCreating arena:%s\n",dmuArenaFileName));
    dmuArena = usinit(dmuArenaFileName);
    if (!dmuArena){
      fprintf (stderr, "Unable to create semaphore arena\n");
      unlink(dmuArenaFileName);
    }
  }
}

//-----------------------------------------------------------
// ~dmuObj
//----------------------------------------------------------
//
//  destructor that gets rid of some stuff
//  
dmuObj::~dmuObj(){
  if (bufferPool){
    DEBUG( fprintf (stderr, "~dmuObj(%s): Deleting buffer pool\n", getName()));
    dmBufferDestroyPool(bufferPool);
  }
}


// setName - set the name of the object to newName
void dmuObj::setName(const char *newName){
  if (name)
    delete(name);

  name = strdup(newName);
}
//-----------------------------------------------------------
// addObject
//----------------------------------------------------------
//
//  This method will add an object to the list children
//  
int dmuObj::addObject(dmuObj *newObj) {
  DEBUG2( fprintf(stderr, "adding Object to child list:%s\n", getName()) );
  DEBUG( if (!newObj) { return DMU_BAD_INPUT; } );

  if (initialized && !newObj->easyToAdd()){
    return (DMU_NO_RUNTIME_ADD);
  }

  // allocate memory for a list item
  dmuObjList *tmpPtr = (dmuObjList *)malloc(sizeof(dmuObjList));
  // make sure that the allocation worked
  DEBUG( assert(tmpPtr) );
  // initialize the new structure with the new Object;
  tmpPtr->obj = newObj;
  // set the new structure next pointer to be the current list
  // note that this depends on the objList being initialized to NULL for
  // the inital case.  
  tmpPtr->next = objList;
  // set the head of the list to be the new object
  objList = tmpPtr;

  // set the parent of the new object to be this object
  newObj->setParent(this);

  DEBUG2( fprintf(stderr, "done adding Object to child list:%s\n", getName()));
  
  // no problems
  return DMU_OK;
}

//----------------------------------------------------------
// init
//----------------------------------------------------------
//
//  This method will call the init method on all of the children
//  in the list created by addObject
//  
int dmuObj::init(dmuParams *params) {
  DEBUG2( fprintf(stderr, "Initing:%s\n", getName()) );

  // create tmp pointer to traverse the list
  dmuObjList *tmpPtr = objList;

  // while the tmp pointer is not NULL
  while (tmpPtr) {
    // make use we don't dereference a NULL value by accident
    DEBUG( assert(tmpPtr->obj) );
    // call init on the current object in the list
    dmuerr( tmpPtr->obj->init() );
    // move to the next object
    tmpPtr = tmpPtr->next;
  }
  
  DEBUG2( fprintf(stderr, "done Initing:%s\n", getName()) );

  initialized = DMU_TRUE;

  // no problems
  return DMU_OK;
}

//----------------------------------------------------------
// start
//----------------------------------------------------------
//
//  This method will call the start method on all of the children
//  in the list created by addObject
//  
int dmuObj::start(int propagate) {
  DEBUG( fprintf(stderr, "Starting:%s\n", getName()) );


  if (propagate){
    // create tmp pointer to traverse the list
    dmuObjList *tmpPtr = objList;
    
    // while the tmp pointer is not NULL
    while (tmpPtr) {
      // make use we don't dereference a NULL value by accident
      DEBUG( assert(tmpPtr->obj) );
      // call init on the current object in the list
      tmpPtr->obj->start(propagate);
      // move to the next object
      tmpPtr = tmpPtr->next;
    }
    
  }
  started = DMU_TRUE;

  DEBUG2( fprintf(stderr, "done Starting:%s\n", getName()) );

  // no problems
  return DMU_OK;
}


int dmuObj::stop() {
  DEBUG( fprintf(stderr, "Stopping:%s\n", getName()) );

  started = DMU_FALSE;

  // create tmp pointer to traverse the list
  dmuObjList *tmpPtr = objList;
  
  // while the tmp pointer is not NULL
  while (tmpPtr) {
    // make use we don't dereference a NULL value by accident
    DEBUG( assert(tmpPtr->obj) );
    // call init on the current object in the list
    tmpPtr->obj->stop();
    // move to the next object
    tmpPtr = tmpPtr->next;
  }
  
  DEBUG2( fprintf(stderr, "Done Stopping:%s\n", getName()) );

  // no problems
  return DMU_OK;
}


//----------------------------------------------------------
// getFD
//----------------------------------------------------------
//
// For this method to be useful, it has to be overridden by
// subclass.
//
int dmuObj::getFD() {
  return (-1);
}

//----------------------------------------------------------
// setFD
//----------------------------------------------------------
//
// For this method to be useful, it has to be overridden by
// subclass.
//
int dmuObj::setFD(fd_set *fdSet,int *maxFD,int propagate){
  int fd;

  // Set our own FD (if it's a valid one)
  if ((fd = getFD()) > 0){
    FD_SET(fd, fdSet);
    DEBUG2(fprintf(stderr, "setting FD %d (for %s) in fd set\n", fd, getName()));
  }

  if (fd > *maxFD){
    *maxFD = fd;
  }

  if (propagate){
    // Pass the set down the list so our children set theirs
    dmuObjList *tmpPtr = objList;
    while (tmpPtr) {
      DEBUG( assert(tmpPtr->obj) );
      tmpPtr->obj->setFD(fdSet,maxFD,1);
      tmpPtr = tmpPtr->next;
    }
  }
  return (DMU_OK);
}

//**********************************************************
//
// Protected Methods
//
//**********************************************************

//----------------------------------------------------------
// propagate
//----------------------------------------------------------
//
// This is a very important method which passes a dmbuffer on
// to the list of children.  This method iterates over the
// children and calls receive on each one of them with the
// dmbuffer as the argument.  This method should be used
// in the handle events method of the subclass object.
//
int dmuObj::propagate(DMbuffer outBuffer) {
  DEBUG2( fprintf(stderr, "Propagating:%s\n", getName()) );

  // create tmp pointer to traverse the list
  dmuObjList *tmpPtr = objList;
  
  // while the tmp pointer is not NULL
  while (tmpPtr) {
    // make use we don't dereference a NULL value by accident
    DEBUG( assert(tmpPtr->obj) );
    // call receive on the current object in the list
    // to pass the dmbuffer on the the children
    dmuerr( tmpPtr->obj->receive(outBuffer) );
    // move to the next object
    tmpPtr = tmpPtr->next;
  }

  DEBUG2( fprintf(stderr, "done Propagating:%s\n",getName()) );
  
  // no problems
  return DMU_OK;
}

//----------------------------------------------------------
// addChildConstraints
//----------------------------------------------------------
//
//  This method will call the addSrcConstraints method on all of the children
//  in the list created by addObject
//
int dmuObj::addChildConstraints(DMparams *params) {
  DEBUG1( fprintf(stderr, "Adding Child Constraints:%s\n", getName()) );

  // create tmp pointer to traverse the list
  dmuObjList *tmpPtr = objList;
  // while the tmp pointer is not NULL
  while (tmpPtr) {
    // make use we don't dereference a NULL value by accident
    DEBUG( assert(tmpPtr->obj) );
    // call receive on the current object in the list
    dmuerr( tmpPtr->obj->addSrcConstraints(params) );
    // move to the next object
    tmpPtr = tmpPtr->next;
  }
  
  DEBUG1( fprintf(stderr, "done Adding Child Constraints:%s\n",getName()) );
  
  // no problems
  return DMU_OK;
}

//----------------------------------------------------------
// addParentConstraints
//----------------------------------------------------------
//
// This method will add the constraints of the parent to the
// parameter structure
//
int dmuObj::addParentConstraints(DMparams *params){
  DEBUG( assert(parent) );
  return (parent->addDstConstraints(params));
}




// getSize - sets returnWidth and returnHeight to be the size of
// a movie frame.  The movie must be opened for this to make 
// any sense.
void dmuObj::getImageDimensions(int *returnWidth, int *returnHeight){
  *returnWidth = imageSizeX;
  *returnHeight = imageSizeY;
}


// getBufferDimensions - sets returnSizes to be the dimensions of the
// dmbuffer used for a frame of the dmuObj
//
void dmuObj::getBufferDimensions(int *sizeX,int *sizeY){
  *sizeX = bufferSizeX;
  *sizeY = bufferSizeY;
}


/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
