 #include "dmuUtil.h"
 #include "dmu.h"
 #include "dmuError.h"
 #include <dmedia/dm_params.h>

 //
 //  Function prototypes
 //
 DMstatus PrintParam(const DMparams *params, const char *paramName,
	    void *arg, DMboolean *pstop);

 //----------------------------------------------------------
 // getErrorString
 //----------------------------------------------------------
 //
 // This function will return the string associated with an
 // error number
 //
 extern const char *getErrorString(int errorID) {
   for (int i = 0; i < numDmuErrors; i ++) {
     if (errorID == dmuErrors[i].errorID) {
       if (dmuErrors[i].fatal == DMU_TRUE) {
	 fprintf(stderr, "Fatal Error:%d:%s\n",
		 errorID, dmuErrors[i].errorString);
	 exit(-1);
       }
       return(dmuErrors[i].errorString);
     }
   }
   return(noErrorMsg);
 }

 //----------------------------------------------------------
 // printDMParams
 //----------------------------------------------------------
 //
 // This function will printout the contents of the params
 // structure
 //
 void printDMParams(const DMparams *params)
 {
   fprintf (stderr, "Contents of Parameter/Value List:\n");

   (void)dmParamsScan(params, PrintParam, 0);

  fprintf (stderr, "--------------------------------\n");

}


DMstatus PrintParam(const DMparams *params, const char *paramName,
		    void *arg, DMboolean *pstop)
{

  switch(dmParamsGetType(params, paramName)) {
  case DM_TYPE_INT:
    fprintf (stderr, "%d\t(int)", dmParamsGetInt(params, paramName));
    break;
  case DM_TYPE_LONG_LONG:
    fprintf(stderr, "%lld\t(longlong)",dmParamsGetLongLong(params, paramName));
    break;
  case DM_TYPE_FLOAT:
    fprintf (stderr, "%3.3f\t(float)", dmParamsGetFloat(params, paramName));
    break;
  case DM_TYPE_STRING:
    fprintf (stderr, "%s\t(string)", dmParamsGetString(params, paramName));
    break;
  case DM_TYPE_ENUM:
    fprintf (stderr, "%d\t(enum)", dmParamsGetEnum(params, paramName));
    break;
  default:
    fprintf (stderr, "type\t(%d)", dmParamsGetType(params, paramName));
    break;
  }
  fprintf (stderr, "\t%s\n", paramName);

  return DM_SUCCESS;
}




int setupFBConfig(Display *dpy, int rSize, int gSize, 
	      int bSize, int aSize, GLXFBConfigSGIX *returnConfig){

  GLXFBConfigSGIX *configs;
  int configCount;

  int configParams[] = { 
    GLX_RENDER_TYPE_SGIX, 	GLX_RGBA_BIT_SGIX,
    GLX_DRAWABLE_TYPE_SGIX,	GLX_PBUFFER_BIT_SGIX | GLX_WINDOW_BIT_SGIX,
    GLX_ALPHA_SIZE,		aSize,
    GLX_RED_SIZE, 		rSize,
    GLX_GREEN_SIZE,             gSize,
    GLX_BLUE_SIZE, 		bSize,
    GLX_DEPTH_SIZE, 		8,
    GLX_X_RENDERABLE_SGIX,	True,
    (int)None,
  };

  configs = glXChooseFBConfigSGIX(dpy,DefaultScreen(dpy),
				  configParams,&configCount);

  
  if (configs == NULL || configCount == 0){
    *returnConfig = 0;
    return (DMU_GL_NO_FBCONFIG);
  }

	
  for ( int i = 0;  i < configCount;  i++ ) {
    int value;
    
    glXGetFBConfigAttribSGIX(dpy,configs[i],GLX_ALPHA_SIZE, &value);
    if ( value != aSize )  continue;
    glXGetFBConfigAttribSGIX(dpy, configs[i],GLX_RED_SIZE, &value);
    if ( value != rSize )  continue;
    glXGetFBConfigAttribSGIX(dpy,configs[i],GLX_GREEN_SIZE, &value);
    if ( value != gSize)  continue;
    glXGetFBConfigAttribSGIX(dpy,configs[i], GLX_BLUE_SIZE, &value);
    if ( value != bSize)  continue;
    
    break;
  }

  if (i == configCount){
    *returnConfig = 0;
    return (DMU_GL_NO_FBCONFIG);
  }

  *returnConfig  = configs[i];
  
  XFree(configs);
  return (DMU_OK);
}



void printFBConfig(Display *dpy, GLXFBConfigSGIX fbConfig){
  static struct { int attrib; const char* name; } attribs [] = {
    { GLX_BUFFER_SIZE, "GLX_BUFFER_SIZE" },
    { GLX_LEVEL, "GLX_LEVEL" },
    { GLX_DOUBLEBUFFER, "GLX_DOUBLEBUFFER" },
    { GLX_AUX_BUFFERS, "GLX_AUX_BUFFERS" },
    { GLX_RED_SIZE, "GLX_RED_SIZE" },
    { GLX_GREEN_SIZE, "GLX_GREEN_SIZE" },
    { GLX_BLUE_SIZE, "GLX_BLUE_SIZE" },
    { GLX_ALPHA_SIZE, "GLX_ALPHA_SIZE" },
    { GLX_DEPTH_SIZE, "GLX_DEPTH_SIZE" },
    { GLX_DRAWABLE_TYPE_SGIX, "GLX_DRAWABLE_TYPE_SGIX" },
    { GLX_RENDER_TYPE_SGIX, "GLX_RENDER_TYPE_SGIX" },
    { GLX_X_RENDERABLE_SGIX, "GLX_X_RENDERABLE_SGIX" },
    { GLX_MAX_PBUFFER_WIDTH_SGIX, "GLX_MAX_PBUFFER_WIDTH_SGIX" },
    { GLX_MAX_PBUFFER_HEIGHT_SGIX, "GLX_MAX_PBUFFER_HEIGHT_SGIX"},
  };
  int i;
  
  for ( i = 0; i < ARRAY_COUNT( attribs ); i++ ){
    int value;
    int errorCode = glXGetFBConfigAttribSGIX(dpy, fbConfig, 
					     attribs[i].attrib, 
					     &value );

    if ( errorCode != 0){
      fprintf (stderr, "glXGetFBConfigAttribSGIX failed" );
    }
    printf( "    %24s = %d\n", attribs[i].name, value );
  }
}


// roundToPower - Round up to the next higher power of 2.  We need to
// do this in case the buffer is going to be used as a texture map.
//
int roundToPower(int num){
  int result = 1;
  for (int i = 0; result < num; i++){
    result = 1 << i;
  }
  return(result);
}

//**********************************************************
//
// Obsolete Functions
//
//**********************************************************

extern void oldPrintDMParams(const DMparams *p){
  int len = dmParamsGetNumElems( p );
  int i;

  fprintf(stderr, " inside printDMParams:%d", len);
  for ( i = 0;  i < len;  i++ ) {
    fprintf(stderr, ".");
    const char* name = dmParamsGetElem    ( p, i );
    DMparamtype type = dmParamsGetElemType( p, i );
	
    printf( "%8s: ", name );
    switch( type ) 
      {
      case DM_TYPE_ENUM:
	printf( "%d", dmParamsGetEnum( p, name ) );
	break;
      case DM_TYPE_INT:
	printf( "%d", dmParamsGetInt( p, name ) );
	break;
      case DM_TYPE_STRING:
	printf( "%s", dmParamsGetString( p, name ) );
	break;
      case DM_TYPE_FLOAT:
	printf( "%f", dmParamsGetFloat( p, name ) );
	break;
      case DM_TYPE_FRACTION:
	{
	  DMfraction f;
	  f = dmParamsGetFract( p, name );
	  printf( "%d/%d", f.numerator, f.denominator );
	}
	break;
      case DM_TYPE_PARAMS:
	printf ("\n**\n");
	printDMParams (dmParamsGetParams( p, name ));
	printf ("**\n");
	break;
      case DM_TYPE_ENUM_ARRAY:
	{
	  int i;
	  const DMenumarray* array = dmParamsGetEnumArray( p, name );
	  for ( i = 0;  i < array->elemCount;  i++ ) {
	    printf( "%d ", array->elems[i] );
	  }
	}
	break;
      case DM_TYPE_INT_ARRAY:
	{
	  int i;
	  const DMintarray* array = dmParamsGetIntArray( p, name );
	  for ( i = 0;  i < array->elemCount;  i++ ) {
	    printf( "%d ", array->elems[i] );
	  }
	}
	break;
      case DM_TYPE_STRING_ARRAY:
	{
	  int i;
	  const DMstringarray* array = 
	    dmParamsGetStringArray( p, name );
	  for ( i = 0;  i < array->elemCount;  i++ ) {
	    printf( "%s ", array->elems[i] );
	  }
	}
	break;
      case DM_TYPE_FLOAT_ARRAY:
	{
	  int i;
	  const DMfloatarray* array = 
	    dmParamsGetFloatArray( p, name );
	  for ( i = 0;  i < array->elemCount;  i++ ) {
	    printf( "%f ", array->elems[i] );
	  }
	}
	break;
      case DM_TYPE_FRACTION_ARRAY:
	{
	  int i;
	  const DMfractionarray* array = 
	    dmParamsGetFractArray( p, name );
	  for ( i = 0;  i < array->elemCount;  i++ ) {
	    printf( "%d/%d ", 
		    array->elems[i].numerator,
		    array->elems[i].denominator );
	  }
	}
	break;
      case DM_TYPE_INT_RANGE:
	{
	  const DMintrange* range = dmParamsGetIntRange( p, name );
	  printf( "%d ... %d", range->low, range->high );
	}
	break;
      case DM_TYPE_FLOAT_RANGE:
	{
	  const DMfloatrange* range = 
	    dmParamsGetFloatRange( p, name );
	  printf( "%f ... %f", range->low, range->high );
	}
	break;
      case DM_TYPE_FRACTION_RANGE:
	{
	  const DMfractionrange* range = 
	    dmParamsGetFractRange( p, name );
	  printf( "%d/%d ... %d/%d", 
		  range->low.numerator, 
		  range->low.denominator, 
		  range->high.numerator,
		  range->high.denominator );
	}
	break;

      default:
	printf( "UNKNOWN TYPE" );
      }
    printf( "\n" );
  }
  fprintf(stderr, "\n");
}







/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
