#include "dmuVideoOut.h"


dmuVideoOut::dmuVideoOut(){
  newBuffer = NULL;
  gotFirstFrame = 0;
}

dmuVideoOut::~dmuVideoOut(){

}

// addSrcConstraints - Need to implement in subclasses.  There are 
// no constraints common to all video objects
int dmuVideoOut::addSrcConstraints(DMparams *params){
  return (DMU_OK);
}


int dmuVideoOut::setup(int destDevice){
  int stat;

  sourceNode = vlGetNode(server, VL_SRC, VL_MEM, VL_ANY);
  if (sourceNode < 0){
    return (DMU_VIDEO_GETNODE_FAILED);
  }

  drainNode = vlGetNode(server, VL_DRN, VL_VIDEO, destDevice);
  if (drainNode < 0) { 
    return (DMU_VIDEO_GETNODE_FAILED);
  }

  path = vlCreatePath(server, VL_ANY, sourceNode, drainNode);
  if (path < 0){
    return (DMU_VIDEO_CREATEPATH_FAILED);
  }

  DEBUG1(fprintf(stderr, "Trying to setup video path\n"));
  stat = vlSetupPaths(server, &path, 1, VL_SHARE, VL_SHARE);
  if (stat != 0){
    return (DMU_VIDEO_SETUPPATH_FAILED);
  }
  DEBUG1(fprintf(stderr, "**Returned:%d\n",stat));
  
  // For debug puposes, we'll listen to all the events..  This list
  // will get trimmed significantly when things are more stable.
  vlSelectEvents(server, path, VLTransferCompleteMask | 
		 VLStreamBusyMask |
		 VLStreamPreemptedMask |
		 VLAdvanceMissedMask |
		 VLStreamAvailableMask |
		 VLSyncLostMask |
		 VLStreamStartedMask |
		 VLStreamStoppedMask |
		 VLSequenceLostMask |
		 VLControlChangedMask |
		 VLTransferCompleteMask |
		 VLTransferFailedMask |
		 VLEvenVerticalRetraceMask |
		 VLOddVerticalRetraceMask |
		 VLFrameVerticalRetraceMask |
		 VLDeviceEventMask |
		 VLDefaultSourceMask |
		 VLControlRangeChangedMask |
		 VLControlPreemptedMask |
		 VLControlAvailableMask |
		 VLDefaultDrainMask |
		 VLStreamChangedMask |
		 VLTransferError);

  
  return (setDefaultControls());
}


// setDefaultControls() - Setup all the default controls on the video
// path.  This assumes that the video nodes and path have been created
// and set up
//
int dmuVideoOut::setDefaultControls(){
  int err = 0;

  err = this->setPacking(VIDEOIN_DEFAULT_PACKING);
  DEBUG(fprintf (stderr, "Setting: %s \n","VIDEOIN_DEFAULT_PACKING"));
  if (err < -1){
    return (DMU_VIDEO_BAD_CONTROL_VALUE);
  }

  err = this->setCaptureType(VIDEOIN_DEFAULT_CAP_TYPE);
  DEBUG(fprintf (stderr, "Setting: %s \n","VIDEOIN_DEFAULT_CAP_TYPE"));
  if (err < -1){
    return (DMU_VIDEO_BAD_CONTROL_VALUE);
  }

  err = this->setFormat(VIDEOIN_DEFAULT_FORMAT);
  DEBUG(fprintf (stderr, "Setting: %s \n","VIDEOIN_DEFAULT_FORMAT"));
  if (err < -1){
    return (DMU_VIDEO_BAD_CONTROL_VALUE);
  }

  err = this->setLayout(VIDEOIN_DEFAULT_LAYOUT);
  DEBUG(fprintf (stderr, "Setting: %s \n","VIDEOIN_DEFAULT_LAYOUT"));
  if (err < -1){
    return (DMU_VIDEO_BAD_CONTROL_VALUE);
  }

  err = this->setZoom(VIDEOIN_DEFAULT_ZOOM_NUM, VIDEOIN_DEFAULT_ZOOM_DENOM);
  DEBUG(fprintf (stderr, "Setting: %s \n","VIDEOIN_DEFAULT_ZOOM"));
  if (err < -1){
    return (DMU_VIDEO_BAD_CONTROL_VALUE);
  }

  return (DMU_OK);
}


int dmuVideoOut::init(dmuParams *){
  newLock = usnewsema(dmuArena, 1);

  return (dmuObj::init());
}


int dmuVideoOut::receive(DMbuffer inBuffer){

  DEBUG1(fprintf (stderr, "videoOut::receive\n"));

  if (!gotFirstFrame){
    int err = vlDMBufferSend(server,path,inBuffer);
    if (err != 0){
      fprintf (stderr, "error sending frame\n");
    }
    else {
      DEBUG(fprintf (stderr, "receive sent first frame\n"));
      gotFirstFrame = 1;
    }
  }
  else {
    DEBUG1(fprintf (stderr, "receive holding a frame\n"));
    lockNewBuffer();

    if (newBuffer){
      dmBufferFree(newBuffer);
    }
    newBuffer = inBuffer;
    dmBufferAttach(newBuffer);
    unlockNewBuffer();
  }
  return (DMU_OK);
}


int dmuVideoOut::handleEvents(fd_set fdSet){
  VLEvent  event;
  int stat;
  DEBUG(fprintf (stderr, "videoOut::handleEvents\n"));

  // Make sure that my FD is set in the fdSet
  if (!FD_ISSET(getFD(), &fdSet)){
    DEBUG2(fprintf (stderr, "videoOut::not my event\n"));
    return (DMU_NOT_MY_EVENT);
  }

  stat = vlEventRecv(server, path, &event);
  if (stat < 0){
    fprintf (stderr, "VideoOut: couldn't get an event\n");
    return (DMU_VIDEO_NO_EVENT);
  }
  

  switch (event.reason){
  case VLStreamBusy:
    DEBUG(fprintf (stderr, "event: VLStreamBusy:\n"));
    break;
  case VLStreamPreempted:
    DEBUG(fprintf (stderr, "event: VLStreamPreempted:\n"));
    break;
  case VLAdvanceMissed:
    DEBUG(fprintf (stderr, "event: VLAdvanceMissed:\n"));
    break;
  case VLStreamAvailable:
    DEBUG(fprintf (stderr, "event: VLStreamAvailable:\n"));
    break;
  case VLSyncLost:
    DEBUG(fprintf (stderr, "event: VLSyncLost:\n"));
    break;
  case VLStreamStarted:
    DEBUG(fprintf (stderr, "event: VLStreamStarted:\n"));
    break;
  case VLStreamStopped:
    DEBUG(fprintf (stderr, "event: VLStreamStopped:\n"));
    break;
  case VLSequenceLost:
    DEBUG(fprintf (stderr, "event: VLSequenceLost:\n"));
    break;
  case VLControlChanged:
    DEBUG(fprintf (stderr, "event: VLControlChanged:\n"));
    break;
  case VLTransferFailed:
    DEBUG(fprintf (stderr, "event: VLTransferFailed:\n"));
    break;
  case VLEvenVerticalRetrace:
    DEBUG(fprintf (stderr, "event: VLEvenVerticalRetrace:\n"));
    break;
  case VLOddVerticalRetrace:
    DEBUG(fprintf (stderr, "event: VLOddVerticalRetrace:\n"));
    break;
  case VLDeviceEvent:
    DEBUG(fprintf (stderr, "event: VLDeviceEvent:\n"));
    break;
  case VLDefaultSource:
    DEBUG(fprintf (stderr, "event: VLDefaultSource:\n"));
    break;
  case VLControlRangeChanged:
    DEBUG(fprintf (stderr, "event: VLControlRangeChanged:\n"));
    break;
  case VLControlPreempted:
    DEBUG(fprintf (stderr, "event: VLControlPreempted:\n"));
    break;
  case VLControlAvailable:
    DEBUG(fprintf (stderr, "event: VLControlAvailable:\n"));
    break;
  case VLDefaultDrain:
    DEBUG(fprintf (stderr, "event: VLDefaultDrain:\n"));
    break;
  case VLStreamChanged:
    DEBUG(fprintf (stderr, "event: VLStreamChanged:\n"));
    break;
  case VLTransferError:
    DEBUG(fprintf (stderr, "event: VLTransferErr:\n"));
    break; 
  case VLFrameVerticalRetrace:
    DEBUG(fprintf (stderr, "event: VLFrameVerticalRetrace:\n"));
    break;
  case VLTransferComplete:
    if (!gotFirstFrame){
      fprintf (stderr, "ERROR - Got vl event before sending first frame\n");
      return (DMU_VIDEO_NO_EVENT);
    }
    lockNewBuffer();
    
    DEBUG(fprintf (stderr, "VIDEOOUT event: TRANSFER COMPLETE:\n"));

    // If we've got a new one to send, then lets send it
    if (newBuffer != NULL){
      DEBUG1(fprintf (stderr, "something to send\n"));

      DEBUG1(fprintf (stderr, "sending it\n"));
      int err = vlDMBufferSend(server,path,newBuffer);
      if (err != 0){
	DEBUG(fprintf (stderr, "error sending frame\n"));

      }
      else {
	DEBUG1(fprintf (stderr, "sent frame\n"));
      }

      // now we can free it up
      //fprintf (stderr, "freeing newBuffer\n");
      dmBufferFree(newBuffer);
      newBuffer = NULL;
    }
    DEBUG1(fprintf (stderr, "unlocking\n"));
    unlockNewBuffer();
    return (DMU_OK);
  }
  return (DMU_VIDEO_NO_EVENT);
}

void dmuVideoOut::lockNewBuffer(){
  uspsema (newLock);
}

void dmuVideoOut::unlockNewBuffer(){
  usvsema (newLock);
}

void dmuVideoOut::unlockCurrentBuffer(){
  uspsema (newLock);
}

void dmuVideoOut::lockCurrentBuffer(){
  usvsema (newLock);
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
