#include "dmu.h"
#include "dmuMovieInCompressed.h"
#include "dmuGLDisplay.h"
#include "dmuICjdecoder.h"
#include <dmedia/vl_mvp.h>

// movieDecodeTest - <dmu> Sample program that displays the playback
// of a movie file to a rotating, textured polygon.  This program uses
// a dmuMovieInCompressed object to read in JPEG compressed frames and
// feed them to a dmuICjdecoder object.  The decoder decompresses the
// frames and feeds them to a dmuGLDisplay object to display them to
// an open GL window.  Note that this sample program can only playback
// JPEG compressed movies that are full-frame (non-interlaced). </dmu>

#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
        exit(-1); \
    }\
    else { \
	     /*fprintf(stderr,"No Error\n");*/ \
    } \
 }


int main(int argc, char *argv[]){
  dmuMovieInCompressed  *movie;
  dmuGLDisplay          *display1;
  dmuGLDisplay          *display2;
  dmuGLDisplay          *display3;
  dmuICjdecoder         *decoder;
  
  fd_set readFDSet;
  int maxFD;
  int imageSizeX,imageSizeY;
  int returnEvent;

  if (argc < 2){
    fprintf (stderr, "usage: movieDecodeTest <movie file name>\n");
    exit (-1);
  }
  
  // -- Construct --

  movie = new dmuMovieInCompressed;
  movie->setName("Compressed Movie In");
  
  decoder = new dmuICjdecoder();
  decoder->setName("Decoder object");

  display1 = new dmuGLDisplay();
  display1->setName("Display object1");

  display2 = new dmuGLDisplay();
  display2->setName("Display object2");

  display3 = new dmuGLDisplay();
  display3->setName("Display object3");

  // -- Wire ---
  movie->addObject(decoder);
  decoder->addObject(display1);


  // -- Setup ---

  // setup movie
  dmuCheck(movie->setup(argv[1]));
  movie->getImageDimensions(&imageSizeX,&imageSizeY);

  // setup the decoder to match the size of the movie
  decoder->setImageDimensions(imageSizeX, imageSizeY);
  dmuCheck( decoder->setup() );

  // setup the display to open a window of the same size and packing
  // as the decoded movie
  display1->setImageDimensions(imageSizeX, imageSizeY);
  display1->setPacking( GL_YCRCB_422_SGIX);
  display1->setup();


  // -- Init --
 fprintf (stderr, "Initializing: \n");
  dmuCheck(movie->init());
  
  // -- Start --
  fprintf (stderr, "Fire it up!: \n");
  dmuCheck(movie->start());

  for (int i = 0; ; i++){
    FD_ZERO(&readFDSet);
    movie->setFD(&readFDSet,&maxFD);
    
    // Wait for something to happen on either the movie or
    // the decoder
    select (maxFD+1, &readFDSet, NULL, NULL, NULL);
    
    // If something happens, handle the event.
    returnEvent = movie->handleEvents(readFDSet);
    if ((returnEvent != DMU_OK) && (returnEvent != DMU_NOT_MY_EVENT)){
      fprintf (stderr, "MovieIn handleEvents error:%s\n",
	       getErrorString(returnEvent));
    }
    returnEvent = decoder->handleEvents(readFDSet);
    if ((returnEvent != DMU_OK) && (returnEvent != DMU_NOT_MY_EVENT)){
      fprintf (stderr, "Decoder handleEvents error:%s\n",
	       getErrorString(returnEvent));
    }

    
    if (i == 200){
      display2->setImageDimensions(imageSizeX, imageSizeY);
      display2->setPacking( GL_YCRCB_422_SGIX);
      display2->setup();

      display2->init();
      display2->start();
      
      decoder->addObject(display2);
    }


    if (i == 400){
      display3->setImageDimensions(imageSizeX, imageSizeY);
      display3->setPacking( GL_YCRCB_422_SGIX);
      display3->setup();

      display3->init();
      display3->start();
      
      decoder->addObject(display3);
    }

    if (i == 1000){
      decoder->stop();
    }

    if (i == 1200){
      decoder->start();
    }
  }

  //  delete (movie);
  //  delete (decoder);
  //  delete (display);
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
