#include "dmu.h"
#include "dmuObj.h"
#include "dmuNetworkSend.h"
#include "dmuGLDisplay.h"
#include "dmuMovieInCompressed.h"
#include "pentTCPSocketClient.h"
#include "pentData.h"

#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
					     /*exit(-1);*/ \
    }\
    else { \
       fprintf(stderr,"No Error\n"); \
    } \
 }


int main(int argc, char *argv[]){
  // dmu object
  dmuMovieInCompressed *movieIn;
  dmuNetworkSend       *netSend;

  // pent objects - now we have to setup our own network
  pentTCPSocketClient *netClient;
  pentData            *netData;

  int sizeX, sizeY;
  
  if (argc < 4){
    fprintf (stderr, "usage: fileSendTest <hostname> <portnum> <movieFile>\n");
    exit(-1);
  }
  
  // dmu construction
  movieIn = new dmuMovieInCompressed();
  movieIn->setName("Movie");

  netSend = new dmuNetworkSend();
  netSend->setName("netSend");

  // dmu chain setup
  movieIn->addObject(netSend);
  dmuCheck(movieIn->setup(argv[3]));
  dmuCheck(netSend->setup());

  // network setup
  netClient = new pentTCPSocketClient();
  netClient->setName("netClient");

  netClient->setAddr(argv[1]);
  netClient->setPort(atoi(argv[2]));
  netClient->setup();
  fprintf (stderr, "client setup finished\n");
  

  netData = new pentData();
  netData->setPrintReturnErr(PENT_TRUE);
  netData->setName("movieSendPentData");
  netData->setHeaderSize(sizeof(int));
  
  netClient->handshake(netData);
  fprintf (stderr, "handshake finished\n");

  netSend->setPentData(netData);
  
  // dmu chain startup
  dmuCheck(movieIn->init());
  dmuCheck(movieIn->start());
  
  fd_set readFDSet;
  int maxFD;
  for (int i = 0; ; i++) {
    FD_ZERO(&readFDSet);
    movieIn->setFD(&readFDSet, &maxFD);
    
    select(maxFD + 1, &readFDSet, NULL, NULL, NULL);

    movieIn->handleEvents(readFDSet);
  }

}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
