#include "dmu.h"
#include "dmuMovieInRender.h"
#include "dmuVideoOut.h"
#include <dmedia/vl_mvp.h>
#include <dmedia/vl.h>

// movieToVideo.c++ - <dmu>Renders a movie into a dmbuffer, and then 
// out to the video output (as selected by vcp). Demonstrates the 
// use of the bufferSize methods on the movie object and the use of 
// the video output object using graphics layout, rgba data.
// </dmu>


// Simple macro for checking the result of a dmu method and printing
// the results if there's ana error.
#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
	       exit(-1); \
    }\
    else { \
	     /*fprintf(stderr,"No Error\n");*/ \
    } \
 }


int main(int argc, char *argv[]){
  // dmu objects we'll use
  dmuMovieInRender *movieIn;
  dmuVideoOut      *videoOut;

  // other variables we'll need
  fd_set readFDSet;
  int sizeX,sizeY;

  if (argc < 2){
    fprintf (stderr, "Usage: movieToVideo <movie file name>\n");
    exit (-1);
  }

  // -- Construction --
  // Create our objects.
  movieIn = new dmuMovieInRender();
  movieIn->setName("Movie input");

  videoOut = new dmuVideoOut();
  videoOut->setName("Video Output");

  // -- Wire --
  // Wire up the path (video->display)
  movieIn->addObject(videoOut);

  // -- Setup --

  //   -- Begin Video Object Setup ----

  dmuCheck(videoOut->setup(VL_ANY));
  dmuCheck(videoOut->setPacking(VL_PACKING_ABGR_8));
  dmuCheck(videoOut->setFormat(VL_FORMAT_RGB));
  dmuCheck(videoOut->setLayout(VL_LAYOUT_GRAPHICS));

  // Get the size of a single frame of video 
  videoOut->getImageDimensions(&sizeX,&sizeY);
  fprintf (stderr, "Video Image Size: %d, %d\n\n",sizeX,sizeY);

  dmuCheck(movieIn->setup(argv[1]));

  // Set the buffer size to video size, so we can send the movie 
  // directly to video.
  dmuCheck(movieIn->setBufferDimensions(sizeX,sizeY));

  // We can set the offset of the movie rendering, so that it's in the
  // middle of the video output.
  movieIn->setImageOffset( (sizeX - movieIn->getImageSizeX())/2,
			   (sizeY - movieIn->getImageSizeY())/2);
    
  //   -- End Video Setup ---

  // Init only has to be called at the head of the chain.
  fprintf (stderr, "Initializing: \n");
  dmuCheck(movieIn->init());

  // -- Start --

  // Start only has to be called at the head of the chain.  The video
  // transfer has now started.
  fprintf (stderr, "Fire it up!: \n");
  dmuCheck(movieIn->start());

  // -- Begin Handle events loop --
  for (int i = 0; ; i++){
    int maxFD = 0;

    // Setup our file descriptor set
    FD_ZERO(&readFDSet);
    movieIn->setFD(&readFDSet, &maxFD);
    videoOut->setFD(&readFDSet, &maxFD);

    // Wait for something to happen on the video path
    select (maxFD + 1,&readFDSet, NULL, NULL, NULL);
    
    movieIn->handleEvents(readFDSet);
    videoOut->handleEvents(readFDSet);

  }
  // -- End Handle events loop --

  // On exit, we should destroy the objects in the chain (except we'll 
  // never get here, because we were in an infite loop above :)
  //  delete (movieIn);
  //  delete (videoOut);
}
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
