#include "dmu.h"
#include "dmuObj.h"
#include "dmuNetworkRecv.h"
#include "dmuGLDisplay.h"
#include "dmuICjdecoder.h"

#include "pentTCPSocketServer.h"
#include "pentUDPSocketServer.h"
#include "pentData.h"

#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
					     /*exit(-1);*/ \
    }\
    else { \
       fprintf(stderr,"No Error\n"); \
    } \
 }

int main(int argc, char *argv[]){
  // dmu objects
  dmuNetworkRecv *netRecv;
  dmuGLDisplay *display;
  dmuICjdecoder *decoder;
  int imgSizeX = 640;
  //int imgSizeX = 320;
  int imgSizeY = 480;
  //int imgSizeY = 240;
  
  int video1FD = 0;
  fd_set readFDSet;

  // pent objects
  pentUDPSocketServer *netServer;
  pentData            *netData;

  // dmu setup
  netRecv = new dmuNetworkRecv();
  netRecv->setName("netRecv");
  decoder = new dmuICjdecoder();
  decoder->setName("decoder");
  display = new dmuGLDisplay();
  
  netRecv->addObject(decoder);
  decoder->addObject(display);
  //netRecv->addObject(display);

  netRecv->setPort(atoi(argv[1]));

  netRecv->setImageDimensions(imgSizeX, imgSizeY);
  netRecv->setup();

  decoder->setImageDimensions(imgSizeX, imgSizeY);
  decoder->setup();

  display->setImageDimensions(imgSizeX, imgSizeY);
  display->setPacking( GL_YCRCB_422_SGIX);
  display->setup();
  

  // network setup
  netServer = new pentUDPSocketServer();
  netServer->setName("netServer");
  
  netServer->setPort(atoi(argv[1]));
  netServer->setup();
  fprintf (stderr, "server setup finished\n");

  netData = new pentData();
  netData->setPrintReturnErr(PENT_TRUE);
  netData->setName("netRecvDataObject");
  netData->setHeaderSize(4);

  netServer->handshake(netData);
  fprintf (stderr, "done w/ server handshake\n");

  netRecv->setPentData(netData);

  // dmu startup
  netRecv->init();
  netRecv->start();

  int maxFD= 0;
  for (int i = 0; i < 5000; i ++) {
    // Setup our file descriptor set
    FD_ZERO(&readFDSet);
    netRecv->setFD(&readFDSet, &maxFD);
    //decoder->setFD(&readFDSet, &maxFD);
    
    // block until one of the fd's has some data on it
    select (maxFD+1, &readFDSet, NULL, NULL, NULL);

    netRecv->handleEvents(readFDSet);
    fprintf(stderr, "before decoder handle events\n");
    decoder->handleEvents(readFDSet);
    fprintf(stderr, "after decoder handle events\n");
  }
}



/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
