#include "dmu.h"
#include "dmuObj.h"
#include "dmuVideoIn.h"
#include "dmuNetworkSend.h"
#include "dmuGLDisplay.h"
#include "pentUDPSocketClient.h"
#include "pentData.h"
#include <dmedia/vl_mvp.h>
#include <dmedia/vl.h>
#include "dmuICjencoder.h"

#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
					     /*exit(-1);*/ \
    }\
    else { \
       fprintf(stderr,"No Error\n"); \
    } \
 }

int main(int argc, char *argv[]){
  dmuVideoIn *video1;
  dmuNetworkSend *netSend;
  dmuGLDisplay *display;
  dmuICjencoder *encoder;
  pentUDPSocketClient *client;
  pentData *dataObj;

  int sizeX, sizeY;
  int imgSizeX = 640;
  //int imgSizeX = 320;
  int imgSizeY = 480;
  //int imgSizeY = 240;

  char *hostname = argv[1];
  int port = atoi(argv[2]);
  int compressionQuality = atoi(argv[3]);
   
  client = new pentUDPSocketClient();
  client->setPrintReturnErr(PENT_TRUE);
  client->setName("client");
  client->setPort(port);
  client->setAddr(hostname);
  client->setup();
  fprintf(stderr, "done client network setup\n");

  dataObj = new pentData();
  dataObj->setPrintReturnErr(PENT_TRUE);
  dataObj->setName("dataObj");
  dataObj->setHeaderSize(4);

  client->handshake(dataObj);
  fprintf(stderr, "done with handshake on the server side\n");
  
  video1 = new dmuVideoIn();
  video1->setName("Video1");
  encoder = new dmuICjencoder();
  encoder->setName("encoder");
  netSend = new dmuNetworkSend();
  netSend->setName("netSend");
  display = new dmuGLDisplay();

  video1->addObject(encoder);
  video1->addObject(display);
  //  video1->addObject(netSend);
  encoder->addObject(netSend);

  dmuCheck(video1->setup(VL_ANY));
  dmuCheck(video1->setPacking(VL_PACKING_YVYU_422_8));
  //  dmuCheck(video1->setZoom(1, 2));
  video1->getImageDimensions(&sizeX, &sizeY);
  fprintf(stderr, "image size:(%d, %d)\n", sizeX, sizeY);

  encoder->setImageDimensions(sizeX, sizeY);
  encoder->setup();
  encoder->setQuality(compressionQuality);
  
  netSend->setHostname(hostname);
  netSend->setPort(port);
  netSend->setup();
  netSend->setPentData(dataObj);

  display->setImageDimensions(imgSizeX, imgSizeY);
  display->setPacking( GL_YCRCB_422_SGIX);
  display->setup();


  dmuCheck(video1->init());

  dmuCheck(video1->start());


  fprintf(stderr, "About to enter event loop\n");
  fd_set readFDSet;
  int maxFD;
  for (int i = 0; i < 2000; i++) {
    FD_ZERO(&readFDSet);
    video1->setFD(&readFDSet, &maxFD);
    
    select(maxFD + 1, &readFDSet, NULL, NULL, NULL);

    video1->handleEvents(readFDSet);
    encoder->handleEvents(readFDSet);
  }

  delete(video1);
  delete(netSend);
}


/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
