//
//  pentData.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
#ifndef PENT_DATA_H
#define PENT_DATA_H

#include "pentCommon.h"

#define PENT_NET_PACKET_MAX_SIZE 12

typedef struct _packetInfo {
  int size;
  int totalSize;
  unsigned int num;
  unsigned int totalNum;
  unsigned int id;
} PacketInfo;

class pentData {
public:
  pentData();
  ~pentData();

  int setDstAddr(struct sockaddr *dstAddr, int dstAddrLen);

  // give the data object an fd to talk over.
  // The user should never call this, instead, pass the data object
  // to the sockets class in the handshake call.
  int setFD(int fd);
  int getFD(int *fd);

  // this will tell this object if the connection it is using is a reliable
  // connection like TCP or an unreliable one like UDP
  int setConnType(int connType);
  int getConnType(int *connType);
  
  // with this call, the user program specifies the size of the header to
  // send across the network.  This must be the same on both sides of the
  // network.  It is up to the user program to ensure that the header size
  // stays the same.  
  int setHeaderSize(int headerSize);
  // this will merely return the current header size
  int getHeaderSize(int *headerSize);

  //
  // SENDING SIDE:
  // 
  // the user program sends the actual data with this call.  Before this
  // call will work, you must have given the data object an fd - probably
  // by passing the data object to the socket class handshake method.  You
  // must also have setup the header size for this class to expect.
  // Arguments:
  //    headerBuf:    pointer to the data to send as the header
  //    dataBuf:      pointer to the data to send as the data
  //    dataBufSize:  Size of the data you want to send
  int sendData(void *headerBuf, void *dataBuf, int dataBufSize);

  //
  // RECEIVING SIDE:
  //
  // get the header for the next packet.
  int getHeader(void *headerBuf);
  // get the size of the data for the next packet
  int getSize(int *dataBufSize);
  // Get the data portion of the packet
  int getData(void *dataBuf);
  // get both the data and the size of the next packet
  int getData(void *dataBuf, int *dataBufSize);

  // DEBUGGING-
  // give the object a name for debugging printing
  int setName(char *name);
  // return the name of the object
  char *getName() {  return name;  }
  // print out when you try to return an error with the PENT_ERR macro
  int setPrintReturnErr(int printReturnErr);

protected:
  PacketInfo   packetInfo;
  int     fd;
  char   *name;
  int     headerSize;
  int     curDataSize;
  int     connType;
  int     printReturnErr;
  int     dstAddrLen;
  int     currentID;
  struct sockaddr *dstAddr;

  int readHeader;
  int readSize;
  int readIOV(struct iovec *iov, int iovLen, int *totalRead, int flags = 0);
};

#endif
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
