//
//  pentTCPSocketServer.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// This object allows you to create a UDP connection across the
// network.  This object is a server, meaning that it has to be waiting
// for a connection before the client object tries to connect.  So once you
// setup this object with the address and call setup, you call handshake
// which waits for the client connection.
// This object uses the TCP network protocol.
//
#ifndef PENT_TCP_SOCKET_SERVER_H
#define PENT_TCP_SOCKET_SERVER_H

#include "pentCommon.h"
#include "pentSocket.h"
#include "pentData.h"

class pentTCPSocketServer : public pentSocket {
public:
  // Constructor: Initialize data members
  pentTCPSocketServer();
  // Destructor: 
  ~pentTCPSocketServer();

  // This method actually create the socket.  You need to have the ports
  // setup before you call setup or a port will be picked automatically.
  // Then you can get the port back from the getPort call.
  // You must call this method before calling handshake.
  int setup();

  // When you call this method it blocks waiting until you get a connection.
  // Once it has a connection, this method passes the file descriptor
  // to the data object you supply.  At this point you can delete this
  // object, or use it again to wait for another connection.  To wait
  // for another connection, call handshake again.
  int handshake(pentData *dataObj);

private:
  // socket(2) file descriptor
  int srvSocketFD;

  // Number of requests allowed to
  // backup in queue before returning ECONNREFUSED
  // (max 1024)
  int backlog;
};

#endif
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
