//
//  pentUDPSocketClient.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// This object allows you to create a UDP connection across the network.
// Because this object is the client, you must have a server running
// before you can use this object.  Once you setup the address for this
// object and call setup, you call handshake to try to talk to the server.
// This class uses the UDP network protocol to talk to the server.
// IMPORTANT: UDP is an unreliable network protocol.  
//
#ifndef PENT_UDP_SOCKET_CLIENT_H
#define PENT_UDP_SOCKET_CLIENT_H

#include "pentCommon.h"
#include "pentSocket.h"
#include "pentData.h"

class pentUDPSocketClient : public pentSocket {
public:
  // Constructor: Initialize data members
  pentUDPSocketClient();
  // Destructor:
  ~pentUDPSocketClient();

  // This method tells the object the name of the server to send data
  // to.  Most of the time you pass the hostname to this method, but
  // you can give it an IP address (in Dot notation).
  int setAddr(char *addr);

  // This method actually create the socket.  You need to have the ports
  // setup before you call setup or a port will be picked automatically.
  // Then you can get the port back from the getPort call.
  // You must call this method before calling handshake.
  int setup();

  // This call will try to establish a connection with the server you
  // specified with the setAddr() call.  It will use the port you
  // specified with the setPort() call.  Once the connection has
  // happened, the "connection" is passed on to the dataObj that you
  // provide.  From that point, you use the dataObj to send data
  // back and forth across the wire.
  int handshake(pentData *dataObj);

private:
  unsigned long serverAddr;
  int srvSockFD;
  struct sockaddr_in *srvAddr;
};

#endif
/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/

