//
//  pentData.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
#include "pentData.h"

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
//
// Constructor. Does nothing beyond initaliziation of
// member variables
//
pentData::pentData() {
  name = NULL;
  fd = -1;
  headerSize = 0;
  curDataSize = 0;
  connType = PENT_CONNECTION_TYPE_UNKNOWN;
  printReturnErr = PENT_FALSE;
  dstAddr = NULL;
  readHeader = PENT_FALSE;
  readSize = PENT_FALSE;
  currentID = 0;
}

//----------------------------------------------------------
// ~pentData
//----------------------------------------------------------
//
// Destructor. closes the connection if one has been established.
//
pentData::~pentData() {

  // if we're connected...
  if (fd != -1) {
    struct linger l;

    l.l_onoff = 0;
    l.l_linger = 0;

    // ask the kernel to discard any undelivered data when
    // we close the file descriptor associated with this socket...
    setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *)&l,sizeof (linger));

    // close it...
    close (fd);
  }
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::setFD(int _fd) {
  fd = _fd;
  PENT_D2(fprintf(stderr, "(%s) pentData::setfd newFD:%d\n", name, fd));

  return(PENT_OK);
}


//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getFD(int *_fd) {
  *_fd = fd;

  return(PENT_OK);
}


//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::setConnType(int _connType) {
  connType = _connType;

  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getConnType(int *_connType) {
  *_connType = connType;

  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::setHeaderSize(int _headerSize) {
  PENT_ERR( (_headerSize < 0)                , PENT_RANGE_ERR);

  PENT_D3(fprintf(stderr, "(%s) + setHeaderSize\n", name));
  PENT_D3(fprintf(stderr, "(%s)   headerSize:%d\n", name, headerSize));

  headerSize = _headerSize;
  
  PENT_D3(fprintf(stderr, "(%s) - setHeaderSize\n", name));
  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getHeaderSize(int *_headerSize) {
  *_headerSize = headerSize;
  
  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::setDstAddr(struct sockaddr *_dstAddr, int _dstAddrLen) {

  dstAddr = _dstAddr;
  dstAddrLen = _dstAddrLen;
  
  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::sendData(void *_headerBuf, void *_dataBuf, int _dataBufSize) {
  struct iovec    iov[3];
  struct msghdr   msg;
  int             bytesSent;
  PacketInfo      packetInfo;

  PENT_D2(fprintf(stderr, "(%s) + sendData:\n", name));

  // ensure that we've got valid info...
  PENT_ERR( (fd < 0)                        , PENT_NO_FD_ERR);
  PENT_ERR( (headerSize < 0)                , PENT_RANGE_ERR);
  PENT_ERR( (_dataBufSize < 0)              , PENT_RANGE_ERR);
  //  PENT_ERR( (_dataBuf == NULL)              , PENT_GENERIC_ERR);
  
  // next we need to set up the msghdr structure so that we can use the
  // sendmsg call
  // package the iov structs for use...
  msg.msg_iov    = iov;
  msg.msg_iovlen = 3;
  if (dstAddr) {
    // if this is a connectionless socket, we need to setup the address
    // in the msghdr structure.  
    msg.msg_name    = (caddr_t)dstAddr;
    msg.msg_namelen = dstAddrLen;
  } else {
    msg.msg_name    = NULL;
    msg.msg_namelen = 0;
  }
  msg.msg_accrights    = NULL;
  msg.msg_accrightslen = 0;


  // set the current id for the packet and then increment the current id
  // This allows us to check to make sure we get all of the parts of a buffer
  // on the recieve side, before we start getting new stuff.  
  packetInfo.id = currentID++;

  // setup the packet info with the total size for the data packet passed
  // in..
  packetInfo.totalSize = _dataBufSize;

  // save the total size so we can decrement it as we send parts of the
  // buffer across the wire.
  int totalSize = _dataBufSize;

  // setup the packet in to hold the total number of fragments we
  // need to send to send the whole buffer
  packetInfo.totalNum = (_dataBufSize / PENT_NET_PACKET_MAX_SIZE) + 1;

  // we want to give a number to each fragment.  So init that number to 1
  int packetCount = 1;
  // set packet info to hold the current fragment count
  packetInfo.num = packetCount;

  // Because we are fragmenting this, the packet size is the max of the
  // current totalSize and the PENT_NET_PACKET_MAX_SIZE
  // totalSize decrements by PENT_NET_PACKET_MAX_SIZE for every
  // fragment we send.  
  if (totalSize < PENT_NET_PACKET_MAX_SIZE) {
    packetInfo.size = totalSize;
  } else {
    packetInfo.size = PENT_NET_PACKET_MAX_SIZE;
  }

  // keep a pointer to the data that we can modify.  This lets us send
  // the correct data for each fragment
  void *dataPtr = _dataBuf;

  // set up the header to send
  // io vector0: user specified header
  iov[0].iov_base = _headerBuf;
  iov[0].iov_len  = headerSize;

  // we need to send the size of the data portion
  // so set up the data size
  // io vector1: size of the user data not including any header size
  //  iov[1].iov_base = (void *) &_dataBufSize;
  //  iov[1].iov_len = sizeof(_dataBufSize);
  iov[1].iov_base = (void *) &packetInfo;
  iov[1].iov_len  = sizeof(PacketInfo);
  
  // Now we can set up the data buffer to send.
  // io vector2: data 
  iov[2].iov_base = dataPtr;
  iov[2].iov_len  = packetInfo.size;

  PENT_D1(fprintf(stderr, "(%s)writing the data:%d\n",name,packetInfo.size));

  // send it... send all of the io vectors together in one go
  bytesSent = sendmsg(fd, &msg, 0);
  
  // make sure no error was returned
  PENT_ERR( (bytesSent < 0)            ,PENT_CONNECTION_CLOSED_ERR);
  // make sure that the number of bytes send = the number we should have sent
  PENT_ERR( (bytesSent != (sizeof(PacketInfo)+headerSize+packetInfo.size)),
	    PENT_DATA_WRITE_ERR);
    


  // now we are setting up to send more fragments if necessary
  totalSize -= PENT_NET_PACKET_MAX_SIZE;
  packetCount++;
  dataPtr = ((char *)dataPtr) + packetInfo.size;
  
  msg.msg_iovlen = 2;

  while (totalSize > 0) {
    fprintf(stderr, "totalsize:%d\n", totalSize);
    packetInfo.num       = packetCount;
    if (totalSize < PENT_NET_PACKET_MAX_SIZE) {
      packetInfo.size = totalSize;
    } else {
      packetInfo.size = PENT_NET_PACKET_MAX_SIZE;
    }
    iov[0].iov_base = (void *) &packetInfo;
    iov[0].iov_len  = sizeof(PacketInfo);
  
    iov[1].iov_base = dataPtr;
    iov[1].iov_len  = packetInfo.size;

    PENT_D1(fprintf(stderr, "(%s)writing the data:%d\n",name,packetInfo.size));

    // send it... send all of the io vectors together in one go
    bytesSent = sendmsg(fd, &msg, 0);
  
    // make sure no error was returned
    PENT_ERR( (bytesSent < 0)            ,PENT_CONNECTION_CLOSED_ERR);
    // make sure that the number of bytes send = the number we should have sent
    PENT_ERR( (bytesSent != (sizeof(PacketInfo)+packetInfo.size)),
	      PENT_DATA_WRITE_ERR);
    
    totalSize -= PENT_NET_PACKET_MAX_SIZE;
    packetCount++;
    dataPtr = ((char *)dataPtr) + packetInfo.size;
  }

  // debugging printouts
  PENT_D3({
    for(int i = 0; i < 1; i++ ){
      fprintf(stderr, "(%s) header[%d]:%d\n", name, i, ((int *)_headerBuf)[i]);
    }
    for(i = 0; i < 16; i++ ){
      fprintf(stderr, "(%s) data[%d]:%d\n", name, i, ((char *)_dataBuf)[i]);
    }
  });

  return(PENT_OK);
}


//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getHeader(void *_headerBuf) {
  PENT_ERR( (fd < 0)                        , PENT_NO_FD_ERR);
  PENT_ERR( (headerSize < 0)                , PENT_RANGE_ERR)
  PENT_ERR( (_headerBuf == NULL)            , PENT_GENERIC_ERR);

  PENT_D2(fprintf(stderr, "(%s) + getHeader:\n", name));

  PENT_D2(fprintf(stderr, "(%s)   headerSize:%d\n", name, headerSize));

  // checks to make sure we do things in the right order
  assert(readHeader == PENT_FALSE);
  assert(readSize   == PENT_FALSE);

  int err;
  int totalRead;
  if (connType == PENT_CONNECTION_TYPE_STREAM) {
    PENT_D3(fprintf(stderr, "(%s)   stream connection:\n", name));
    // if we are using a TCP stream socket
    //
    struct iovec  iov[1];
    // setup the iov to fill in the header from the stream
    iov[0].iov_base = _headerBuf;
    iov[0].iov_len  = headerSize;

    // go and read the data from the socket
    err = readIOV(iov, 1, &totalRead);

    // make sure there is no error
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);

    // because this is a stream, we need to track what we have read out
    // of the stream.  So now we have read the header
    readHeader = PENT_TRUE;
    
  } else if (connType == PENT_CONNECTION_TYPE_DGRAM) {
    PENT_D3(fprintf(stderr, "(%s)   dgram connection\n", name));
    // if we are using a UDP datagram
    //

    // if we don't have the return address, fetch it from the first
    // packet.
    if (!dstAddr) {
      char tmpBuf[4];
      struct sockaddr *returnAddr;
      returnAddr = (struct sockaddr *)malloc(sizeof(struct sockaddr_in));
      int returnAddrLen = sizeof(struct sockaddr_in);
      err = recvfrom(fd, tmpBuf, 4, MSG_PEEK, returnAddr, &returnAddrLen);
      PENT_ERR( (err < 0)                    , PENT_DATA_READ_ERR);

      dstAddr = returnAddr;
      dstAddrLen = returnAddrLen;
      fprintf (stderr, "SETTING DEST ADDR\n");
    }


    struct iovec iov[1];
    // setup the iov to fill in the header from the datagram
    iov[0].iov_base = _headerBuf;
    iov[0].iov_len  = headerSize;

    // go read the data from the datagram, but only peek at the data
    // so we can read more of it later
    err = readIOV(iov, 1, &totalRead, MSG_PEEK);
    
    // make sure there is no error
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);

    // we need to track what we have read out of the stream.
    // So now we have read the header    
    readHeader = PENT_TRUE;
  }

  PENT_D3(fprintf(stderr, "(%s) - getHeader:\n", name));
  
  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getSize(int *_dataBufSize) {
  PENT_ERR( (fd < 0)                        , PENT_NO_FD_ERR);
  PENT_ERR( (headerSize < 0)                , PENT_RANGE_ERR);

  PENT_D2(fprintf(stderr, "(%s) + getSize:\n", name));

  // checks to make sure we do things in the right order
  assert(readHeader == PENT_TRUE);
  assert(readSize   == PENT_FALSE);
  
  int err;
  int totalRead;
  if (connType == PENT_CONNECTION_TYPE_STREAM) {
    PENT_D3(fprintf(stderr, "(%s)   stream connection:\n", name));
    struct iovec  iov[1];
    //    iov[0].iov_base = _dataBufSize;
    //    iov[0].iov_len = sizeof(_dataBufSize);
    iov[0].iov_base = (void *)&packetInfo;
    iov[0].iov_len  = sizeof(PacketInfo);
    
    err = readIOV(iov, 1, &totalRead);
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);
    //    curDataSize = *_dataBufSize;
    curDataSize = packetInfo.size;
    *_dataBufSize = packetInfo.totalSize;
    fprintf(stderr, "packetInfo.size:%d\n", packetInfo.size);
    fprintf(stderr, "packetInfo.totalSize:%d\n", packetInfo.totalSize);
    fprintf(stderr, "packetInfo.num:%d\n", packetInfo.num);
    fprintf(stderr, "packetInfo.totalNum:%d\n", packetInfo.totalNum);
    readSize = PENT_TRUE;
  } else if (connType == PENT_CONNECTION_TYPE_DGRAM) {
    PENT_D3(fprintf(stderr, "(%s)   dgram connection:\n", name));
    char tmpBuf[256];
    PENT_ERR( (headerSize > 256)            , PENT_RANGE_ERR);
    struct iovec iov[2];
    
    iov[0].iov_base = tmpBuf;
    iov[0].iov_len  = headerSize;
    //    iov[1].iov_base = (void *)_dataBufSize;
    //    iov[1].iov_len = sizeof(*_dataBufSize);    
    iov[0].iov_base = (void *)&packetInfo;
    iov[0].iov_len  = sizeof(PacketInfo);

    err = readIOV(iov, 2, &totalRead, MSG_PEEK);
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);
    //    curDataSize = *_dataBufSize;
    curDataSize = packetInfo.size;
    *_dataBufSize = packetInfo.totalSize;
    readSize = PENT_TRUE;
  }

  PENT_D3(fprintf(stderr, "(%s) - getSize:\n", name));

  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getData(void *_dataBuf) {
  PENT_ERR( (fd < 0)                        , PENT_NO_FD_ERR);
  PENT_ERR( (headerSize < 0)                , PENT_RANGE_ERR);
  //   PENT_ERR( (_dataBuf == NULL)              , PENT_GENERIC_ERR);

  PENT_D2(fprintf(stderr, "(%s) + getData:\n", name));

  PENT_D2(fprintf(stderr, "(%s)   datasize:%d\n", name, curDataSize));
  
  // checks to make sure we do things in the right order
  assert(readHeader == PENT_TRUE);
  assert(readSize == PENT_TRUE);

  void *dataPtr = _dataBuf;
  int err;
  int totalRead;
  if (connType == PENT_CONNECTION_TYPE_STREAM) {
    PENT_D3(fprintf(stderr, "(%s)   stream connection:\n", name));
    struct iovec  iov[1];
    iov[0].iov_base = (void *)dataPtr;
    iov[0].iov_len = curDataSize;

    err = readIOV(iov, 1, &totalRead);
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);
    
    dataPtr = ((char *)dataPtr) + packetInfo.size;
    
    readHeader = PENT_FALSE;
    readSize = PENT_FALSE;
  } else if (connType == PENT_CONNECTION_TYPE_DGRAM) {
    PENT_D3(fprintf(stderr, "(%s)   dgram connection:\n", name));
    char tmpBuf[256];
    int  tmpInt;
    PENT_ERR( (headerSize > 256)            , PENT_RANGE_ERR);
    struct iovec iov[3];
    
    iov[0].iov_base = tmpBuf;
    iov[0].iov_len  = headerSize;
    iov[1].iov_base = (void *)&tmpInt;
    iov[1].iov_len  = sizeof(tmpInt);
    iov[2].iov_base = dataPtr;
    iov[2].iov_len  = curDataSize;    

    err = readIOV(iov, 3, &totalRead);
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);
    readHeader = PENT_FALSE;
    readSize = PENT_FALSE;
  }

  while (packetInfo.num != packetInfo.totalNum) {
    struct iovec  iov[2];

    iov[0].iov_base = (void *) &packetInfo;
    iov[0].iov_len  = sizeof(PacketInfo);
    
    iov[1].iov_base = (void *)dataPtr;
    iov[1].iov_len = curDataSize;

    err = readIOV(iov, 2, &totalRead);
    PENT_ERR( (err != PENT_OK)              , PENT_DATA_READ_ERR);

    dataPtr = ((char *)dataPtr) + packetInfo.size;
  }


  PENT_D3(fprintf(stderr, "(%s) - getData:\n", name));

  return(PENT_OK);
}

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::getData(void *_dataBuf, int *_dataBufSize) {
  PENT_ERR( (fd < 0)                        , PENT_NO_FD_ERR);
  PENT_ERR( (headerSize < 0)                , PENT_RANGE_ERR);
  PENT_ERR( (_dataBuf == NULL)              , PENT_GENERIC_ERR);
  return(PENT_OK);
  
}

//****************************************************************
//
//  Protected methods
//
//****************************************************************

//----------------------------------------------------------
// pentData
//----------------------------------------------------------
int pentData::readIOV(struct iovec *_iov, int _iovLen, int *_totalRead,
		      int flags) {
  struct msghdr msg;
  int numRead = 0;
  int totalRead = 0;
  int numLeft = 0;
  int offset = 0;
  
  PENT_D2(fprintf(stderr, "(%s)      readIOV:\n", name));

  // check initial conditions
  PENT_ERR( (fd < 0)                        , PENT_NO_FD_ERR);
  PENT_ERR( (_iov == NULL)                  , PENT_GENERIC_ERR);

  // we haven't read anything yet.
  *_totalRead = 0;

  // setup the msg structure
  msg.msg_iov = _iov;
  msg.msg_iovlen = _iovLen;
  msg.msg_name = NULL;
  msg.msg_namelen = 0;
  msg.msg_accrights = NULL;
  msg.msg_accrightslen = 0;

  PENT_D3(fprintf(stderr, "(%s)      readingIOV:\n", name));

  if (_iovLen == 1) {
    // If we only have one iov then we assume that it's
    // a stream.  So we have to make sure all the data is there
    // before we return
    offset = 0;
    numRead = 0;
    totalRead = 0;
    numLeft = _iov[0].iov_len;
    while (numLeft > 0) {
      numRead = recvmsg(fd, &msg, flags);
      PENT_ERR( (numRead < 0)               , PENT_DATA_READ_ERR);
      PENT_ERR( (numRead == 0)              , PENT_CONNECTION_CLOSED_ERR);
      totalRead += numRead;
      numLeft -= numRead;
      offset += numRead;
      _iov[0].iov_base = (((char *)_iov[0].iov_base) + offset);
      _iov[0].iov_len  = numLeft;
    }

  } else {
    // because iov is not 1 we now that we are dealing with a whole packet
    // and we are no longer waiting for 
    
    numRead = recvmsg(fd, &msg, flags);
    PENT_ERR( (numRead < 0)               , PENT_DATA_READ_ERR);
    PENT_ERR( (numRead == 0)              , PENT_CONNECTION_CLOSED_ERR);
    totalRead = numRead;
  }

  *_totalRead = totalRead;
  
  return(PENT_OK);
}


//****************************************************************
//
//  Debug methods
//
//****************************************************************

// used in the debug macro to see if we want to print the
// error before we return
int pentData::setPrintReturnErr(int _printReturnErr) {
  printReturnErr = _printReturnErr;

  return(PENT_OK);
}

// give the object a name so we can print out use full information
int pentData::setName(char *_name) {
  if (_name) {
    name = strdup(_name);
  }
  return(PENT_OK);
}

/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
