//
//  pentSocket.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
//  See header file for description of this class
//
#include "pentSocket.h"

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// pentSocket
//----------------------------------------------------------
//
// Constructor. Does nothing beyond initaliziation of
// member variables
//
pentSocket::pentSocket() {
  portNum =     0;
  name =        "NULL";
  localhostname = NULL;
  connecthostname = NULL;
}

//----------------------------------------------------------
// ~pentSocket
//----------------------------------------------------------
//
// Destructor. Does nothing
//
pentSocket::~pentSocket() {
}

//----------------------------------------------------------
// setPort
//----------------------------------------------------------
//
// give the object the port number to connect on
//
int pentSocket::setPort(int _portNum) {
  // check initial conditions
  PENT_ERR( (_portNum < 0), PENT_RANGE_ERR);

  // set the new port number
  portNum = _portNum;

  return(PENT_OK);
}

//----------------------------------------------------------
// setPort
//----------------------------------------------------------
//
// return the port number that the object is using to
// connect
//
int pentSocket::getPort(int *_portNum) {
  // return the portnumber
  *_portNum = portNum;

  return(PENT_OK);
}

//----------------------------------------------------------
// setup
//----------------------------------------------------------
//
// This is a virtual function that needs to be overridden
//
int pentSocket::setup() {
  fprintf(stderr, "(%s) PENT error:%d (line:%d file:%s)\n", name,
	  PENT_VIRTUAL_CALLED_ERR, __LINE__, __FILE__);
  return(PENT_VIRTUAL_CALLED_ERR);
}

//----------------------------------------------------------
// handshake
//----------------------------------------------------------
//
// This is a virtual function that needs to be overridden
//
int pentSocket::handshake(pentData *_dataObj) {
  fprintf(stderr, "(%s) PENT error:%d (line:%d file:%s)\n", name,
	  PENT_VIRTUAL_CALLED_ERR, __LINE__, __FILE__);
  return(PENT_VIRTUAL_CALLED_ERR);
}

//****************************************************************
//
//  Debugging methods
//
//****************************************************************

// used in the debug macro to see if we want to print the
// error before we return
int pentSocket::setPrintReturnErr(int _printReturnErr) {
  printReturnErr = _printReturnErr;

  return(PENT_OK);
}

// give the object a name so we can print out use full information
int pentSocket::setName(char *_name) {
  if (_name) {
    name = strdup(_name);
  }
  return(PENT_OK);
}

/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/

