//
//  pentTCPSocketServer.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
//  See header file for description of this class
//
#include "pentTCPSocketServer.h"

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// pentTCPSocketServer
//----------------------------------------------------------
//
// Constructor. Does nothing beyond initaliziation of
// member variables
//
pentTCPSocketServer::pentTCPSocketServer() {
  // Initialization (in case user fails to call setup -- avoids core in
  // destructor)
  srvSocketFD = -1;
  backlog = 5;
}

//----------------------------------------------------------
// ~pentTCPSocketServer
//----------------------------------------------------------
//
// Destructor. Closes the socket it if has been open
//
pentTCPSocketServer::~pentTCPSocketServer() {
  if (srvSocketFD != -1) {
    close (srvSocketFD);
  }
}

//----------------------------------------------------------
// setup
//----------------------------------------------------------
//
// This method actually creates the socket and does all of the stuff
// to get it in a state where you can send data across it
//
int pentTCPSocketServer::setup() {
  int status;

  // negative ports are not allowed, bail...
  PENT_ERR( (portNum < 0)        , PENT_TCP_CLIENT_SETUP_ERR);
  
  // open a TCP (SOCK_STREAM/AF_INET) socket
  // bail if unable to secure the socket...
  srvSocketFD = socket(AF_INET, SOCK_STREAM, AF_UNSPEC);
  PENT_ERR( (srvSocketFD < 0)    , PENT_TCP_SERVER_CREATE_ERR);
 
  // prep socket addressing info with network byte-order
  // addresses (htnol/htons) for bind...
  struct sockaddr_in srvAddr;
  memset((char *) &srvAddr, 0, sizeof(srvAddr));
  srvAddr.sin_family      = AF_INET;
  srvAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  srvAddr.sin_port        = htons(portNum);

  // name the socket, bail if there is a problem
  status = bind(srvSocketFD, (struct sockaddr *)&srvAddr, sizeof(srvAddr));
  PENT_ERR( (status < 0)            , PENT_TCP_SERVER_BIND_ERR);
  
  // specifiy max queue length (limited to 1024)
  status = listen(srvSocketFD, backlog);
  PENT_ERR( (status < 0)            , PENT_TCP_SERVER_LISTEN_ERR);

  // if the user specified dynamic port allocation (i.e. port 0)
  // fetch the 'bind' assigned socket info and retrieve the
  // port (in host byte-order)...
  if (portNum == 0) {
    struct sockaddr_in socketName;
    int nameSize = sizeof(socketName);
    getsockname (srvSocketFD, (struct sockaddr *)&socketName, &nameSize);
    portNum = ntohs(socketName.sin_port);
  }
  
  return(PENT_OK);
}

//----------------------------------------------------------
// handshake
//----------------------------------------------------------
//
// This method listens on the socket for a connection.  Once
// a connection is established, this object passes the fd to
// the data object passed in (_dataObj).  From this point
// on, use the data object to communicate over the socket.
// At this point you can safely destroy the pentTCPSocketServer
// object also.  You can also keep it around waiting for another
// connection by calling handshake again.  
//
int pentTCPSocketServer::handshake(pentData *_dataObj) {

  // bail if we haven't got a valid port...
  PENT_ERR( (srvSocketFD < 0)       , PENT_TCP_SERVER_ACCEPT_ERR);

  // bail if we've been given a bogus network (pent) data object
  PENT_ERR( (_dataObj == NULL)      , PENT_TCP_SERVER_SETUP_ERR);
  
  // otherwise wait for a connection...
  struct sockaddr_in clientAddr;
  int clientLen = sizeof(clientAddr);
  int clientSocketFD = accept(srvSocketFD,
			      (struct sockaddr *)&clientAddr, &clientLen);
  PENT_ERR( (clientSocketFD < 0)    , PENT_TCP_SERVER_ACCEPT_ERR);

  // pass connection information to the data object
  _dataObj->setFD(clientSocketFD);
  _dataObj->setConnType(PENT_CONNECTION_TYPE_STREAM);

  return(PENT_OK);
}






