//
//  pentUDPSocketClient.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
//  See header file for description of this class
// 
#include "pentUDPSocketClient.h"

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// pentUDPSocketClient
//----------------------------------------------------------
//
// Constructor. Does nothing beyond initaliziation of
// member variables
//
pentUDPSocketClient::pentUDPSocketClient() {
  serverAddr = 0;
  srvSockFD = -1;
  srvAddr = (struct sockaddr_in *)  malloc (sizeof(struct sockaddr_in));
}

//----------------------------------------------------------
// ~pentUDPSocketClient
//----------------------------------------------------------
//
// Destructor. Does nothing.
//
pentUDPSocketClient::~pentUDPSocketClient() {
}

//----------------------------------------------------------
// setAddr
//----------------------------------------------------------
//
// setup the address to send data to.  This method tweaks the
// address into something that it can actually use.
//
int pentUDPSocketClient::setAddr(char *_addr) {
  struct hostent *hostInfo;
    
  if ( (serverAddr = inet_addr(_addr)) == INADDR_NONE) {
    if ( (hostInfo = gethostbyname(_addr)) != NULL) {
      bcopy(hostInfo->h_addr, (char *) &serverAddr, hostInfo->h_length);
    } else {
      if (printReturnErr == PENT_TRUE) {
	fprintf(stderr, "PENT error:%d (line:%d file:%s)\n",
		PENT_BAD_ADDR_ERR, __LINE__, __FILE__);
      }
      return(PENT_BAD_ADDR_ERR);
    }
  }
  
  return(PENT_OK);
}

//----------------------------------------------------------
// setup
//----------------------------------------------------------
//
// This method actually creates the socket and does all of the stuff
// to get it in a state where you can send data across it
//
int pentUDPSocketClient::setup() {
   // check initial conditions
  PENT_ERR( (portNum < 0)     , PENT_TCP_CLIENT_SETUP_ERR);
  PENT_ERR( (portNum == 0)    , PENT_TCP_CLIENT_SETUP_ERR);
  PENT_ERR( (serverAddr == 0) , PENT_TCP_CLIENT_SETUP_ERR);

  // create a socket
  srvSockFD = socket(AF_INET, SOCK_DGRAM, 0);
  PENT_ERR( (srvSockFD < 0)   , PENT_TCP_CLIENT_CREATE_ERR);

  // setup the address of the server to connect to
  int srvAddrLen = sizeof(srvAddr);
  bzero ((char *) srvAddr, sizeof(struct sockaddr_in));
  srvAddr->sin_family         =  AF_INET;
  srvAddr->sin_addr.s_addr    =  serverAddr;
  srvAddr->sin_port           =  htons(portNum);

  // setup the address that we use locally.  
  struct sockaddr_in clientAddr;
  int clientAddrLen = sizeof(clientAddr);
  bzero ((char *) &clientAddr, sizeof(clientAddr));
  clientAddr.sin_family      =  AF_INET;
  clientAddr.sin_addr.s_addr =  htonl(INADDR_ANY);
  clientAddr.sin_port        =  htons(0);

  // grab the port that we asked for so the other side can talk back
  // to us
  int status;
  status = bind(srvSockFD, (struct sockaddr *)&clientAddr, sizeof(clientAddr));
  PENT_ERR( (status < 0)      , PENT_TCP_CLIENT_SETUP_ERR);

  // everythings cool.
  return(PENT_OK);
}

//----------------------------------------------------------
// handshake
//----------------------------------------------------------
//
int pentUDPSocketClient::handshake(pentData *_dataObj) {
  // check initial conditions
  PENT_ERR( (_dataObj == NULL)      , PENT_TCP_SERVER_SETUP_ERR);
  PENT_ERR( (srvSockFD < 0)         , PENT_TCP_SERVER_CREATE_ERR);  

  // initial setup of the data object provided
  _dataObj->setFD(srvSockFD);
  _dataObj->setConnType(PENT_CONNECTION_TYPE_DGRAM);

  // we need to send one packet to setup the return address on the
  // other side.  So send a bit of data.  
  char tmpBuf[1];
  tmpBuf[0] = 'a';
  int err = sendto(srvSockFD, tmpBuf, 1, 0, (struct sockaddr *)srvAddr,
		   sizeof(struct sockaddr_in));
  PENT_ERR( (err < 0)               , PENT_DATA_READ_ERR);


  // now we setup the address to send to for the data object
  _dataObj->setDstAddr((struct sockaddr *)srvAddr,sizeof(struct sockaddr_in));

  // everything's cool.
  return(PENT_OK);
}


/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
