//
//  pentUDPSocketServer.c++
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
//  See header file for description of this class
// 
#include "pentUDPSocketServer.h"

//****************************************************************
//
//  Public methods
//
//****************************************************************

//----------------------------------------------------------
// pentUDPSocketServer
//----------------------------------------------------------
//
// Constructor. Does nothing beyond initaliziation of
// member variables
//
pentUDPSocketServer::pentUDPSocketServer() {
  srvSocketFD = -1;
}

//----------------------------------------------------------
// ~pentUDPSocketServer
//----------------------------------------------------------
//
// Destructor. Does nothing.
//
pentUDPSocketServer::~pentUDPSocketServer() {
}

//----------------------------------------------------------
// setup
//----------------------------------------------------------
//
// This method actually creates the socket and does all of the stuff
// to get it in a state where you can send data across it
//
pentUDPSocketServer::setup() {
  struct sockaddr_in srvAddr;
  int status;

  // check initial conditions
  PENT_ERR( (portNum < 0)           , PENT_TCP_CLIENT_SETUP_ERR);

  // create socket and bail if the creation fails
  srvSocketFD = socket(AF_INET, SOCK_DGRAM, 0);
  PENT_ERR( (srvSocketFD < 0)          , PENT_TCP_SERVER_CREATE_ERR);

  // setup the address to listen on
  memset((char *) &srvAddr, 0, sizeof(srvAddr));
  srvAddr.sin_family      = AF_INET;
  srvAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  srvAddr.sin_port        = htons(portNum);

  // attach the socket to the address we just setup and as always, bail
  // if that fails
  status = bind(srvSocketFD, (struct sockaddr *)&srvAddr, sizeof(srvAddr));
  PENT_ERR( (status < 0)            , PENT_TCP_SERVER_BIND_ERR);

  // if we didn't set a port initially, one was assigned for
  // us so now we need to get the port back.  
  if (portNum == 0) {
    struct sockaddr_in socketName;
    int nameSize = sizeof(socketName);
    getsockname (srvSocketFD, (struct sockaddr *)&socketName, &nameSize);
    portNum = ntohs(socketName.sin_port);
  }

  return(PENT_OK);  
}

//----------------------------------------------------------
// handshake
//----------------------------------------------------------
//
pentUDPSocketServer::handshake(pentData *_dataObj) {
  // check initial conditions
  PENT_ERR( (_dataObj == NULL)      , PENT_TCP_SERVER_SETUP_ERR);
  PENT_ERR( (srvSocketFD < 0)       , PENT_TCP_SERVER_CREATE_ERR);  

  // setup the data object with the info it needs
  _dataObj->setFD(srvSocketFD);
  _dataObj->setConnType(PENT_CONNECTION_TYPE_DGRAM);

  // we don't have a return address yet, so fetch it from the first packet.  
  int err;
  char tmpBuf[1];
  struct sockaddr *clientAddr;
  clientAddr = (struct sockaddr *)malloc(sizeof(struct sockaddr_in));
  int clientAddrLen = sizeof(struct sockaddr_in);
  err = recvfrom(srvSocketFD, &tmpBuf, 1, 0, clientAddr, &clientAddrLen);
  PENT_ERR( (err < 0)                    , PENT_DATA_READ_ERR);

  // tell the data object what the return address is
  _dataObj->setDstAddr(clientAddr, clientAddrLen);

  return(PENT_OK);
}


/****************************************************************************
  *                                                                          *
  *               Copyright (c) 1996-97, Silicon Graphics, Inc.              *
  *                                                                          *
  *   Permission to use,  copy, modify, distribute, and sell this software   *
  *   and its documentation for any purpose is hereby granted without fee,   *
  *   provided that the name of  Silicon  Graphics  may not be used in any   *
  *   advertising  or  publicity  relating  to  the  software  without the   *
  *   specific, prior written permission of Silicon Graphics.                *
  *                                                                          *
  *   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
  *   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
  *   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
  *                                                                          *
  *   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
  *   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
  *   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
  *   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
  *   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
  *   PERFORMANCE OF THIS SOFTWARE.                                          *
  *                                                                          *
  ****************************************************************************/
