#include "dmu.h"
#include "dmuMovieInRender.h"
#include "dmuVideoIn.h"
#include "dmuGLCtx.h"
#include "ScratchWindow.h"

#include "dmuMovieInRender.h"
#include "PSound.h"
#include "PSoundFile.h"

#include <X11/GLw/GLwMDrawA.h>
#include <X11/keysym.h>

#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
               getErrorString(dmuErrID) ); \
               exit(-1); \
    }\
    else { \
             /*fprintf(stderr,"No Error\n");*/ \
    } \
}


int sizeX,sizeY;
int movieSizeX,movieSizeY;
int exploding;
int gotFirstFrame;

Display     *xDisplay;

dmuVideoIn    *videoIn;
dmuGLCtx      *videoCtx;
ScratchWindow *window;

dmuGLCtx         *explodeCtx;
dmuMovieInRender *movieIn;
PSoundFile       *sound;

void render(int framenum);
void drawCube(void);
void drawFace(Bool borders,float z);
int setupWindow(int argc, char *argv[]);

int main (int argc, char *argv[]){
  exploding = 0;
  gotFirstFrame = 0;

  xDisplay = XOpenDisplay(":0");

  sound = new PSoundFile;
  sound->loadFile("/hosts/vermouth/d2/eda/trees/demoTree/dmedia/framework/research/videoBlast/data/explosion.aiff");

  // -- construct --
  videoIn = new dmuVideoIn();
  videoCtx = new dmuGLCtx();
  

  movieIn = new dmuMovieInRender;
  explodeCtx = new dmuGLCtx();

  // -- wire --
  videoIn->addObject(videoCtx);

  movieIn->addObject(explodeCtx);

  // -- setup --

  // --- videoIn setup --
  videoIn->setup(VL_ANY, VL_VIDEO);
  videoIn->setPacking(VL_PACKING_ABGR_8);
  videoIn->setCaptureType(VL_CAPTURE_INTERLEAVED);
  videoIn->setLayout(VL_LAYOUT_GRAPHICS);
  videoIn->getImageDimensions(&sizeX, &sizeY);


  // --- videoContext setup --
  videoCtx->setup(xDisplay, sizeX, sizeY);


  movieIn->setup("/hosts/vermouth/d2/eda/trees/demoTree/dmedia/framework/research/videoBlast/data/explosionVeryBig.qt");
  movieIn->getImageDimensions(&movieSizeX,&movieSizeY);
  movieIn->setBufferDimensions(movieSizeX,movieSizeY);
  movieIn->disableLoop();
  
  explodeCtx->setup(xDisplay, movieSizeX,movieSizeY);

  // -- init
  videoIn->init();
  movieIn->init();

  // -- start
  videoIn->start();

  movieIn->start();
  movieIn->pause();
  movieIn->gotoFrame(0);
  
  // --Explosion


  setupWindow(argc,argv);
  window->setContext();
  GLXDrawable drawable = glXGetCurrentDrawable();

  int firstVideoFrame = 0;
  
  for (int i = 0 ;  ; i++){

    fd_set readFDSet;
    int maxFD = 0;
    int err;
    int refresh = 0;

    FD_ZERO(&readFDSet);
    videoIn->setFD(&readFDSet, &maxFD);
    movieIn->setFD(&readFDSet, &maxFD);
    
    select(maxFD + 1, &readFDSet, NULL, NULL, NULL);

    err = movieIn->handleEvents(readFDSet);
    if (!gotFirstFrame){
      if (err == DMU_OK){
	gotFirstFrame = 1;
	fprintf (stderr, "**************\n");
      }
    }

    err = videoIn->handleEvents(readFDSet);

    if (err == VIDEOIN_TRANSFER_COMPLETE){

      err = videoCtx->grab();
      if (err == DMU_OK ){
	videoCtx->makeCurrent();
	
	if (!firstVideoFrame){
	  glEnable (GL_BLEND);
	  glEnable(GL_DEPTH_TEST);

	  glViewport( 0, 0, sizeX, sizeY);
	  glMatrixMode (GL_PROJECTION);
	  glLoadIdentity ();
	  glOrtho(0,sizeX,0,sizeY,0,1000);
	  glDisable(GL_DEPTH_TEST);
	  glMatrixMode( GL_MODELVIEW );
	  glLoadIdentity();
	  firstVideoFrame = 1;
	}

	window->setContext();
	videoCtx->makeCurrentRead();
	glPixelZoom(1,1);
	glRasterPos2f(0,0);
	fprintf (stderr, ".");
	glCopyPixels(0,0,sizeX,sizeY,GL_COLOR);
	videoCtx->release();
	glLineWidth(5);

	glColor3f(1.0,0.0,0.0);
	glBegin(GL_LINES);

	glVertex3i(sizeX/2 - 60, sizeY/2,0);
	glVertex3i(sizeX/2 - 10, sizeY/2,0);

	glVertex3i(sizeX/2 + 60, sizeY/2,0);
	glVertex3i(sizeX/2 + 10, sizeY/2,0);

	glVertex3i(sizeX/2, sizeY/2 - 60, 0);
	glVertex3i(sizeX/2, sizeY/2 - 10, 0);

	glVertex3i(sizeX/2, sizeY/2 + 60, 0);
	glVertex3i(sizeX/2, sizeY/2 + 10, 0);

	glEnd();
	refresh = 1;
      }
    }
    else if (err == VIDEOIN_SHUTTER_EVENT){
      fprintf (stderr, "explode\n");
      movieIn->gotoFrame(0);
      movieIn->resume();
      exploding = 1;
      sound->startPlaying();
    }

    if (exploding && refresh){
      if (gotFirstFrame){
	explodeCtx->grab();
	explodeCtx->makeCurrent();
	window->setContext();
	explodeCtx->makeCurrentRead();
	//fprintf (stderr, "make current read\n");
	glEnable (GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_CONSTANT_COLOR_EXT);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
	glRasterPos2f(sizeX/2 - (movieSizeX/2),sizeY/2 - (movieSizeY/2));
	glBlendColorEXT(0,0,0,0);
	glCopyPixels(0,0,movieSizeX,movieSizeY, GL_COLOR);
	glDisable (GL_BLEND);
	explodeCtx->release();
	int currentFrame = movieIn->getCurrentFrameIndex();
	
	if (currentFrame >= 25){
	  movieIn->pause();
	  exploding = 0;
	  fprintf (stderr, "endExplode\n");
	}
      }
    }

    if (refresh){
      glXSwapBuffers(xDisplay, glXGetCurrentDrawable());
    }
  }

}



void render(int frameNumber) {

  glColor3f(1.0,0.0,0.5);

  glBlendFunc(GL_ONE_MINUS_CONSTANT_COLOR_EXT,GL_ONE);
  glBlendColorEXT(0.0,1.0,1.0,0.01);

  glClearColor(0,0,0,0);
  glClear(GL_DEPTH_BUFFER_BIT);

  glMatrixMode( GL_MODELVIEW );

  glPushMatrix();
  glTranslatef(sizeX/2,sizeY/2,0);
  glRotatef(frameNumber/2,0.0,1.0,0.0);
  glTranslatef(200,0,0);
  glRotatef(frameNumber/4,1.0,0.0,0.0);
  
  glScalef(0.2,0.2,0.2);
  drawCube();
  glPopMatrix();


  glPushMatrix();
  glTranslatef(sizeX/2,sizeY/2,0);
  glRotatef(frameNumber/2,1.0,0.0,0.0);
  glTranslatef(0,200,0);
  glRotatef(frameNumber/4,0.0,1.0,0.0);
  
  glScalef(0.2,0.2,0.2);
  drawCube();
  glPopMatrix();
}


void drawFace(Bool borders,float z) {

  glBegin( GL_QUADS );

  glVertex3i( -160, -160, z );
  glVertex3i( -160,  160, z );
  glVertex3i(  160,  160, z );
  glVertex3i(  160, -160, z );

  glEnd();
    
}

void drawCube(void) {
    glColor4f( 0.5, 0.5, 0.5, 1.0 );
    
    /* front */
    drawFace( False, 160 );
    
    /* right */
    glPushMatrix();
    glRotatef( 90.0, 0.0, 1.0, 0.0 );
    drawFace( False, 160 );
    glPopMatrix();
    
    /* left */
    glPushMatrix();
    glRotatef( -90.0, 0.0, 1.0, 0.0 );
    drawFace( False, 160 );
    glPopMatrix();
    
    /* back */
    glPushMatrix();
    glRotatef( 180.0, 0.0, 1.0, 0.0 );
    drawFace( False, 160 );
    glPopMatrix();
    
    /* top */
    glPushMatrix();
    glRotatef( -90.0, 1.0, 0.0, 0.0 );
    drawFace( False, 160 );
    glPopMatrix();
    
    /* bottom */
    glPushMatrix();
    glRotatef( 90.0, 1.0, 0.0, 0.0 );
    drawFace( False, 160);
    glPopMatrix();
}

int setupWindow(int argc, char *argv[]){
  Display *theDisplay = XOpenDisplay( ":0" );

  // Setup an Open GL window to render into - this is not a dmuObj
  window = ScratchWindow::create(theDisplay, sizeX, sizeY, 
				 &argc, argv);
  
  window->setContext();
  if (!window){
    fprintf (stderr, "Failed to create scratch window!\n");
    return (-1);
  }

  // OpenGL setup for the window
  glViewport( 0, 0, sizeX, sizeY);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,sizeX,0,sizeY,0,1000);

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();

  glDisable(GL_TEXTURE_2D);
  glDisable(GL_DEPTH_TEST);
  return (0);
}
