/*
 * Copyright 1996 Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 *
 * You may use and modify this software and its documentation for your own
 * internal purposes without fee, provided that you agree that you will not
 * remove this notice from this software under any circumstances.  You may not
 * disclose the contents of this file to third parties in any form, in whole
 * or in part, without the prior written permission of Silicon Graphics, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT ANY WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABLIITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * In no event shall Silicon Graphics be liable for any special, incidental,
 * indirect or consequential damages of any kind, or any damages whatsoever
 * resulting from loss of use, data or profits, whether or not advised of the
 * possibility of damage, and on any theory of liability, arising out of or
 * in connection with the possession, use or performance of this software.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND
 * Use, duplication, or disclosure by the Government is subject to restrictions
 * set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013 and/or in
 * similar or successor clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the United States.
 * Contractor/manufacturer is Silicon Graphics, Inc., 2011 N. Shoreline Blvd.,
 * Mountain View, CA 94039-7311.
 */


#include "glxContext.h"

glxContextC::glxContextC()
{
Set();
}

glxContextC::glxContextC(Display *display, GLXDrawable drawable,
	GLXContext glxContext)
{
ctxt = glxContext; dpy = display; drbl = drawable;
}

void glxContextC::Set()
{
ctxt = glXGetCurrentContext();
drbl = glXGetCurrentDrawable();
dpy = glXGetCurrentDisplayEXT();
if(dpy == NULL || drbl == None || ctxt == NULL)
	{
	cerr << "Warning:  Unable to establish default context." << endl;
	cerr << "          Instantiate restore after context is defined." << endl;
	}
}

void glxContextC::Current()
{
if(!(glXMakeCurrent(dpy, drbl, ctxt)))
	{
	cerr << "Makecurrent failed for: " << this << endl;
	cerr << "Using previous context." << endl;
	}
}

void glxContextC::Current(GLXDrawable newDrawable)
{
if(!(glXMakeCurrent(dpy, newDrawable, ctxt)))
	{
	cerr << "Makecurrent failed for: " << this << endl;
	cerr << "Using previous context." << endl;
	}
}

void glxContextC::Push()
{
tempCtxt = glXGetCurrentContext();
tempDrbl = glXGetCurrentDrawable();
tempDpy = glXGetCurrentDisplayEXT();
}

void glxContextC::Pop()
{
if(!(glXMakeCurrent(tempDpy, tempDrbl, tempCtxt)))
	{
	cerr << "Makecurrent failed for: " << this << endl;
	cerr << "Using previous context." << endl;
	}
}
