// vtO2.c++
//
//	Video Texturing using the fast path (dmbuffers) on O2
//
// CC -o vto2 vtO2.c++ glxContext.c++ -ldmedia -lvl -lGLw -lGL -lXt -lX11
//

#include <stdlib.h>
#include <iostream.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/keysym.h>
#include <Xm/Xm.h>
#include <dmedia/vl.h>
#include <dmedia/dm_buffer.h>
#include <dmedia/dm_image.h>
#include <GL/GLwDrawA.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <unistd.h>
#include <sys/time.h>

#include "glxContext.h"

// Global Variables
glxContextC *defCtxt, *pbufCtxt;
VLServer svr;
VLPath path;
VLNode src, drn;
XtAppContext appContext;
Display *dpy;
Window win;
int texWidth, texHeight;
int fieldWidth, fieldHeight;
int frameWidth, frameHeight;

void ExitVtO2()
{
glXWaitGL();
vlEndTransfer(svr, path); 	// End the data transfer
vlDestroyPath(svr,path);	// Destroy the path, free the memory it used 
vlCloseVideo(svr); 			// Disconnect from the daemon 
XCloseDisplay(dpy);
exit(0);
}


void GinitCB(Widget w, XtPointer, XtPointer)
{
XVisualInfo *vi;

XtVaGetValues(w, GLwNvisualInfo, &vi, NULL);
defCtxt = new glxContextC(dpy, XtWindow(w),
	glXCreateContext(dpy,vi,NULL,GL_TRUE));
defCtxt->Current();
glViewport(0, 0, frameWidth, frameHeight);
glMatrixMode(GL_PROJECTION);
glLoadIdentity();
glFrustum(-0.25 * frameWidth, 0.25 * frameWidth, -frameHeight * 0.25,
        frameHeight * 0.25, frameHeight, 3.0 * frameHeight);
glMatrixMode(GL_MODELVIEW);
glEnable(GL_TEXTURE_2D);
glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}


void InputCB(Widget w, XtPointer, XtPointer callData)
{
XmDrawingAreaCallbackStruct *dacs = (XmDrawingAreaCallbackStruct *) callData;
KeySym keySym;
char buf[32];

if(dacs->event->type == KeyPress)
	{
	XLookupString((XKeyEvent *) dacs->event, buf, 32, &keySym, NULL);
	if(keySym == XK_Escape)
		ExitVtO2();
	}
}


void CreateWindow(int argc, char *argv[])
{
Widget top, frameArea;
int attribs[] = {GLX_RGBA, GLX_RED_SIZE, 8, GLX_DOUBLEBUFFER,
   GLX_GREEN_SIZE, 8, GLX_BLUE_SIZE, 8, GLX_ALPHA_SIZE, 8, None};
char geometryStr[64];

sprintf(geometryStr, "%dx%d+140+140", frameWidth, frameHeight);
top = XtVaAppInitialize(&appContext,"vto2",NULL,0,&argc,argv,NULL,
	 XmNtitle, argv[0], XmNgeometry, geometryStr, NULL);
dpy = XtDisplay(top);
frameArea = XtVaCreateManagedWidget("frame", glwDrawingAreaWidgetClass, top,
	GLwNattribList, attribs, NULL);
XtAddCallback(frameArea,GLwNginitCallback, GinitCB, (XtPointer) NULL);
XtAddCallback(frameArea,GLwNinputCallback, InputCB, (XtPointer) NULL);
XtRealizeWidget(top);
win = XtWindow(frameArea);
}


void CreatePbuffer(GLXPbufferSGIX *dmPbuf)
{
int i, match, items, a[4];
GLXFBConfigSGIX *fbConfig;
GLXContext dmpc;
int pbAttribs[] = {GLX_DIGITAL_MEDIA_PBUFFER_SGIX, GL_TRUE, None};
int configAttribs[] = {
	GLX_RENDER_TYPE_SGIX, GLX_RGBA_BIT_SGIX,
	GLX_DRAWABLE_TYPE_SGIX, GLX_PBUFFER_BIT_SGIX,
	GLX_RED_SIZE, 8,
	GLX_GREEN_SIZE, 8,
	GLX_BLUE_SIZE, 8,
	GLX_ALPHA_SIZE, 8,
	None };
	
if(!(fbConfig = glXChooseFBConfigSGIX(defCtxt->dpy, 
	DefaultScreen(defCtxt->dpy), configAttribs, &items)))
	{ cout << "Unable to get pbuffer fbconfig." << endl; exit(0); }

for(i = 0; i < items; i++)
	{
	glXGetFBConfigAttribSGIX(dpy, fbConfig[i], GLX_RED_SIZE, &a[0]);
	glXGetFBConfigAttribSGIX(dpy, fbConfig[i], GLX_GREEN_SIZE, &a[1]);
	glXGetFBConfigAttribSGIX(dpy, fbConfig[i], GLX_BLUE_SIZE, &a[2]);
	glXGetFBConfigAttribSGIX(dpy, fbConfig[i], GLX_ALPHA_SIZE, &a[3]);
	if(a[0] == 8 && a[1] == 8 && a[2] == 8 && a[3] == 8)
		{ match = i; break; }
	}

if(!(dmpc = glXCreateContextWithConfigSGIX(defCtxt->dpy,
	fbConfig[match], GLX_RGBA_TYPE_SGIX, NULL, GL_TRUE)))
	{ cerr << "Unable to create pbuffer context." << endl; exit(0); }

if(!(*dmPbuf = glXCreateGLXPbufferSGIX(defCtxt->dpy, fbConfig[match], 
		fieldWidth, fieldHeight, pbAttribs)))
	{ cerr << "Unable to create pbuffer." << endl; return; }

pbufCtxt = new glxContextC(defCtxt->dpy, *dmPbuf, dmpc);
}


void InitTexture(int texId)
{
glBindTextureEXT(GL_TEXTURE_2D, texId);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8_EXT, texWidth, texHeight, 0, 
	GL_RGBA, GL_UNSIGNED_BYTE, NULL);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
}

int PowerOf2(int value)
{
int x = 1;
while(x <= value) x <<= 1;
return(x);
}

void SetVLPathControls()
{
VLControlValue val;

val.intVal = VL_PACKING_ABGR_8;
vlSetControl(svr, path, drn, VL_PACKING, &val);

// Set control for field rendering.  This conserves on the size of the
// defined texture and improves performance.
val.intVal = VL_CAPTURE_NONINTERLEAVED;
vlSetControl(svr, path, drn, VL_CAP_TYPE, &val);

// Query memory node for field size
vlGetControl(svr, path, drn, VL_SIZE, &val);
fieldWidth = val.xyVal.x; fieldHeight = val.xyVal.y;

// Query video node for frame size
vlGetControl(svr, path, src, VL_SIZE, &val);
frameWidth = val.xyVal.x; frameHeight = val.xyVal.y;
texWidth = PowerOf2(fieldWidth); texHeight = PowerOf2(fieldWidth);

val.intVal = VL_FORMAT_RGB;
vlSetControl(svr, path, drn, VL_FORMAT, &val);
val.intVal = VL_LAYOUT_GRAPHICS;
vlSetControl(svr, path, drn, VL_LAYOUT, &val);
}


void main(int argc, char *argv[])
{
XEvent event;
GLXPbufferSGIX dmPbuf;
int glBufCt, videoBufCt, texIndex, latitude;
GLuint texObj;

// Setup video path
svr = vlOpenVideo("");	// Open connection to video daemon
src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY); // Setup source node
drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);// Setup drain node
path = vlCreatePath(svr, VL_ANY, src, drn); 	// Setup path from src to drn

// Set up the hardware for and define the usage of the path 
vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE);
SetVLPathControls();

CreateWindow(argc, argv);
CreatePbuffer(&dmPbuf);

// Create image params to correspond to video size
DMparams *videoFormat;
dmParamsCreate(&videoFormat);
dmSetImageDefaults(videoFormat, fieldWidth, fieldHeight,
	DM_IMAGE_PACKING_RGBA);
dmParamsSetEnum(videoFormat, DM_IMAGE_LAYOUT, DM_IMAGE_LAYOUT_GRAPHICS);

// Create and initialize a dmbuffer pool for use by VL and GL.
// This is the logic that makes this code example fast path video texturing
// for O2 !
DMbufferpool pool;
DMparams *poolParams;
dmParamsCreate(&poolParams);
dmBufferSetPoolDefaults(poolParams, 0, 0, DM_FALSE, DM_FALSE);

// VL and GL are clients of the dmbuffer.  A call to vlDMPoolGetParams
// or dmBufferGetGLPoolParams *sets* the pools buffer count to
//		MAX(previous_ct, required_buffers)
//
// where previous_ct is the buffer count of the pool prior to calling
// the function and required_buffers is the number of buffers required
// by the client; VL in the vlDMPoolGetParams case and GL in the 
// dmBufferGetGLPoolParams case.
//
// When multiple clients access the pool the buffer count needs to
// be the sum of the required buffers of all the clients.

// Get VL pool constraints
vlDMPoolGetParams(svr, path, drn, poolParams);
videoBufCt = dmParamsGetInt(poolParams, DM_BUFFER_COUNT);

// Get GL pool constraints
dmParamsSetInt(poolParams, DM_BUFFER_COUNT, 0);
dmBufferGetGLPoolParams(videoFormat, poolParams);
glBufCt = dmParamsGetInt(poolParams, DM_BUFFER_COUNT);

// Add the buffer counts of GL and VL and set the pool accordingly.
// Add one buffer for our application (yet another client of the dmbuffer).
dmParamsSetInt(poolParams, DM_BUFFER_COUNT, videoBufCt + glBufCt + 1); 

dmBufferCreatePool(poolParams, &pool);
dmParamsDestroy(poolParams);

// Allocate a dmbuffer and associate with the pbuffer in order to initialize
// the OpenGL state of the pbuffer
DMbuffer dmbuf;
dmBufferAllocate(pool, &dmbuf);
glXAssociateDMPbufferSGIX(defCtxt->dpy, dmPbuf, videoFormat, dmbuf);

// Initialize pbuffer
pbufCtxt->Current();
glViewport(0, 0, frameWidth, frameHeight);
glMatrixMode(GL_PROJECTION);
glLoadIdentity();
glOrtho(0, frameWidth, 0, frameHeight, -1, 1);
glMatrixMode(GL_MODELVIEW);
glClear(GL_COLOR_BUFFER_BIT);
glRasterPos2i(0, 0);
dmBufferFree(dmbuf);
defCtxt->Current();

// Generate a texture id, bind and initialize
glGenTexturesEXT(1, &texObj);
InitTexture(texObj);
glFinish();

vlDMPoolRegister(svr, path, drn, pool);// Register dmbuffer with VL
vlBeginTransfer(svr, path, 0, NULL);	// Begin the data transfer 

float minS = 0.0;
float maxS = fieldWidth / (float) texWidth;
float minT = 0.0;
float maxT = fieldHeight / (float) texHeight;

VLEvent vle;
DMbuffer field;

while(1)
	{
	// Handle pending UI events
	while(XtAppPending(appContext))
		{ XtAppNextEvent(appContext, &event); XtDispatchEvent(&event); }
	// Handle pending video events
	if(vlEventRecv(svr, path, &vle) < 0)
		{ sginap(1); continue; }

	vlEventToDMBuffer(&vle, &field);
	glXAssociateDMPbufferSGIX(defCtxt->dpy, dmPbuf, videoFormat, field);
	glXMakeCurrentReadSGI(defCtxt->dpy, defCtxt->drbl, dmPbuf, defCtxt->ctxt);

	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -frameHeight * 2.0);
	glRotatef(-(latitude % 360), 1.0, 0.0, 0.0);	// Tumble vt mapped polygon

	// Redefine texture for each new video frame
	glBindTextureEXT(GL_TEXTURE_2D, texObj);
	glCopyTexSubImage2DEXT(GL_TEXTURE_2D, 0, 0, 0, 0, 0,
		fieldWidth, fieldHeight);

	glBegin(GL_QUADS);
	glTexCoord2f(minS, minT);
	glVertex2i(-frameWidth / 2.0, -frameHeight / 2.0);
	glTexCoord2f(maxS, minT);
	glVertex2i(frameWidth / 2.0, -frameHeight / 2.0);
	glTexCoord2f(maxS, maxT);
	glVertex2i(frameWidth / 2.0, frameHeight / 2.0);
	glTexCoord2f(minS, maxT);
	glVertex2i(-frameWidth / 2.0, frameHeight / 2.0);
	glEnd();

	// Tell VL that we are done using this dmbuffer so it can use it for
	// future frames.  This is not a release of memory it is a release of
	// a lock.
	dmBufferFree(field);
	glXSwapBuffers(dpy, win);
	latitude += 2;
	texIndex %= 2;
	}
}
