// vtSirius.c++
//
// Fast path (video source extension) example of video texturing on Sirius
// Identical logic to ../../OCTANE/vtOctane.c++.
//
// CC -o vtsirius vtSirius.c++ -lvl -lGLw -lGL -lXt -lX11

#include <stdlib.h>
#include <iostream.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/keysym.h>
#include <dmedia/vl.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <GL/GLwDrawA.h>
#include <Xm/Xm.h>
#include <unistd.h>

// Global Variables
GLuint texObj[2], texIndex = 0;
GLXVideoSourceSGIX videoSrc;
VLServer svr;
VLPath path;
VLNode src, drn;
XtAppContext appContext;
Display *dpy;
Window win;
int fieldWidth, fieldHeight;
int frameWidth, frameHeight;
int texHeight, texWidth;

void ExitVTSirius()
{
glXWaitGL();
glDeleteTexturesEXT(2, texObj);
vlEndTransfer(svr, path);  // End the data transfer
vlDestroyPath(svr,path);   // Destroy the path, free the memory it used 
vlCloseVideo(svr);         // Disconnect from the daemon 
XCloseDisplay(dpy);
exit(0);
}

void GinitCB(Widget w, XtPointer, XtPointer)
{
XVisualInfo *vi;
GLXContext gfxCtxt;

XtVaGetValues(w, GLwNvisualInfo, &vi, NULL);
gfxCtxt = glXCreateContext(dpy,vi,NULL,GL_TRUE);
glXMakeCurrent(dpy, XtWindow(w), gfxCtxt);
// Hint to OpenGL to use the fast path for video texturing
glHint(GL_TEXTURE_MULTI_BUFFER_HINT_SGIX,GL_FASTEST);
videoSrc = glXCreateGLXVideoSourceSGIX(dpy, DefaultScreen(dpy), svr,
	path, VL_TEXTURE, drn);
glXMakeCurrentReadSGI(dpy, XtWindow(w), videoSrc, gfxCtxt);
glViewport(0, 0, frameWidth, frameHeight);
glMatrixMode(GL_PROJECTION);
glLoadIdentity();
glFrustum(-0.25 * frameWidth, 0.25 * frameWidth, -frameHeight * 0.25,
	frameHeight * 0.25, frameHeight, 3.0 * frameHeight);
glMatrixMode(GL_MODELVIEW);
glEnable(GL_TEXTURE_2D);
glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

void InputCB(Widget, XtPointer, XtPointer callData)
{
XmDrawingAreaCallbackStruct *dacs = (XmDrawingAreaCallbackStruct *) callData;
KeySym keySym;
char buf[32];

if(dacs->event->type == KeyPress)
	{
	XLookupString((XKeyEvent *) dacs->event, buf, 32, &keySym, NULL);
	if(keySym == XK_Escape)
		ExitVTSirius();
	}
}

void CreateWindow(int argc, char *argv[])
{
Widget top, frameArea;
int attribs[] = {GLX_RGBA, GLX_RED_SIZE, 1, GLX_DOUBLEBUFFER,
   GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, GLX_ALPHA_SIZE, 1, None};
char geometryStr[64];

sprintf(geometryStr, "%dx%d+140+140", frameWidth, frameHeight);
top = XtVaAppInitialize(&appContext,"vtoctane",NULL,0,&argc,argv,NULL,
	 XmNtitle, argv[0], XmNgeometry, geometryStr, NULL);
dpy = XtDisplay(top);
frameArea = XtVaCreateManagedWidget("frame", glwDrawingAreaWidgetClass, top,
	GLwNattribList, attribs, NULL);
XtAddCallback(frameArea,GLwNginitCallback, GinitCB, (XtPointer) NULL);
XtAddCallback(frameArea,GLwNinputCallback, InputCB, (XtPointer) NULL);
XtRealizeWidget(top);
win = XtWindow(frameArea);
}

void CreateTexture(GLuint *texId)
{
// Initialize the textures large enough to hold a field
glGenTexturesEXT(1, texId);
glBindTextureEXT(GL_TEXTURE_2D, *texId);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 1024, 256, 0, GL_RGBA,
	GL_UNSIGNED_BYTE, NULL);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
}

int PowerOf2(int value)
{
int x = 1;
while(x <= value) x <<= 1;
return(x);
}

void main(int argc, char *argv[])
{
VLControlValue val;

svr = vlOpenVideo("");   // Open connection to video daemon
src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);   // Setup source node
drn = vlGetNode(svr, VL_DRN, VL_TEXTURE, VL_ANY); // Setup drain node
path = vlCreatePath(svr, VL_ANY, src, drn);   // Setup path from src to drn

// Set up the hardware for and define the usage of the path 
vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE);

val.intVal = VL_PACKING_ABGR_8;
vlSetControl(svr, path, drn, VL_PACKING, &val);
val.intVal = VL_CAPTURE_INTERLEAVED;
vlSetControl(svr, path, drn, VL_CAP_TYPE, &val);
vlGetControl(svr, path, drn, VL_SIZE, &val);
fieldWidth = val.xyVal.x; fieldHeight = val.xyVal.y;
vlGetControl(svr, path, src, VL_SIZE, &val);
frameWidth = val.xyVal.x; frameHeight = val.xyVal.y;

// We need to find the smallest texture with power of 2 dimensionality
// that will accomodate our field.
texWidth = PowerOf2(fieldWidth); texHeight = PowerOf2(fieldHeight);

CreateWindow(argc, argv);
vlBeginTransfer(svr, path, 0, NULL);	// Begin the data transfer 

// Create two texture objects for a double buffering scheme that will
// not drop frames.
CreateTexture(&texObj[0]);
CreateTexture(&texObj[1]);

// Establish offsets that will align right side of the video texture on
// a 32 texel boundary.  This is a requirement of Impact graphics.
int xOffset = ((fieldWidth + 0x001f) & ~0x001f) - fieldWidth;

// Assign texture coordinates to match the subtextured video frame to the
// spatial coordinates of the quad else only a subregion will be texture
// mapped with video.
float minS = xOffset / (float) texWidth;
float maxS = (xOffset + fieldWidth) / (float) texWidth;
float minT = 0.0;
float maxT = fieldHeight / (float) texHeight;

XEvent event;
GLint latitude = 0;

while(1)
	{
	// Handle pending UI events
	while(XtAppPending(appContext))
		{ XtAppNextEvent(appContext, &event); XtDispatchEvent(&event); }

	glClear(GL_COLOR_BUFFER_BIT);

	// Tumble video texture mapped polygon
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -frameHeight * 2.0);
	glRotatef(-(latitude % 360), 1.0, 0.0, 0.0);

	glBindTextureEXT(GL_TEXTURE_2D, texObj[texIndex++]);

	// Redefine texture for each new video frame
	glCopyTexSubImage2DEXT(GL_TEXTURE_2D, 0, xOffset, 0,
		0, 0, fieldWidth, fieldHeight);

	glBegin(GL_QUADS);
	glTexCoord2f(minS, maxT);
	glVertex2i(-frameWidth / 2.0, -frameHeight / 2.0);
	glTexCoord2f(maxS, maxT);
	glVertex2i(frameWidth / 2.0, -frameHeight / 2.0);
	glTexCoord2f(maxS, minT);
	glVertex2i(frameWidth / 2.0, frameHeight / 2.0);
	glTexCoord2f(minS, minT);
	glVertex2i(-frameWidth / 2.0, frameHeight / 2.0);
	glEnd();

	glXSwapBuffers(dpy, win);
	latitude += 2;
	texIndex %= 2;
	}
}
