// minVideoIn.c++
//
// This is a pared down essential logic only version of video in.
// Written as an introduction to basic VL concepts.
//
// CC -o minvideoin minVideoIn.c++ -lGLw -lGL -lvl -lXt -lX11

#include <stdio.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <GL/GLwDrawA.h>
#include <dmedia/vl.h>
#include <iostream.h>

Window win;
VLBuffer buffer;
VLServer svr;
VLPath path;
VLNode src, drn;
Display *dpy;
int width, height; // Field/Frame size

void GinitCB(Widget w, XtPointer, XtPointer)
{
XVisualInfo *vi;

XtVaGetValues(w, GLwNvisualInfo, &vi, NULL);
glXMakeCurrent(dpy, XtWindow(w), glXCreateContext(dpy,vi,NULL,GL_TRUE));
glOrtho(0, width, 0, height, -1, 1);
glRasterPos2i(0, height);
glPixelZoom(1.0, -1.0);
}

void CreateWindow(int argc, char *argv[])
{
XEvent event;
XtAppContext appContext;
Widget top, frameArea;
char geometryStr[64];
int attribs[] = {GLX_RGBA, GLX_DOUBLEBUFFER, None};

sprintf(geometryStr, "%dx%d+140+140", width, height);
top = XtVaAppInitialize(&appContext,"MinVideoIn",NULL,0,&argc,argv,NULL,
	 XtNtitle, "MinVideoIn", XtNgeometry, geometryStr, NULL);
dpy = XtDisplay(top);
frameArea = XtVaCreateManagedWidget("frame", glwDrawingAreaWidgetClass, top,
	GLwNattribList, attribs, NULL);
XtAddCallback(frameArea,GLwNginitCallback, GinitCB, (XtPointer) NULL);
XtRealizeWidget(top);
while(XtAppPending(appContext))
	{ XtAppNextEvent(appContext, &event); XtDispatchEvent(&event); }
win = XtWindow(frameArea);
}

void ProcessEvent(VLServer svr, VLEvent ev, void *data)
{
VLInfoPtr info;
GLubyte *dataPtr;

if(!(info = vlGetNextValid(svr, buffer))) return;
if(!(dataPtr = (GLubyte *) vlGetActiveRegion(svr, buffer, info)))
	return;
glDrawPixels(width, height, GL_RGBA, GL_UNSIGNED_BYTE, dataPtr);
vlPutFree(svr, buffer);
glXSwapBuffers(dpy, win);
}

void main(int argc, char *argv[])
{
VLControlValue val;

// Open a connection to the video daemon
svr = vlOpenVideo("");
dpy = XOpenDisplay(NULL);

// Define source and drain nodes for the video path
src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);
drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
path = vlCreatePath(svr, VL_ANY, src, drn); 

// Be certain to do this prior to setting any controls
vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE);

// Establish packing and format compatible with glDrawPixels
val.intVal = VL_FORMAT_RGB;
vlSetControl(svr, path, drn, VL_FORMAT, &val);
val.intVal = VL_PACKING_ABGR_8;
vlSetControl(svr, path, drn, VL_PACKING, &val);
val.intVal = VL_CAPTURE_INTERLEAVED;
vlSetControl(svr, path, drn, VL_CAP_TYPE, &val);

// Get the default frame size for window sizing and drawpixels dimensions
vlGetControl(svr, path, src, VL_SIZE, &val);
width = val.xyVal.x; height = val.xyVal.y;
CreateWindow(argc, argv);

// Create video ring buffer for frames being delivered by the server
buffer = vlCreateBuffer(svr, path, drn, 3);
vlRegisterBuffer(svr, path, drn, buffer);
vlSelectEvents(svr, path, VLTransferCompleteMask);
vlAddCallback(svr, path, VLTransferCompleteMask, 
					(VLCallbackProc) ProcessEvent, NULL);
vlBeginTransfer(svr, path, 0, NULL);
vlMainLoop();
}
