// vtSGI.c++
//
// Simple video texturing example for O2, Octane, and Onyx2 DIVO.
//
//
// CC -o vtsgi vtSGI.c++ -ldmedia -lvl -lGLw -lGL -lXt -lX11

#include <stdlib.h>
#include <iostream.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/keysym.h>
#include <GL/GLwDrawA.h>
#include <Xm/Xm.h>
#include <dmedia/vl.h>
#include <unistd.h>

// Global Variables
VLServer svr;
VLPath path;
VLNode src, drn;
Window win;
Display *dpy;
XtAppContext appContext;
int frameWidth, frameHeight;
int fieldWidth, fieldHeight;
int texHeight, texWidth;

void ExitVTSgi()
{
vlEndTransfer(svr, path); // End the data transfer
vlDestroyPath(svr,path);  // Destroy the path, free the memory it used 
vlCloseVideo(svr);        // Disconnect from the daemon 
XCloseDisplay(dpy);
exit(0);
}

void GinitCB(Widget w, XtPointer, XtPointer)
{
XVisualInfo *vi;

XtVaGetValues(w, GLwNvisualInfo, &vi, NULL);
glXMakeCurrent(dpy, XtWindow(w), glXCreateContext(dpy,vi,NULL,GL_TRUE));
glViewport(0, 0, frameWidth, frameHeight);
glMatrixMode(GL_PROJECTION);
glFrustum(-0.25 * frameWidth, 0.25 * frameWidth, -frameHeight * 0.25,
        frameHeight * 0.25, frameHeight, 3.0 * frameHeight);
glMatrixMode(GL_MODELVIEW);
glEnable(GL_TEXTURE_2D);
glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
}

void InputCB(Widget, XtPointer, XtPointer callData)
{
XmDrawingAreaCallbackStruct *dacs = (XmDrawingAreaCallbackStruct *) callData;
KeySym keySym;
char buf[32];

if(dacs->event->type == KeyPress)
	{
	XLookupString((XKeyEvent *) dacs->event, buf, 32, &keySym, NULL);
	if(keySym == XK_Escape)
		ExitVTSgi();
	}
}

void CreateWindow(int argc, char *argv[])
{
Widget top, frameArea;
int attribs[] = {GLX_RGBA, GLX_RED_SIZE, 1, GLX_DOUBLEBUFFER,
   GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, GLX_ALPHA_SIZE, 1, None};
char geometryStr[64];

sprintf(geometryStr, "%dx%d+140+140", frameWidth, frameHeight);
top = XtVaAppInitialize(&appContext,"VTSGI",NULL,0,&argc,argv,NULL,
	 XmNtitle, argv[0], XmNgeometry, geometryStr, NULL);
dpy = XtDisplay(top);
frameArea = XtVaCreateManagedWidget("frame", glwDrawingAreaWidgetClass, top,
	GLwNattribList, attribs, NULL);
XtAddCallback(frameArea,GLwNginitCallback, GinitCB, (XtPointer) NULL);
XtAddCallback(frameArea,GLwNinputCallback, InputCB, (XtPointer) NULL);
XtRealizeWidget(top);
win = XtWindow(frameArea);
}

int PowerOf2(int value)
{
int x = 1;
while(x <= value) x <<= 1;
return(x);
}

void main(int argc, char *argv[])
{
VLControlValue val;

svr = vlOpenVideo("");	// Open connection to video daemon
src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY); // Setup source node
drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);	// Setup drain node
path = vlCreatePath(svr, VL_ANY, src, drn); 	// Setup path from src to drn

// Set up the hardware for and define the usage of the path 
vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE);

val.intVal = VL_PACKING_ABGR_8;
vlSetControl(svr, path, drn, VL_PACKING, &val);
val.intVal = VL_CAPTURE_NONINTERLEAVED;       // Set to use fields by default
vlSetControl(svr, path, drn, VL_CAP_TYPE, &val);
vlGetControl(svr, path, drn, VL_SIZE, &val);  // Get field size from mem node
fieldWidth = val.xyVal.x; fieldHeight = val.xyVal.y;
vlGetControl(svr, path, src, VL_SIZE, &val);  // Get frame size from video node
frameWidth = val.xyVal.x; frameHeight = val.xyVal.y;

// We need to find the smallest texture with power of 2 dimensionality
// that will accomodate our field.
texWidth = PowerOf2(fieldWidth); texHeight = PowerOf2(fieldHeight);

// Window dimensions depend on frame size so call after this is established
CreateWindow(argc, argv);

// Place DM buffer logic after calling vlSetupPaths and before vlBeginTransfer.
DMparams *poolParams;
dmParamsCreate(&poolParams);

// Set up the dmbuffer pool to hold as many buffers as VL requires plus
// one for the application to use while texture mapping.
// The call to vlDMGetParams will set the VL appropriate DM_BUFFER_COUNT
// and DM_BUFFER_SIZE on poolParams for us.
dmBufferSetPoolDefaults(poolParams, 0, 0, DM_FALSE, DM_FALSE); 
dmParamsSetInt(poolParams, DM_BUFFER_COUNT, 0);

// Accomodate the 6.3 API for O2
#if (RELEASE == 63)
vlDMPoolGetParams(svr, path, drn, poolParams);
#else
vlDMGetParams(svr, path, drn, poolParams);
#endif

DMbufferpool pool;
int videoBufCt = dmParamsGetInt(poolParams, DM_BUFFER_COUNT);
dmParamsSetInt(poolParams, DM_BUFFER_COUNT, videoBufCt + 1);
dmBufferCreatePool(poolParams, &pool);
dmParamsDestroy(poolParams);
vlDMPoolRegister(svr, path, drn, pool);	// Register dmbuffer with VL
vlBeginTransfer(svr, path, 0, NULL);	// Begin the data transfer 

// Initialize the default texture large enough to hold a field
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8_EXT, texWidth, texHeight, 0, GL_RGBA,
	GL_UNSIGNED_BYTE, NULL);

// Establish offsets that will align right side of the video texture on
// a 32 texel boundary.  This is a requirement of Impact graphics.
int xOffset = ((fieldWidth + 0x001f) & ~0x001f) - fieldWidth;

// Assign texture coordinates to match the subtextured video frame to the
// spatial coordinates of the quad else only a subregion will be texture
// mapped with video 
float minS = xOffset / (float) texWidth;
float maxS = (xOffset + fieldWidth) / (float) texWidth;
float minT = 0.0;
float maxT = fieldHeight / (float) texHeight;

XEvent event;
DMbuffer eDmbuf;
GLubyte *pix;
GLint latitude = 0;

// Accomodate the 6.3 API for O2
#if (RELEASE == 63)
VLEvent vle;
#endif

while(1)
	{
	// Handle pending UI events
	while(XtAppPending(appContext))
		{ XtAppNextEvent(appContext, &event); XtDispatchEvent(&event); }

	// Accomodate the 6.3 API for O2
	// Handle pending video events
#if (RELEASE == 63)
	if(vlEventRecv(svr, path, &vle) < 0)
		{ sginap(1); continue; }
	vlEventToDMBuffer(&vle, &eDmbuf);
#else
	if(vlDMBufferGetValid(svr, path, drn, &eDmbuf) < 0)
		{ sginap(1); continue; }
#endif

	pix = (GLubyte *) dmBufferMapData(eDmbuf);
	glClear(GL_COLOR_BUFFER_BIT);

	// Tumble video texture mapped polygon
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -frameHeight * 2.0);  // Centered between near & far
	glRotatef(-(latitude % 360), 1.0, 0.0, 0.0); // clipping planes

	// Redefine texture for each new video frame
	glTexSubImage2DEXT(GL_TEXTURE_2D, 0, xOffset, 0,
		fieldWidth, fieldHeight, GL_RGBA, GL_UNSIGNED_BYTE, pix);

	glBegin(GL_QUADS);
	glTexCoord2f(minS, maxT);
	glVertex2i(-frameWidth / 2.0, -frameHeight / 2.0);
	glTexCoord2f(maxS, maxT);
	glVertex2i(frameWidth / 2.0, -frameHeight / 2.0);
	glTexCoord2f(maxS, minT);
	glVertex2i(frameWidth / 2.0, frameHeight / 2.0);
	glTexCoord2f(minS, minT);
	glVertex2i(-frameWidth / 2.0, frameHeight / 2.0);
	glEnd();

	// Tell VL that we are done using this dmbuffer so it can use it for
	// future frames.  This is not a release of memory it is a release of
	// a lock.
	dmBufferFree(eDmbuf);
	glXSwapBuffers(dpy, win);
	latitude += 2;
	}
}
